/*
 * WdogClient.java
 *
 * Created on 03 July 2008, 16:04
 */
package de.desy.mst.common.diag.wdogclient;

import de.desy.tine.client.TLink;
import de.desy.tine.client.TLinkCallback;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TMode;
import de.desy.tine.queryUtils.TQuery;
import de.desy.tine.structUtils.TTaggedStructure;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.border.TitledBorder;
import javax.swing.table.AbstractTableModel;
  
/**
 *
 * @author  karol
 */
public class WdogClient extends javax.swing.JFrame
{

    private int MAX_PROCESS = 32;
    private String serverPath;
    private ProcessInfo[] processInfoData;
    private TLink monitoringLink = null;
    private WdogTableModel wdogTableModel;
    /** Creates new form WdogClient */
    public WdogClient()
    {
        initComponents();       
        loadWdogSelections();
         
        processInfoData = new ProcessInfo[MAX_PROCESS];    
        for (int i = 0; i < MAX_PROCESS; i++) processInfoData[i] = new ProcessInfo();
        JTable_processList.setModel( wdogTableModel = new WdogTableModel(processInfoData));
    }

    private WdogClient(String serverName)
    {
        initComponents();
        JPanel_watchdogSelection.setVisible(false);

        processInfoData = new ProcessInfo[MAX_PROCESS];
        for (int i = 0; i < MAX_PROCESS; i++) processInfoData[i] = new ProcessInfo();
        JTable_processList.setModel( wdogTableModel = new WdogTableModel(processInfoData));
        connectToWatchdogServer(serverName);
    }

    private void attachMonitoringLink()
    {
        TDataType dout = new TDataType(processInfoData);
        String devname = serverPath + "/#0";
        
        if( monitoringLink != null ) monitoringLink.close();
        monitoringLink = new TLink(devname,  "PROCESSINF", dout, null ,TAccess.CA_READ);        
        if( monitoringLink.attach(TMode.CM_POLL, 
                new TLinkCallback() 
                {
                    public void callback(TLink l)
                    {
                        if( l.getLinkStatus() == 0)
                        {
                            int sr = JTable_processList.getSelectedRow();
                            wdogTableModel.fireTableDataChanged();                    
                            JTable_processList.getSelectionModel().setSelectionInterval(sr, sr);                                                                       
                        }
                    } 
                } 
                ,500 ) < 0 ) System.out.println("Error : link.attach " + monitoringLink.getLinkStatus());
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        JPanel_watchdogSelection = new javax.swing.JPanel();
        jLabel46 = new javax.swing.JLabel();
        JComboBox_deviceContext = new javax.swing.JComboBox();
        jLabel48 = new javax.swing.JLabel();
        JComboBox_deviceSubsystem = new javax.swing.JComboBox();
        jLabel47 = new javax.swing.JLabel();
        JComboBox_deviceServer = new javax.swing.JComboBox();
        JButton_connect = new javax.swing.JButton();
        JPanel_processes = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jScrollPane2 = new javax.swing.JScrollPane();
        JTable_processList = new javax.swing.JTable();
        jPanel2 = new javax.swing.JPanel();
        JButton_stop = new javax.swing.JButton();
        JButton_start = new javax.swing.JButton();
        JButton_autorestartOn = new javax.swing.JButton();
        JButton_autorestartOff = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        JButton_start1 = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Watchdog Client");

        JPanel_watchdogSelection.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "Watchdog selection", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 12))); // NOI18N

        jLabel46.setFont(new java.awt.Font("Tahoma", 1, 11));
        jLabel46.setText(" Device Context: ");

        JComboBox_deviceContext.setMaximumSize(new java.awt.Dimension(100, 30));

        jLabel48.setFont(new java.awt.Font("Tahoma", 1, 11));
        jLabel48.setText(" Device Subsystem: ");

        JComboBox_deviceSubsystem.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        JComboBox_deviceSubsystem.setMaximumSize(new java.awt.Dimension(100, 30));

        jLabel47.setFont(new java.awt.Font("Tahoma", 1, 11));
        jLabel47.setText(" Device Server: ");

        JComboBox_deviceServer.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        JComboBox_deviceServer.setMaximumSize(new java.awt.Dimension(100, 30));

        JButton_connect.setText("Connect");
        JButton_connect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JButton_connectActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPanel_watchdogSelectionLayout = new javax.swing.GroupLayout(JPanel_watchdogSelection);
        JPanel_watchdogSelection.setLayout(JPanel_watchdogSelectionLayout);
        JPanel_watchdogSelectionLayout.setHorizontalGroup(
            JPanel_watchdogSelectionLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPanel_watchdogSelectionLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPanel_watchdogSelectionLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(jLabel46, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(JComboBox_deviceContext, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPanel_watchdogSelectionLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(jLabel48, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(JComboBox_deviceSubsystem, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(JPanel_watchdogSelectionLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel47, javax.swing.GroupLayout.PREFERRED_SIZE, 139, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JComboBox_deviceServer, 0, 265, Short.MAX_VALUE))
                .addGap(31, 31, 31)
                .addComponent(JButton_connect)
                .addContainerGap())
        );

        JPanel_watchdogSelectionLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {JComboBox_deviceContext, JComboBox_deviceSubsystem});

        JPanel_watchdogSelectionLayout.setVerticalGroup(
            JPanel_watchdogSelectionLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPanel_watchdogSelectionLayout.createSequentialGroup()
                .addGap(11, 11, 11)
                .addGroup(JPanel_watchdogSelectionLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel48)
                    .addComponent(jLabel46)
                    .addComponent(jLabel47))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPanel_watchdogSelectionLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JComboBox_deviceContext, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JComboBox_deviceSubsystem, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JComboBox_deviceServer, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JButton_connect, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JPanel_watchdogSelectionLayout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {JComboBox_deviceContext, JComboBox_deviceServer, JComboBox_deviceSubsystem});

        JPanel_processes.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "Processes", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 12))); // NOI18N

        JTable_processList.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane2.setViewportView(JTable_processList);

        jScrollPane1.setViewportView(jScrollPane2);

        javax.swing.GroupLayout JPanel_processesLayout = new javax.swing.GroupLayout(JPanel_processes);
        JPanel_processes.setLayout(JPanel_processesLayout);
        JPanel_processesLayout.setHorizontalGroup(
            JPanel_processesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 615, Short.MAX_VALUE)
        );
        JPanel_processesLayout.setVerticalGroup(
            JPanel_processesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 162, Short.MAX_VALUE)
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "Process control", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 12))); // NOI18N

        JButton_stop.setText("Stop");
        JButton_stop.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JButton_stopActionPerformed(evt);
            }
        });

        JButton_start.setText("Start");
        JButton_start.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JButton_startActionPerformed(evt);
            }
        });

        JButton_autorestartOn.setText("Autorestart ON");
        JButton_autorestartOn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JButton_autorestartOnActionPerformed(evt);
            }
        });

        JButton_autorestartOff.setText("Autorestart OFF");
        JButton_autorestartOff.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JButton_autorestartOffActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JButton_start)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JButton_stop)
                .addGap(26, 26, 26)
                .addComponent(JButton_autorestartOn)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(JButton_autorestartOff)
                .addContainerGap(97, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JButton_start, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JButton_stop)
                    .addComponent(JButton_autorestartOn)
                    .addComponent(JButton_autorestartOff))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "PC Reboot ", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 11))); // NOI18N

        JButton_start1.setText("Reboot");
        JButton_start1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JButton_rebootActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(20, 20, 20)
                .addComponent(JButton_start1)
                .addContainerGap(21, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(JButton_start1, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(12, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPanel_processes, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(JPanel_watchdogSelection, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JPanel_processes, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jPanel1, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPanel_watchdogSelection, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void loadWdogSelections()
    {
        JComboBox_deviceContext.addActionListener(new ActionListener()
        {

            @Override
            public void actionPerformed(ActionEvent e)
            {
                fillInSubsystem();
            }
        });
        JComboBox_deviceSubsystem.addActionListener(new ActionListener()
        {

            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (JComboBox_deviceSubsystem.getSelectedIndex() < 0)
                {
                    return;
                }
                fillInDeviceServers();
            }
        });

        fillInContexts();
    }

    private void fillInContexts()
    {
        // copied from InstanBrawser.java  
        JComboBox_deviceContext.removeAllItems();
        JComboBox_deviceContext.addItem("DEFAULT");
        String[] contexts = TQuery.getContexts();
        if (contexts != null)
        {
            Arrays.sort(contexts);
            for (int i = 0; i < contexts.length; i++)
            {
                JComboBox_deviceContext.addItem(contexts[i].trim());
            }
        }
    }

    private void fillInSubsystem()
    {
        JComboBox_deviceSubsystem.removeAllItems();
        JComboBox_deviceSubsystem.addItem("ALL");
        if (JComboBox_deviceContext.getSelectedIndex() < 0)
        {
            return;
        }
        String[] subsystems = TQuery.getDeviceSubsystems(JComboBox_deviceContext.getSelectedItem().toString());
        if (subsystems != null)
        {
            Arrays.sort(subsystems);
            for (int i = 0; i < subsystems.length; i++)
            {
                if (!subsystems[i].trim().equals(""))
                {
                    JComboBox_deviceSubsystem.addItem(subsystems[i].trim());
                }
            }
        }
    }

    private void fillInDeviceServers()
    {
        if (JComboBox_deviceContext.getSelectedIndex() < 0)
        {
            return;
        }
        if (JComboBox_deviceSubsystem.getSelectedIndex() < 0)
        {
            return;
        }
        JComboBox_deviceServer.removeAllItems();
        String[] svrs = TQuery.getDeviceServers(JComboBox_deviceContext.getSelectedItem().toString(),
                JComboBox_deviceSubsystem.getSelectedItem().toString());
        if (svrs == null)
        {
            return;
        }
        Arrays.sort(svrs);
        for (int i = 0; i < svrs.length; i++)
        {
            String s = svrs[i].trim();
            if( s.toUpperCase().endsWith("_WD")) JComboBox_deviceServer.addItem(s);
        }
    }

private void JButton_connectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JButton_connectActionPerformed
      
    if (JComboBox_deviceContext.getSelectedIndex() < 0 || JComboBox_deviceServer.getSelectedIndex() < 0) return;
    connectToWatchdogServer("/" + JComboBox_deviceContext.getSelectedItem().toString().trim() + "/" + JComboBox_deviceServer.getSelectedItem().toString().trim());    
}//GEN-LAST:event_JButton_connectActionPerformed

private void connectToWatchdogServer(String tinePath)
{
    selectWdogServer(tinePath);
    for( int i =0; i < processInfoData.length; i++ )processInfoData[i].zero();
    wdogInfoRead(serverPath, processInfoData);
    buildProcessList();
    attachMonitoringLink();
    TitledBorder  b =  (TitledBorder)JPanel_processes.getBorder();
    b.setTitle("Processes of: " + tinePath);
    JPanel_processes.repaint();
}

private void JButton_startActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JButton_startActionPerformed
    
    int idx = JTable_processList.getSelectedRow();
    if( idx < 0 || idx > MAX_PROCESS-1) return;
    String alias = processInfoData[idx].alias.toString();
    int err = startRemoteProcess( serverPath, alias );
    System.out.println( "start: err = " + err );
}//GEN-LAST:event_JButton_startActionPerformed

private void JButton_stopActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JButton_stopActionPerformed
 
    int idx = JTable_processList.getSelectedRow();
    if( idx < 0 || idx > MAX_PROCESS-1) return;
    String alias = processInfoData[idx].alias.toString();
    int err = stopRemoteProcess( serverPath, alias );
    System.out.println( "stop: err = " + err );
}//GEN-LAST:event_JButton_stopActionPerformed

private void JButton_autorestartOnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JButton_autorestartOnActionPerformed
    int idx = JTable_processList.getSelectedRow();
    if( idx < 0 || idx > MAX_PROCESS-1) return;
    String exe = processInfoData[idx].exe.toString();
    int err = setAutorestart( serverPath, exe, true );
    System.out.println( "autorestart ON: err = " + err ); 
}//GEN-LAST:event_JButton_autorestartOnActionPerformed

private void JButton_autorestartOffActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JButton_autorestartOffActionPerformed
    int idx = JTable_processList.getSelectedRow();
    if( idx < 0 || idx > MAX_PROCESS-1) return;
    String exe = processInfoData[idx].exe.toString();
    int err = setAutorestart( serverPath, exe, false );
    System.out.println( "autorestart OFF: err = " + err ); 
}//GEN-LAST:event_JButton_autorestartOffActionPerformed

private void JButton_rebootActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_JButton_rebootActionPerformed
{//GEN-HEADEREND:event_JButton_rebootActionPerformed
    int result = JOptionPane.NO_OPTION;

    result = JOptionPane.showConfirmDialog( null,
                                            "Do you really want to perform remote reboot?",
                                            "Confirm",
                                            JOptionPane.YES_NO_OPTION);
    if( result == JOptionPane.YES_OPTION )
    {
        int err = rebootRemotePc(serverPath);
        System.out.println( "reboot: err = " + err );
    }

}//GEN-LAST:event_JButton_rebootActionPerformed

private int rebootRemotePc(String server)
{
     String devname = new String( server+"/#0");
     TLink lnk = new TLink(devname,"REBOOTPC", new TDataType(new int[1]), new TDataType(new int[1]),TAccess.CA_WRITE);
     return lnk.executeAndClose(1000);
}
private void buildProcessList()
{   
    wdogTableModel.fireTableDataChanged();
}

//---------------[for Igor]-----------------------------------------------------
private int wdogInfoRead( String server, ProcessInfo[] processInfoData)
{
     int error;     
     String devname = new String( server+"/#0");
        
     TLink infoLink = new TLink(devname,"PROCESSINF",new TDataType(processInfoData),null,TAccess.CA_READ);        
     if( (error = infoLink.executeAndClose(1000)) != 0 )
         return error;
     return 0;
}

private int startRemoteProcess( String server, String alias )
{    
     String devname = new String( server+"/#0");
     TLink lnk = new TLink(devname,"ALIASSTART",null, new TDataType(alias,64),TAccess.CA_WRITE);        
     return lnk.executeAndClose(1000);
}

private int stopRemoteProcess( String server, String alias )
{
     String devname = new String( server+"/#0");    
     TLink lnk = new TLink(devname,"ALIASSTOP",null, new TDataType(alias,64), TAccess.CA_WRITE);        
     return lnk.executeAndClose(1000);
}

 private int setAutorestart(String serverPath, String exe, boolean on)
 {
     String devname = new String( serverPath +"/#0");    
     exe = exe + "," + on;
     TLink lnk = new TLink(devname,"AUTORST",null, new TDataType(exe,80), TAccess.CA_WRITE);        
     return lnk.executeAndClose(1000);
 }

//------------------------------------------------------------------------------

void selectWdogServer(String serverPath)
{
    System.out.println("Selecting: " + serverPath);
    this.serverPath = serverPath;
}
    /**
    * @param args the command line arguments
    */
    public static void main(final String args[])
    {
        try
        {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());          
        }
        catch( Exception e)
        {
            throw new RuntimeException(e);
        }
        java.awt.EventQueue.invokeLater(new Runnable()
        {
            public void run() 
            {
                if( args.length > 0 ) new WdogClient(args[0]).setVisible(true);
                else new WdogClient().setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    public javax.swing.JButton JButton_autorestartOff;
    public javax.swing.JButton JButton_autorestartOn;
    public javax.swing.JButton JButton_connect;
    public javax.swing.JButton JButton_start;
    public javax.swing.JButton JButton_start1;
    public javax.swing.JButton JButton_stop;
    public javax.swing.JComboBox JComboBox_deviceContext;
    public javax.swing.JComboBox JComboBox_deviceServer;
    public javax.swing.JComboBox JComboBox_deviceSubsystem;
    public javax.swing.JPanel JPanel_processes;
    public javax.swing.JPanel JPanel_watchdogSelection;
    public javax.swing.JTable JTable_processList;
    public javax.swing.JLabel jLabel46;
    public javax.swing.JLabel jLabel47;
    public javax.swing.JLabel jLabel48;
    public javax.swing.JPanel jPanel1;
    public javax.swing.JPanel jPanel2;
    public javax.swing.JScrollPane jScrollPane1;
    public javax.swing.JScrollPane jScrollPane2;
    // End of variables declaration//GEN-END:variables

private class WdogTableModel extends AbstractTableModel
{
        ProcessInfo[] processInfo;
        String[] columnHeader = { "PROCESS ALIAS", "AUTORESTART", "STATUS", "STATUS CHANGE" };
        public WdogTableModel( ProcessInfo[] processInfo )
        {
            this.processInfo = processInfo;
        }
        
        public int getRowCount()
        {            
            for( int i = 0; i < processInfo.length; i++ ) 
            {       
                if( processInfo[i].alias.body[0] == '\0') return i;
            }
            return processInfo.length;
        }

        public int getColumnCount()
        {
            return 4;
        }

        public Object getValueAt(int rowIndex, int columnIndex)
        {
            switch( columnIndex )
            {
                case 0: return processInfo[rowIndex].alias.toString();            
                case 1: return processInfo[rowIndex].autoRestart[0] > 0 ? "YES" : "NO";
                case 2: return processInfo[rowIndex].running[0] > 0 ? "Started" : "Stopped";
                case 3:
                        if( processInfo[rowIndex].running[0] == 0 ) return getTimeStamp( processInfo[rowIndex].stopped[0] );
                        else
                        {
                            if(processInfo[rowIndex].attached[0] > 0) return getTimeStamp( processInfo[rowIndex].attached[0] );
                            else return getTimeStamp( processInfo[rowIndex].started[0] );
                        }
            }            
            return "";
        }
        
        @Override
        public String getColumnName( int colIdx )
        {
            return columnHeader[colIdx];
        }
    
}

private class ProcessInfo extends TTaggedStructure
{
    public FixedString alias = new FixedString(64);
    public FixedString dependency = new FixedString(64);
    public FixedString exe = new FixedString(128);
    public byte[]  autoRestart = new byte[1];
    public byte[]  checkHeartBeat = new byte[1];
    public byte[]  running = new byte[1];
    public byte[]  reserved = new byte[1];
    public int[]   started = new int[1];
    public int[]   stopped = new int[1];
    public int[]   attached = new int[1];
    
    ProcessInfo()
    {
        super("PROCINFO");
        
        addField( alias.getBytes(), "alias");
        addField( dependency.getBytes(), "dependency");
        addField( exe.getBytes(), "exe");
        addField( autoRestart, "autoRestart");
        addField( checkHeartBeat, "checkHeartBeat");
        addField( running, "running");
        addField( reserved, "reserved");
        addField( started, "started");
        addField( stopped, "stopped");
        addField( attached, "attached");
        initDone();      
    }
    
    public void zero()
    {
     alias.zero();
     dependency.zero();
     exe.zero();
     autoRestart[0] = 0;
     checkHeartBeat[0] = 0;
     running[0] = 0;
     started[0] = 0;
     stopped[0] = 0;
     attached[0] =0;
    }
    
    void dump()
    {
        System.out.println( "exe name : " + exe );
        System.out.println( "PROCINFO dump:");
        System.out.println( "alias : " + alias );
        System.out.println( "dependent on : " + dependency );        
        System.out.println( "running : " + (running[0] != 0 ? "YES" : "NO"));
        System.out.println( "auto-restart : " + (autoRestart[0] != 0 ? "YES" : "NO"));
        System.out.println( "heartbeat check : " + (checkHeartBeat[0] != 0 ? "YES" : "NO"));
        System.out.println( "started : " + (started[0] == 0 ? "NEVER" :  getTimeStamp(started[0])) );
        System.out.println( "started : " + (stopped[0] == 0 ? "NEVER" :  getTimeStamp(started[0])) );
        System.out.println( "started : " + (attached[0] == 0 ? "NEVER" :  getTimeStamp(started[0])) ); 
        System.out.println("---");
    }
 }
  
private class ProcessListItem extends Component
{
        ProcessInfo processInfo;
        public ProcessListItem(ProcessInfo processInfo)
        {
            this.processInfo = processInfo;
        }    
        
        @Override
        public String toString()
        {
            return processInfo.alias.toString();
        }                
}

static public String getTimeStamp(int seconds)
{
    if( seconds == 0 ) return "";

    Calendar c = Calendar.getInstance();
    c.setTime(new Date((long)seconds*1000));
    String s = new String();
    s = String.format( "%02d", c.get(Calendar.DAY_OF_MONTH)) + "."
            + String.format( "%02d", c.get(Calendar.MONTH)+1) + "."
            + String.format( "%04d", c.get(Calendar.YEAR) )+ ", "
            + String.format( "%02d", c.get(Calendar.HOUR_OF_DAY)) + ":"
            + String.format( "%02d", c.get(Calendar.MINUTE)) +":"
            + String.format( "%02d", c.get(Calendar.SECOND));                      
    return s;
} 
}
