package de.desy.mst.common.ser.jddddisplayer;

// Java imports
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.Toolkit;	
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.URL;
import java.util.Arrays;
import java.util.List;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Properties;

import javax.imageio.ImageIO;
import javax.script.ScriptEngineManager;
import javax.swing.*;

//MstApp imports
import de.desy.mst.libs.framework.mstapp.IOnExit;
import de.desy.mst.libs.framework.mstapp.Main;
import de.desy.mst.libs.framework.mstapp.MstFrameMain;
import de.desy.mst.libs.framework.mstapp.MstApp;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.mode.IOpModeObserver;
import de.desy.mst.libs.framework.mstapp.mode.Maschine;
import de.desy.mst.libs.framework.mstapp.mode.OpMode;
import de.desy.mst.libs.framework.mstapp.mode.OpModeListe;
import de.desy.mst.libs.framework.mstapp.mode.ServerAnwahlListe;
//import de.desy.mst.libs.framework.mstapp.util.applicationresizemanager.managers.ApplicationResizeManager;

//DOOCS imports
import editor.Console;
//import editor.StartParameters;
import editor.jddd;
import runtime.jddd_run;
import ttf.doocs.clnt.EqCall;

/**
 * jddd Displayer; embeds a jddd-based display within a MstApp frame
 * @author R.Bacher, E. Sombrowski
 * @version 2.3.1
 */
public class JdddDisplayer extends MstFrameMain implements IOnExit, IOpModeObserver {

	// *********************************************************
	// Achtung: Client Initialisierungsdaten, bei Bedarf aendern!
//	private double xScale = 1.0;
//	private double yScale = 1.0;
	private String version = "2.3.1";
	private static String file = "";
    private static String title = "";
    private static String repository = "";
    private static String base = "///";
    private static String iconFile = "/de/desy/mst/common/ser/jddddisplayer/JdddDisplayer.png";
    private static URL icon = null;
    private static boolean showConsole = false;
	private static String captureFileName = "";
	private static int socketId = -1;
	// *********************************************************

	private static final long serialVersionUID = 1L;
	private static final String thisClassName = JdddDisplayer.class.getName();
	private static final ServerAnwahlListe implementierteServerAnwahlen = ServerAnwahlListe.D_;
	private static final OpModeListe implementierteOpModes = OpModeListe.BBE;
	private OpMode currentOpMode = OpMode.BETRIEB;

	private static boolean captureEnabled = false;
	private SocketListener socketListener = null;
	
	/**
	 * Default Constructor
	 */
	public JdddDisplayer () {
		
		super();
	}
	
	/**
	 * Constructor
	 * @param mstApp MstApp instance attached
	 */
	public JdddDisplayer(MstApp mstApp) {
		
		super(mstApp);
		this.initialize();
	}
	
	/**
	 * Main method of jdddDisplayer
	 * @param arguments 
	 * arg0: title of application displayed
	 * arg1: repository server url 
	 * arg2: xml-file to be loaded from jddd repository
	 * arg3: xml-file DOOCS base address 
	 * arg4: show/hide console flag 
	 * arg5: name / location of captured image file (will be saved as .png-file)
	 * arg6: socketId 
	 */
	public static void main(String[] arguments) {
		
		title = arguments[0];
		Logger.SWApp.info("   Title: " + title);
		repository = arguments[1];
		Logger.SWApp.info("   Repository: " + repository);
		file = arguments[2];
		Logger.SWApp.info("   File: " + file);
		title = title + " (" + file.split("/")[file.split("/").length - 1] + ")";
		base = arguments[3];
		Logger.SWApp.info("   Base address: " + base);
		try {
		    if (arguments[4].equals("showConsole")) showConsole = true;
		    else if (arguments[4].equals("hideConsole")) showConsole = false;
		    else showConsole = false;
		    Logger.SWApp.info("   jddd Console: " + String.valueOf(showConsole));
			captureFileName = arguments[5];
			socketId = Integer.valueOf(arguments[6]);
			if (socketId > 0) {
				if (captureFileName.equals("")) captureFileName = "JdddScreenshot";
				if (captureFileName.indexOf(".") >= 0) captureFileName = captureFileName.substring(0, captureFileName.indexOf("."));
				captureEnabled = true;
				Logger.SWApp.info("   Listen to localhost / port: " + socketId);
				Logger.SWApp.info("   Capture file: " + captureFileName);
			    Logger.SWApp.info("   Capturing enabled!");
			}
			else {
				captureEnabled = true;
			    Logger.SWApp.info("Capturing disabled!");
			}
		} 
		catch (Exception e) {
			captureEnabled = false;
		}
		
//		file = repository + file;
		
		String machine = "NA";	
		if (file.contains("PETRA")) machine = "Pe";
		else if (file.contains("REGAE")) machine = "Rg";
		else if (file.contains("DESY")) machine = "D2";
		else if (file.contains("LINAC")) machine = "L2";
		else {
			if (arguments[4].contains("petra")) machine = "Pe";
			else if (arguments[4].contains("regae")) machine = "Rg";
			else machine = "NA";
		}
		Main.startMSTApp(thisClassName, title, machine, arguments);
	}
	
	/**
	 * Initializes this
	 */
	private void initialize() {
		
		try {
	        Logger.SWApp.info("initializing application");
        	this.setTitle(title + " [jDDD Displayer Version " + version + "]");  		
    		this.setSize(870, 730);
        	List<Maschine> erlaubteMaschinen = Arrays.asList(new Maschine[]{Maschine.Pe, Maschine.D2, Maschine.L2, Maschine.Rg});		
        	if (file.contains("PETRA")) {this.getMstApp().getMaschineMgr().setErlaubteMaschinenUndMaschine(Maschine.Pe, erlaubteMaschinen);}
    		if (file.contains("DESY")) {this.getMstApp().getMaschineMgr().setErlaubteMaschinenUndMaschine(Maschine.D2, erlaubteMaschinen);}
    		if (file.contains("LINAC")) {this.getMstApp().getMaschineMgr().setErlaubteMaschinenUndMaschine(Maschine.L2, erlaubteMaschinen);}
    		if (file.contains("REGAE")) {this.getMstApp().getMaschineMgr().setErlaubteMaschinenUndMaschine(Maschine.Rg, erlaubteMaschinen);}
        	this.setIconImage(Toolkit.getDefaultToolkit().getImage(getClass().getResource(iconFile)));
        	this.fireInitializationZuProzentFertig(0);
        	this.getMstApp().setImplServerAnwahlen(implementierteServerAnwahlen);
        	this.getMstApp().setImplOpModes(implementierteOpModes);
        	Main.registerForOnExit(this);
        	this.fireInitializationZuProzentFertig(100);
		}
		catch (java.lang.Throwable e) {
			Logger.SWApp.info("initializing application failed: " + e.toString());
		}
    	
//		ApplicationResizeManager applicationResizeManager = new ApplicationResizeManager(this);
//		applicationResizeManager.setReferenceSize(xScale, yScale);	
//		applicationResizeManager.setResizeModeToZoomMaxOnly();
	}
	
	/**
	 * Does appropriate actions after operation mode changes.
	 * @param mode operation mode
	 */
	public void onOpModeChange(OpMode mode) {
		
		currentOpMode = mode;
	}	
	
	/**
	 * This method is called after frame creation.
	 * Instantiates all necessary application specific classes.
	 */
	protected void afterFrameCreated() {
		
		super.afterFrameCreated();
		
		startJddd();
		
		if (captureEnabled == true) {
			socketListener = new SocketListener(this);
		}
	}
	
	/**
	 * This method is called on exit.
	 */
	public void onExit(){

    	if (socketListener != null) socketListener.stopListener();
		setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        Logger.SWApp.info("exiting application");
        try {}
        catch(Throwable e){
        	Logger.SWApp.error("exiting application failed: " + e.toString());
        }
	}
	
	/**
	 * This method constructs the jddd instance (implementation by E.Sombrowski)
	 */
	private void startJddd() {
		String host = "ldap://ldapenssvr2.desy.de:ldap://ldapenssvr1.desy.de:ldap://ldapenssvr3.desy.de:enssvr1:enssvr2:enssvr3";
		System.setProperty("ENSHOST", host);
      
//        StartParameters startParams = StartParameters.getInstance();
//        Console.getInstance().setVisible(startParams.getShowConsole());
        Console.getInstance().setVisible(showConsole);
        readJdddVersion();
        jddd.setLookAndFeel();
        jddd.setUncaughtExceptionHandler();
        jddd.registerForUpdateNotification();
        checkJavaVersion();
        ScriptEngineManager mgr = new ScriptEngineManager();
        jddd.jsEngine = mgr.getEngineByName("JavaScript");
        
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        jddd_run.startPanel(file, base, repository, this);
	}

	/**
	 * This method reads the actual jddd version (implementation by E.Sombrowski)
	 */
    public static void readJdddVersion() {
        Properties properties = new Properties();
        try {
            properties.load(jddd.class.getResourceAsStream("jdddVersion.properties"));
            jddd.versionString = properties.getProperty("version");
            jddd.versionDate = properties.getProperty("date");
            System.out.println("jdddEditor Version " + jddd.versionString + "/" + EqCall.versionString());
            System.out.println("JAVA Version " + System.getProperty("java.version"));
        } 
        catch (IOException e) {}
    }
    
	/**
	 * This method checks the actual Java VM version (implementation by E.Sombrowski)
	 */
    public static void checkJavaVersion() {
        String javaVersion = System.getProperty("java.version");
        if (javaVersion.compareTo("1.6.0_10") < 0) {
            JOptionPane.showMessageDialog(null, "<html>You are running Java version "+javaVersion+
                    "<br>Please update your Virtual Machine to a newer version"+
                    "<br>to benefit from the improved Java performance.<br>"+
                    "<br><a href=\"http://java.sun.com/javase/downloads/index.jsp\">http://java.sun.com/javase/downloads/index.jsp</a><br></html>");
        }
    }
	
	/**
	 * SocketListener: Listens to a socket to receive commands to capture the screen
	 */
	class SocketListener implements Runnable {
		
		private Thread listener = null;
		private JdddDisplayer jdddDisplayer = null;
		private boolean listenerStarted = false;
		
		/**
		 * Constructor: Instantiates and starts the listener thread
		 * @param displayer BrowserDisplayer instance using this listener
		 */
		public SocketListener(JdddDisplayer displayer) {
			
			jdddDisplayer = displayer;
			
			listener = new Thread(this);
			listenerStarted = true;
			listener.start();
		}

		/**
		 * Listens all the time to a socket to receive commands to capture the screen
		 */
		public void run() {
			
			if (captureEnabled == true) {

				ServerSocket socket = null;
				Socket client = null;
				BufferedReader reader = null;
				
				try {
					socket = new ServerSocket(socketId);
					Logger.SWApp.info("Connected to localhost / port " + socketId);
			
					while(listenerStarted == true) {
						
						client = socket.accept();
//						System.out.println("Client connection accepted at localhost / port " + socketId);
						
						reader = new BufferedReader(new InputStreamReader(client.getInputStream()));
						String action = reader.readLine();
						if (action == null) action = "null";
						
						if (action.equals("setVisible")) {                                             // shows the window
							jdddDisplayer.setVisible(true);
							jdddDisplayer.setExtendedState(JFrame.MAXIMIZED_BOTH);
							jdddDisplayer.setAlwaysOnTop(true);
							jdddDisplayer.toFront();
						}
						
						if (action.equals("doCapture") && jdddDisplayer.isVisible()) {              // captures the window
							Robot robot = new Robot();
							int x = jdddDisplayer.getX() + jdddDisplayer.getComponent(0).getX();
							int y = jdddDisplayer.getY() + jdddDisplayer.getComponent(0).getY() + jdddDisplayer.jJMenuBar.getHeight();
							int captureWidth = jdddDisplayer.getContentPane().getComponent(2).getWidth();
							int captureHeight = jdddDisplayer.getContentPane().getComponent(2).getHeight();
							
							BufferedImage image = robot.createScreenCapture(new Rectangle(x, y, captureWidth, captureHeight));					
							if (image != null) {
								float xScale1280 = 1280.0f / captureWidth;
								float yScale1280 = 1024.0f / captureHeight;
								BufferedImage img1280 = new BufferedImage((int)(captureWidth * xScale1280), (int)(captureHeight * yScale1280), BufferedImage.TYPE_INT_RGB);
								AffineTransform at1280 = new AffineTransform();
								at1280.scale(xScale1280, yScale1280);
								AffineTransformOp scaleOp1280 = new AffineTransformOp(at1280, AffineTransformOp.TYPE_BICUBIC);
								img1280 = scaleOp1280.filter(image, img1280);
								File captureFile1280 = new File(captureFileName + "_1280.png");
								ImageIO.write(img1280, "png", captureFile1280);			
//								System.out.println("Captured at " + System.currentTimeMillis() + ", Saved at " + captureFile1280.getAbsolutePath());
			
								float xScale800 = 800.0f / captureWidth;
								float yScale800 = 600.0f / captureHeight;
								BufferedImage img800 = new BufferedImage((int)(captureWidth * xScale800), (int)(captureHeight * yScale800), BufferedImage.TYPE_INT_RGB);
								AffineTransform at800 = new AffineTransform();
								at800.scale(xScale800, yScale800);
								AffineTransformOp scaleOp800 = new AffineTransformOp(at800, AffineTransformOp.TYPE_BICUBIC);
								img800 = scaleOp800.filter(image, img800);
								File captureFile800 = new File(captureFileName + "_800.png");
								ImageIO.write(img800, "png", captureFile800);			
//								System.out.println("Captured at " + System.currentTimeMillis() + ", Saved at " + captureFile800.getAbsolutePath());
								
								float xScale400 = 400.0f / captureWidth;
								float yScale400 = 320.0f / captureHeight;
								BufferedImage img400 = new BufferedImage((int)(captureWidth * xScale400), (int)(captureHeight * yScale400), BufferedImage.TYPE_INT_RGB);
								AffineTransform at400 = new AffineTransform();
								at400.scale(xScale400, yScale400);
								AffineTransformOp scaleOp400 = new AffineTransformOp(at400, AffineTransformOp.TYPE_BICUBIC);
								img400 = scaleOp400.filter(image, img400);
								File captureFile400 = new File(captureFileName + "_400.png");
								ImageIO.write(img400, "png", captureFile400);			
//								System.out.println("Captured at " + System.currentTimeMillis() + ", Saved at " + captureFile400.getAbsolutePath());
							}	
						}
						
						if (action.equals("setInvisible")) {										 // hides the window
							jdddDisplayer.setExtendedState(JFrame.NORMAL);
							jdddDisplayer.setAlwaysOnTop(false);
							jdddDisplayer.toBack();
							jdddDisplayer.setVisible(false);
						}
						
						reader.close();
						client.close();
					}
				} 
				catch (Exception e)  {
					try {
						if (reader != null) reader.close();
						if (client != null) client.close();
						if (socket != null) socket.close();
					}
					catch (IOException ex) {}
					jdddDisplayer.setAlwaysOnTop(false);
					jdddDisplayer.toBack();
					jdddDisplayer.setVisible(false);	
					Logger.SWApp.error("Error performing image capture");
					System.exit(0);
				}
			}	
		}
		
		/**
		 * Terminates the listener thread
		 */
		public void stopListener() {
			
			listenerStarted = false;		
		}
	}	
}