/**
 * File: GlobalCounter.java
 * Date: Feb 21, 2013
 */
package de.desy.mst.libs.accelerator;

/**
 * Global counter class which includes indirect increments the internal counter.
 * If the counter is bigger then the before set value the method of {@link #isFinished()}
 * returns {@code true} otherwise {@code false}. To increment the counter you
 * should call {@link #increment()}, which also returns when the counter exceeds 
 * its limit.
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: CounterGlobal.java 53955 2013-02-25 15:47:32Z walla $
 */
public final class CounterGlobal {
	// --- CLASS FIELDS --------------------------------------------------------
	private static final CounterInstance instance = new CounterInstance(CounterInstance.DEFAULT_LIMIT);

	// --- CLASS METHODS -------------------------------------------------------
	/**
	 * Resets the counter to 1 and the limit to 3. 
	 * @see #reset(int)
	 */
	public static final void reset() { CounterGlobal.instance.reset(); }
	
	/**
	 * Increments the counter. The counter is not incremented if the counter
	 * exceeds its limit.
	 * @return Returns if the counter exceeds limit a {@code true} otherwise {@code false}.
	 */
	public static final boolean increment() { return CounterGlobal.instance.increment(); }

	/**
	 * Returns {@code true} if the counter exceeds its limit otherwise
	 * {@code false}. The counter is not manipulated by this method.
	 * @return Depending on the relationship between the limit and counter the
	 * 		corresponding boolean value is returned. 
	 */
	public static final boolean isFinished() { return CounterGlobal.instance.isFinished(); }
	
	/**
	 * Delivers if this counter has any effects on the {@link #increment()} method
	 * or not or, in other words: if this instance is a real counter or a dummy.   
	 * @return Returns the effectiveness of {@link #increment()}
	 */
	public static final boolean isCountable() { return CounterGlobal.instance.isCountable(); }
}
/* end of file */