/*
 *
 * File: EGraphicComponent.java
 * Date: Mar 26, 2007; 3:05:49 PM
 */
package de.desy.mst.libs.accelerator;

import de.desy.mst.libs.accelerator.exception.EnumerationNotUsedException;

/**
 * <p>These is the enumeration of all possible containers which a displayed swing
 * frame can contain. It is returned by the method of 
 * {@link de.desy.mst.libs.accelerator.view.IViewComponent#fetchGraphicComponent()}.</p>
 * 
 * <p>Here is a list of all components:</p>
 * <table cellpadding="4" cellspacing="4">
 * <tr><th>EGraphicComponent instance</th><th>Name</th></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #LABEL_FIXED}</td><td>An ordinary graphical label component.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #BUTTON_FIXED}</td><td>An ordinary graphical button component.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #SLIDER_FIXED}</td><td>An ordinary graphical slider component.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #CHECKBOX_FIXED}</td><td>An ordinary graphical check box component.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #LIST_FIXED}</td><td>An ordinary graphical list component.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #LIST_MODEL}</td><td>An ordinary graphical list model, which is <b>NOT</b> a component.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #SCROLLPANE_FIXED}</td><td>A complex graphical scroll pane component.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #TABBEDPANE_FIXED}</td><td>A complex graphical tab pane component.</td></tr>
 * <tr bgcolor="#A0A0E0"><td>{@link #PANEL_FIXED}</td><td>A complex graphical panel component.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #PANEL}</td><td>A complex graphical panel component.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #TEXT_AREA}</td><td>A complex graphical text area component.</td></tr>
 * </table>
 *  
 * <p>All of these components are currently returned by the java SWING library. 
 * But they can also be returned by the SWI or anything else.</p>
 * 
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: EGraphicComponent.java 63206 2014-11-21 14:18:43Z walla $
 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchGraphicComponent()
 */
public enum EGraphicComponent implements IEEnum 
{
	LABEL_FIXED("This is the component label which is fixed in its size."),
	BUTTON_FIXED("This is the component button which is fixed in its size."),
	SLIDER_FIXED("This is the component slider which is fixed in its size."),
	CHECKBOX_FIXED("This is the component check box which is fixed in its size."),
	RADIOBUTTON_FIXED("This is the component radio button which is fixed in its size."),
	LIST_FIXED("This is the component list which is fixed in its size."),
	SCROLLPANE_FIXED("This is the component scroll pane which is fixed in its size."),
	TABBEDPANE_FIXED("This is the component tab pane which is fixed in its size."),
	PANEL_FIXED("This is the component panel which is fixed in its size."),
	PANEL("This is the component of a panel which is not fixed in its size."),
	TEXT_AREA("This is the component of a text area which is not fixed in its size."),
	SPINNER("This is the component of a spinner which is not fixed in its size."),
	WHEELSWITCH_FIXED("This is the component wheel switch which is fixed in its size."),
	KNOB_FIXED("This is the component knob which is fixed in its size.")
	;
	
	/** The description string. */
	private String _description;
	
	/**
	 * This will construct each enumeration item.
	 * 
	 * @param description_ Is the description of the enumeration element.
	 */
	private EGraphicComponent(String description_)
	{
		validate(description_);
		_description = description_;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.IEEnum#getName(de.desy.mst.libs.accelerator.EName)
	 */
	public String getName(final EName description_)
		throws EnumerationNotUsedException
	{
		switch(description_)
		{
		case DESCRIPTION:
			return _description;
		default:
			throw new EnumerationNotUsedException("The enumeration indeed" +
					" exists but is not used: " + description_.toString());
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.IEEnum#putName(de.desy.mst.libs.accelerator.EName, java.lang.String)
	 */
	public void putName(final EName description_, final String name_)
	{
		throw new EnumerationNotUsedException("No enumeration is used in this" +
				" enumeration.");
	}
	
	/**
	 * Validates all variables.
	 * 
	 * @param description_ Is the description of the enumeration element.
	 */
	private void validate(String description_)
	{
		if (description_ == null)
			throw new NullPointerException("The description can not be null.");
	}
}
/* end of file */