/*
 *
 * File: IDeviceSubject.java
 * Date: Oct 5, 2006; 1:54:29 PM
 */
package de.desy.mst.libs.accelerator;

import java.awt.Component;

/**
 * This is the subject interface class of the observer design pattern. This is
 * the control class of this pattern which shows a connection between the subject
 * and its data instances, the observer objects.
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: IDeviceSubject.java 19706 2008-12-14 12:41:25Z walla $
 */
public interface IDeviceSubject
{
	/**
	 * <p>This method registers an observer. It will get the device view which is
	 * the view pattern of MVC.</p>
	 * <p>It is nearly the same as the TINE device name of a device context and
	 * server except that it will display its contents on the screen. The
	 * device view is the same as an observer object except that it can not be
	 * null.</p>
	 *
	 * @param deviceView_ Is the device view of the corresponding observer object.
	 */
	public abstract void register(IEViewDevice deviceView_);

	/**
	 * Fetches the device view and returns the observer. The observer can also be
	 * null.
	 *
	 * @param deviceView_ The enumeration of the device view.
	 * @return The observer. If a device view has not a corresponding observer
	 * 		then there also can be respond with a null value.
	 */
	public abstract IObserver fetch(IEViewDevice deviceView_);

	/**
	 * Fetches the device view and returns the component. There can also be a null
	 * returned by this method.
	 *
	 * @param deviceView_ The enumeration of the device view.
	 * @return The component. If a device view has not a corresponding component
	 * 		then there also can be respond with a null value.
	 */
	public abstract Component fetchContainer(IEViewDevice deviceView_);

	/**
	 * This will remove the corresponding observer by calling it with its
	 * device view.
	 *
	 * @param deviceView_ Is the device view of the observer.
	 */
	public abstract void remove(IEViewDevice deviceView_);

	/**
	 * This method will notify all observers that something has changed and
	 * that they should be updated.
	 *
	 * @param obj_ Is the new object you wanted to update.
	 */
	public abstract void notifyObservers(Object obj_);
}
/* end of file */