/*
 *
 * File: IEDataFactor.java
 * Date: Jan 28, 2008; 2:19:58 PM
 */
package de.desy.mst.libs.accelerator;

/**
 * This interface allows to get from a data value its send and receive factor
 * of a TINE property.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: IEDataFactor.java 12066 2008-05-29 08:35:57Z walla $
 */
public interface IEDataFactor
{
	
	/**
	 * Returns the send conversion factor value as a {@code float}. This is
	 * the simplified value for transforming the value from the client to the 
	 * server.
	 * 
	 * @return The send conversion factor is returned as a {@code float} value.
	 */
	public abstract float sendFactor();

	/**
	 * Returns the receive conversion factor value as a {@code float}. This is
	 * the simplified value for transforming the value from the server to the 
	 * client.
	 * 
	 * @return The receive conversion factor is returned as a {@code float} 
	 * 		value.
	 */
	public abstract float recvFactor();
}
/* end of file */