/*
 *
 * File: IEDataMetaDataType.java
 * Date: Mar 2, 2009; 12:20:41 PM
 */
package de.desy.mst.libs.accelerator;

import de.desy.tine.types.TCompoundDataObject;

/**
 * <p>The enumeration interface of {@code IEDataMetaDataType} is part of the 
 * {@link IEData} interface, which returns a list of inherited 
 * {@code TCompoundDataObject} instances which should first be send to the 
 * server before reading the data. The method for these values is called 
 * {@link IEDataMetaDataType#getMetaValues()}.</p> 
 * 
 * <p>This enumerations is mostly used for Petra3 data elements, which needs to 
 * send a constants field to the server before returning the corresponding 
 * values.</p>
 * <p>The following one gives a short example of what to do. The TINE property of 
 * {@code FehlerAktuell} from the TINE address {@code /PETRA/PERF.ML/Read/...} 
 * will write the constant string field of the corresponding sever, for example 
 * {@code PE_SR_Cy1}, before it will deliver all error messages via TINE with
 * the data type from the base class of {@code TTaggedStructure} to the 
 * calling application.</p>
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: IEDataMetaDataType.java 34671 2010-03-30 10:26:05Z walla $
 */
public interface IEDataMetaDataType<T extends TCompoundDataObject>
{
	/**
	 * <p>This method returns the send meta values as a list of 
 	 * {@code TCompoundDataObject} ones. This is used to send 
	 * at first values to the server before receiving the expected values.</p>
	 * <p>The send values are defined in the enumeration instances in the 
	 * concrete definition and so fore there is no needed also to 
	 * set the value at a latter time (for example 
	 * {@code setMetaValues(TCompoundDataObject[]):void}).</p>  
	 *  
	 * @return Returns all pre send meta values which inherits the interface
	 * 		of {@code TCompoundDataObject}'s.
	 */
	public abstract T[] getMetaValues();
}
/* end of file */