/*
 *
 * File: TineUserException.java
 * Date: Mar 10, 2008; 3:51:41 PM
 */
package de.desy.mst.libs.accelerator.exception;


import de.desy.mst.auxiliary.devl.deviceserverlib.exceptions.UserException;
import de.desy.mst.libs.accelerator.model.EErrorExt;
import de.desy.tine.definitions.TErrorList;

/**
 * This exception class will report TINE error extension for the device server
 * of Josef.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: TineUserException.java 47183 2011-11-29 13:11:36Z walla $
 */
public class TineUserException extends UserException {
	// --- CLASS FIELDS --------------------------------------------------------
	private static final long serialVersionUID = 4194823598513591954l;
	
	// --- CONSTRUCTORS --------------------------------------------------------
	/**
	 * A TINE error code exception is created with the error extension
	 * enumeration. 
	 * @param error_ Is the error extension enumeration.
	 */
	public TineUserException(final EErrorExt error_) {
		super(error_.getErrorCode(), error_.getError());
	}

	/**
	 * A TINE error code exception is created with the error extension
	 * enumeration. 
	 * @param error_ Is the error extension enumeration.
	 * @param message_ Is a detailed explanation of the error.
	 */
	public TineUserException(final EErrorExt error_, final String message_) {
		super(error_.getErrorCode(), error_.getError() + ((message_ == null || message_.isEmpty())?(""):(": " + message_)));
	}

	/**
	 * A TINE error code exception is created with the error extension
	 * enumeration. 
	 * @param error_ Is the error extension enumeration.
	 * @param value_ Is the subjected error number.
	 */
	public TineUserException(final EErrorExt error_, final int value_) {
		super(error_.getErrorCode(), error_.getError() + ": " + value_);
	}

	/**
	 * A TINE error code exception is created with the error extension
	 * enumeration. 
	 * @param error_ Is the error extension enumeration.
	 * @param value_ Is the subjected error number.
	 */
	public TineUserException(final EErrorExt error_, final float value_) {
		super(error_.getErrorCode(), error_.getError() + ": " + value_);
	}
	
	/**
	 * Constructor of a TINE failure exception. For the error code number
	 * you should have a look at the TINE class {@code TErrorList}.
	 * @param error_ Is the TINE error code.
	 */
	public TineUserException(final int error_) {
		super(error_);
	}

	/**
	 * Constructor of a TINE failure exception. For the error code number
	 * you should have a look at the TINE class {@code TErrorList}.
	 * @param error_ Is the TINE error code.
	 * @param message_ Is a detailed explanation of the error.
	 */
	public TineUserException(final int error_, final String message_) {
		super(error_, ((message_ == null || message_.isEmpty())?(""):(": " + message_)));
	}

	/**
	 * Constructor of a TINE failure exception. For the error code number
	 * you should have a look at the TINE class {@code TErrorList}.
	 * @param error_ Is the TINE error code.
	 * @param value_ Is the subjected error number.
	 */
	public TineUserException(final int error_, final int value_) {
		super(error_, "" + value_);
	}

	/**
	 * Constructor of a TINE failure exception. For the error code number
	 * you should have a look at the TINE class {@code TErrorList}.
	 * @param error_ Is the TINE error code.
	 * @param value_ Is the subjected error number.
	 */
	public TineUserException(final int error_, final float value_) {
		super(error_, "" + value_);
	}
	
	// --- INSTANCE METHODS ----------------------------------------------------
	@Override public String toString() {
		final String message = super.getMessage() == null ? "" : super.getMessage() + " - ";
		return "TINE failure: " + message + TErrorList.getErrorString(super.getErrorKey()) + " (" + super.getErrorKey() + ')';
 	}
}
/* end of file */