/*
 *
 * File: DByte.java
 * Date: Nov 29, 2006; 4:56:58 PM
 */
package de.desy.mst.libs.accelerator.number;

/**
 * This will encapsulate a single byte value.
 *
 * @author <a href="marcus.walla at desy.de">walla</a>
 * @version $Id: DByte.java 22583 2009-03-10 11:22:31Z walla $
 * @see INumber
 * @see DShort
 * @see DInteger
 * @see DLong
 * @see DFloat
 * @see DDouble
 */
public final class DByte implements INumber
{
	
	/** The internal byte variable. */
	private Byte _byte = 0;
	
	/**
	 * Constructor of a the field byte. The byte value is set to zero.
	 */
	public DByte()
	{
		_byte = 0;
	}
	
	/**
	 * The constructor of the field byte. Will get one argument as a byte value.
	 * 
	 * @param byte_ Gets one argument as a byte value.
	 */
	public DByte(byte byte_)
	{
		_byte = byte_;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#booleanValue()
	 */
	public boolean booleanValue()
	{
		return _byte == 1 ? true : false;
	}
	
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#byteValue()
	 */
	public byte byteValue()
	{
		return _byte;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#doubleValue()
	 */
	public double doubleValue()
	{
		return _byte.doubleValue();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#floatValue()
	 */
	public float floatValue()
	{
		return _byte.floatValue();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#intValue()
	 */
	public int intValue()
	{
		return _byte.intValue();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#longValue()
	 */
	public long longValue()
	{
		return _byte.longValue();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#shortValue()
	 */
	public short shortValue()
	{
		return _byte.shortValue();
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(byte)
	 */
	public void setValue(boolean bool_)
	{
		_byte = (byte) (bool_ == true ? 1 : 0);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(byte)
	 */
	public void setValue(byte byte_)
	{
		_byte = byte_;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(short)
	 */
	public void setValue(short short_)
	{
		setValue((byte) short_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(int)
	 */
	public void setValue(int int_)
	{
		setValue((byte) int_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(long)
	 */
	public void setValue(long long_)
	{
		setValue((byte) long_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(float)
	 */
	public void setValue(float float_)
	{
		setValue((byte) float_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(double)
	 */
	public void setValue(double double_)
	{
		setValue((byte) double_);
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(final Object object_)
	{
		return _byte.equals(object_);
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode()
	{
		return _byte.hashCode();
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		return _byte.toString();
	}

}
/* end of file */