/*
 *
 * File: DFloat.java
 * Date: Nov 29, 2006; 6:12:58 PM
 */
package de.desy.mst.libs.accelerator.number;

/**
 * This will encapsulate a single float value.
 *
 * @author <a href="marcus.walla at desy.de">walla</a>
 * @version $Id: DFloat.java 22583 2009-03-10 11:22:31Z walla $
 * @see INumber
 * @see DByte
 * @see DShort
 * @see DLong
 * @see DInteger
 * @see DDouble
 *
 */
public final class DFloat implements INumber
{
	
	/** The internal float variable. */
	private Float _float = 0.0f;
	
	/**
	 * Constructor of a the field float. The float value is set to zero.
	 */
	public DFloat()
	{
		_float = 0.0f;
	}
	
	/**
	 * The constructor of the field float. Will get one argument as a float value.
	 * 
	 * @param float_ Gets one argument as a float value.
	 */
	public DFloat(float float_)
	{
		_float = float_;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#booleanValue()
	 */
	public boolean booleanValue()
	{
		return _float == 1.0f ? true : false;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#byteValue()
	 */
	public byte byteValue()
	{
		return _float.byteValue();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#doubleValue()
	 */
	public double doubleValue()
	{
		return _float.doubleValue();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#floatValue()
	 */
	public float floatValue()
	{
		return _float;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#intValue()
	 */
	public int intValue()
	{
		return _float.intValue();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#longValue()
	 */
	public long longValue()
	{
		return _float.longValue();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#shortValue()
	 */
	public short shortValue()
	{
		return _float.shortValue();
	}

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(byte)
	 */
	public void setValue(boolean bool_)
	{
		_float = (bool_ == true ? 1.0f : 0.0f);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(byte)
	 */
	public void setValue(byte byte_)
	{
		setValue((float) byte_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(short)
	 */
	public void setValue(short short_)
	{
		setValue((float) short_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(int)
	 */
	public void setValue(int int_)
	{
		setValue((float) int_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(long)
	 */
	public void setValue(long long_)
	{
		setValue((float) long_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(float)
	 */
	public void setValue(float float_)
	{
		_float = float_;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(double)
	 */
	public void setValue(double double_)
	{
		setValue((float) double_);
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(final Object object_)
	{
		return _float.equals(object_);
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode()
	{
		return _float.hashCode();
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		return _float.toString();
	}
}
/* end of file */