/*
 *
 * File: CFrame.java
 * Date: Jan 08, 2008; 03:21:00 PM
 */
package de.desy.mst.libs.accelerator.view;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;

import javax.swing.BorderFactory;
import javax.swing.JComponent;

import de.desy.mst.libs.accelerator.CGeneralConstant;
import de.desy.mst.libs.accelerator.EDebug;
import de.desy.mst.libs.accelerator.view.CColor;
import de.desy.mst.libs.framework.mstapp.Screen;

/**
 * All graphical static constant fields are defined here. Here are all constant
 * fields defined for displaying different {@code JComponent}'s on screen.
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: CFrame.java 35394 2010-04-23 15:38:58Z walla $
 */
public class CFrame
{

	/**
	 * Creates an outer border around the component for debugging purposes.
	 * Remember also to set the value of
	 * {@link de.desy.mst.libs.accelerator.CGeneralConstant#DEBUG_GRAPHIC} to
	 * {@code true} otherwise no border will be drawn!
	 *
	 * @param component_ Is the component where a red border should be drawn.
	 * 		If this component is null nothing will happen.
	 * @see de.desy.mst.libs.accelerator.CGeneralConstant#DEBUG_GRAPHIC
	 */
	public final static void DEBUG_BORDER(final JComponent component_)
	{
		if (component_ != null &&
				CGeneralConstant.isDebug(EDebug.DEBUG_GRAPHIC))
			component_.setBorder(BorderFactory.createLineBorder(CColor.RED));
	}

	/** This is the width size of the main RF frame. */
	public final static int DIM_RF_FRAME_WIDTH = Screen.FULL_SCREEN.width;
	/** This is the height size of the main RF frame. */
	public final static int DIM_RF_FRAME_HEIGHT = Screen.FULL_SCREEN.height;
	/** This is the dimension of the main RF frame. */
	public final static Dimension DIM_RF_FRAME = new Dimension(DIM_RF_FRAME_WIDTH
			, DIM_RF_FRAME_HEIGHT);

	/** This is the width size of the main RF panel. */
	public final static int DIM_RF_PANEL_WIDTH = DIM_RF_FRAME_WIDTH * 96/100;
	/** This is the height size of the main RF panel. */
	public final static int DIM_RF_PANEL_HEIGHT = DIM_RF_FRAME_HEIGHT * 84/100;
	/** This is the dimension of the main RF panel. */
	public final static Dimension DIM_RF_PANEL = new Dimension(DIM_RF_FRAME_WIDTH
			, DIM_RF_PANEL_HEIGHT);

	/** This is the width size of the RF switch check box. */
	public final static int DIM_SWITCH_CHECKBOX_WIDTH = 76;
	/** This is the height size of the RF switch check box. */
	public final static int DIM_SWITCH_CHECKBOX_HEIGHT = 24;
	/** This is the dimension of the RF switch check box. */
	public final static Dimension DIM_SWITCH_CHECKBOX = new Dimension(DIM_SWITCH_CHECKBOX_WIDTH
			, DIM_SWITCH_CHECKBOX_HEIGHT);



	/** Get the width of the desktop frame, which means the width of the hole
	 * platform dependent size. */
	public final static int DIM_DESKTOP_WIDTH =
		Toolkit.getDefaultToolkit().getScreenSize().width;

	/** Get the height of the desktop frame, which means the height of the hole
	 * platform dependent size. */
	public final static int DIM_DESKTOP_HEIGHT =
		Toolkit.getDefaultToolkit().getScreenSize().height;

	/** The following value is for calculating the right dimension of the font
	 * according to the size of the desktop width. */
	public final static float DIM_RATIO = DIM_DESKTOP_HEIGHT / 1024.0f;



	/** The gap between all components of the status panel. */
	public final static int GAP_STATUS = 30;



	/** Set the font family and type. */
	public static final String FONT_TYPE = "Arial";

	/** */
	public static final Font FONT_HEADING =
		new Font(FONT_TYPE, Font.BOLD, (int) (28 * DIM_RATIO));

	public static final Font FONT_CAVITY =
		new Font(FONT_TYPE, Font.PLAIN, (int) (16 * DIM_RATIO));

	public static final Font FONT_CAVITY_TAB =
		new Font(FONT_TYPE, Font.PLAIN, (int) (18 * DIM_RATIO));

	public static final Font FONT_NORMAL =
		new Font(FONT_TYPE, Font.PLAIN, (int) (12 * DIM_RATIO));

	public static final Font FONT_NORMAL_BOLD =
		new Font(FONT_TYPE, Font.BOLD, (int) (12 * DIM_RATIO));

	public static final Font FONT_MORE_NORMAL =
		new Font(FONT_TYPE, Font.PLAIN, (int) (16 * DIM_RATIO));

	public static final Font FONT_MORE_NORMAL_BOLD =
		new Font(FONT_TYPE, Font.BOLD, (int) (16 * DIM_RATIO));

	public static final Font FONT_BIG =
		new Font(FONT_TYPE, Font.PLAIN, (int) (20 * DIM_RATIO));

	public static final Font FONT_BIG_BOLD =
		new Font(FONT_TYPE, Font.BOLD, (int) (20 * DIM_RATIO));

	public static final Font FONT_BIGGER =
		new Font(FONT_TYPE, Font.PLAIN, (int) (24 * DIM_RATIO));

	public static final Font FONT_BIGGER_BOLD =
		new Font(FONT_TYPE, Font.BOLD, (int) (24 * DIM_RATIO));

	public static final Font FONT_TABBED =
		new Font(FONT_TYPE, Font.PLAIN, (int) (24 * DIM_RATIO));

	public static final Font FONT_CONTROL =
		new Font(FONT_TYPE, Font.PLAIN, (int) (30 * DIM_RATIO));

	public static final Font FONT_STATUS =
		new Font(FONT_TYPE, Font.PLAIN, (int) (18 * DIM_RATIO));

	public static final Font FONT_VALUE =
		new Font(FONT_TYPE, Font.PLAIN, (int) (22 * DIM_RATIO));

	public static final Font FONT_SLIDER =
		new Font(FONT_TYPE, Font.PLAIN, (int) (12 * DIM_RATIO));

	public static final Font FONT_SWITCH =
		new Font(FONT_TYPE, Font.PLAIN, (int) (16 * DIM_RATIO));

	public static final Font FONT_TRANSMITTER =
		new Font(FONT_TYPE, Font.PLAIN, (int) (22 * DIM_RATIO));

	public static final Font FONT_COMBOBOX =
		new Font(FONT_TYPE, Font.PLAIN,	(int) (14 * DIM_RATIO));

	public static final Font FONT_TITLE_BORDER =
		new Font(FONT_TYPE, Font.PLAIN, (int) (12 * DIM_RATIO));

}
/* end of file */