/*
 *
 * File: RFApplicationInfo.java
 * Date: Aug 13, 2008; 3:37:35 PM
 */
package de.desy.mst.libs.accelerator.view;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

import de.desy.mst.libs.accelerator.Version;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Displays application and FEC information in a modal dialog box.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFApplicationInfo.java 20008 2009-01-05 12:21:06Z walla $
 */
public final class RFApplicationInfo extends JDialog
{
	public final static long serialVersionUID = 1362459006365316793L;

	public final static int GAP_VERTICAL_APP = 6;

	public final static int GAP_HORIZONTAL_APP = 10;

	public final static int GAP_BORDER = 5;

	public final static int GAP_BUTTON = 10;

	private static RFApplicationInfo _dialog;

	private JPanel _panel;

	private JLabel lblApp = new JLabel("Applikation:");
	private JLabel lblCommon = new JLabel("Allgemeine Bibliothek:");
	private JLabel lblLib = new JLabel("HF Bibliothek:");

	private JLabel lblAppName;
	private JLabel lblCommonName;
	private JLabel lblLibName;

	private JLabel lblAppVersion;
	private JLabel lblCommonVersion;
	private JLabel lblLibVersion;


	public final static RFApplicationInfo getInstance(final Version... version_)
	{
		return getInstance(null, version_);
	}

	public synchronized final static RFApplicationInfo getInstance(
			final Frame frame_, final Version... version_)
	{
		if (_dialog == null)
			_dialog = new RFApplicationInfo(frame_, true, version_);

		return _dialog;
	}

	/**
	 *
	 * @param frame_
	 * @param modal_
	 */
	private RFApplicationInfo(final Frame frame_, final boolean modal_,
			final Version... version_)
	{
		super(frame_, modal_);
		setVersion(version_);
		getMainPanel();
	}

	private void getMainPanel()
	{
		if (_panel == null)
		{
			_panel = new JPanel();
			GroupLayout layout = new GroupLayout(_panel);
			_panel.setLayout(layout);

			JButton btnOK = new JButton("OK");
			btnOK.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e)
				{
					Logger.SWApp.info("You pressed the OK button.");
					setVisible(false);
				}

			});

			layout.setHorizontalGroup(layout.createSequentialGroup()
					.addContainerGap()
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.CENTER)
						.addGroup(layout.createSequentialGroup()
							.addGroup(layout.createParallelGroup()
								.addComponent(lblApp)
								.addComponent(lblCommon)
								.addComponent(lblLib))
							.addGap(GAP_HORIZONTAL_APP)
							.addGroup(layout.createParallelGroup()
									.addComponent(lblAppName)
									.addComponent(lblCommonName)
									.addComponent(lblLibName))
							.addGap(GAP_HORIZONTAL_APP)
							.addGroup(layout.createParallelGroup()
									.addComponent(lblAppVersion)
									.addComponent(lblCommonVersion)
									.addComponent(lblLibVersion))
							)
						.addGap(GAP_BUTTON)
						.addComponent(btnOK)
					)
					.addContainerGap()
			);

			layout.setVerticalGroup(layout.createSequentialGroup()
					.addContainerGap()
					.addGroup(layout.createParallelGroup()
						.addGroup(layout.createSequentialGroup()
								.addComponent(lblApp)
								.addGap(GAP_VERTICAL_APP)
								.addComponent(lblCommon)
								.addGap(GAP_VERTICAL_APP)
								.addComponent(lblLib))
						.addGroup(layout.createSequentialGroup()
								.addComponent(lblAppName)
								.addGap(GAP_VERTICAL_APP)
								.addComponent(lblCommonName)
								.addGap(GAP_VERTICAL_APP)
								.addComponent(lblLibName))
						.addGroup(layout.createSequentialGroup()
								.addComponent(lblAppVersion)
								.addGap(GAP_VERTICAL_APP)
								.addComponent(lblCommonVersion)
								.addGap(GAP_VERTICAL_APP)
								.addComponent(lblLibVersion)))
					.addGap(GAP_BORDER)
					.addComponent(btnOK)
					.addContainerGap()
			);

			this.getContentPane().add(_panel);
		}
	}

	private void setVersion(final Version... version_)
	{
		if (version_.length != 3)
			throw new IllegalArgumentException("There must be three versions" +
					" to install the right version information panel: "
					+ version_.length);

		if (lblAppName == null)
			lblAppName = new JLabel(version_[0].getApplicationName());
		else
			lblAppName.setText(version_[0].getApplicationName());

		if (lblCommonName == null)
			lblCommonName = new JLabel(version_[1].getApplicationName());
		else
			lblCommonName.setText(version_[1].getApplicationName());

		if (lblLibName == null)
			lblLibName = new JLabel(version_[2].getApplicationName());
		else
			lblLibName.setText(version_[2].getApplicationName());

		if (lblAppVersion == null)
			lblAppVersion = new JLabel(version_[0].getApplicationVersion());
		else
			lblAppVersion.setText(version_[0].getApplicationVersion());

		if (lblCommonVersion == null)
			lblCommonVersion = new JLabel(version_[1].getApplicationVersion());
		else
			lblCommonVersion.setText(version_[1].getApplicationVersion());

		if (lblLibVersion == null)
			lblLibVersion = new JLabel(version_[2].getApplicationVersion());
		else
			lblLibVersion.setText(version_[2].getApplicationVersion());
	}

}
/* end of file */