/*
 *
 * File: TrigonometricalFunction.java
 * Date: Nov 6, 2006; 3:30:34 PM
 */
package de.desy.mst.libs.accelerator.view;

/**
 * <p>This class will calculate the values of the sine and cosine function in
 * steps of 1 degree where the degree runs from 0 to 359. The tangent function
 * is not calculated.</p>
 * <p>These class was used for the never realized project from Mr. Timmy Lensch
 * (Hardware developer, Group MDI 1, Building 30).</p>
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: TrigonometricalFunction.java 19706 2008-12-14 12:41:25Z walla $
 */
public class TrigonometricalFunction
{
	/** Is the singelton of this class. */
	private static volatile TrigonometricalFunction _trigonomical =
		new TrigonometricalFunction();

	/**
	 * This method returns the singelton of the trigonometrical function class.
	 * It is thread-safe.
	 *
	 * @return A thread-safe singelton is returned.
	 */
	public static final TrigonometricalFunction getInstance()
	{
		return _trigonomical;
	}

	/** An array which holds the value of the cosinus function between 0 and 359. */
	private float[] _cos = null;

	/** An array which holds the value of the sinus function between 0 and 359. */
	private float[] _sin = null;

	/**
	 * The private constructor of this class.
	 */
	private TrigonometricalFunction()
	{
		_cos = new float[360];
		_sin = new float[360];

		initialize();
	}

	/**
	 * Initializes the sinus and cosinus trigonomical functions.
	 */
	private void initialize()
	{
		for(int i = 0; i < 360; i++)
		{
			_sin[i] = (float) Math.sin(Math.PI / 180.0 * i);
			_cos[(i + 90) % 360] = _sin[i];
		}
	}

	/**
	 * Returns the sine function value of x, where x is between 0 and 359.
	 *
	 * @param x The argument of the sine function. An integer value is
	 * 			specified.
	 * @return Returns the function value.
	 */
	public float sin(final int x)
	{
		if (x < 0 || x > 359)
			throw new IllegalArgumentException("The argument is not in the" +
					" range from 0 to 359:" + x);
		return _sin[x];
	}

	/**
	 * Returns the cosine function value of x, where x is between 0 and 359.
	 *
	 * @param x The argument of the cosine function. An integer value is
	 * 			specified.
	 * @return Returns the function value.
	 */
	public float cos(final int x)
	{
		if (x < 0 || x > 359)
			throw new IllegalArgumentException("The argument is not in the range from 0 to 359."
					+ " The argument was " + x);
		return _cos[x];
	}
}
/* end of file */