/**
 * Sep  6, 2006 - Sep  6, 2006	ws: created from scratch
 * Aug 29, 2007 - Aug 29, 2007 	ws: String appName -> ApplicationInfo appInfo
 * May 29, 2008 - May 29, 2008	ws: createIconImage mit Webstart default
 * Aug  4, 2008 - Aug  4, 2008	ws: added: TOOLTIP_DISMISSDELAY stuff
 * Aug 15, 2008 - Aug 15, 2008	ws: appName -> alName (nur Bezeichnung)
 */
package de.desy.mst.libs.framework.mstapp;

import java.awt.BorderLayout;
import java.awt.Image;

import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.ToolTipManager;

import de.desy.mst.libs.framework.mstapp.ini.MasterMgrForIniFiles;
import de.desy.mst.libs.framework.mstapp.menu.IPopupMenuMgr;
import de.desy.mst.libs.framework.mstapp.util.gui.MstImage;

import javax.swing.WindowConstants;

/**
 * Basis Frame fuer MstApp Frames<br>
 * Unterstuetzt
 * <li> den MstApp Kontext MstApp
 * <li> den PopUpMgr (rechtsklicks)
 * <li> isInsideVisualEditor
 *
 * @version 1.0.0
 * @author  ws
 */
public class MstFrame extends JFrame {
	private static final long serialVersionUID = 1L;
	/**5 Minuten (siehe AppMeeting 2008-08-04)*/
	private static final int  TOOLTIP_DISMISSDELAY = 5* 60 * 1000;
	static{
		ToolTipManager.sharedInstance().setDismissDelay(TOOLTIP_DISMISSDELAY);
	}

//=============================================================================

    public final boolean isInsideVisualEditor;
	private final MstApp mstApp; //Die MstApp Kontext: Fassade zum allgemeinen Support

	private JPanel jContentPane = null;

//=============================================================================

	/**
	 * Der Konstruktor zum benutzen.
	 * @throws NullPointerException, falls mstApp == null
	 */
	public MstFrame(final MstApp mstApp) {
		super();
		if(mstApp == null)  throw new NullPointerException("Das Kontextobjekt MstApp darf nicht null sein");
		this.mstApp = mstApp;
		this.isInsideVisualEditor = false;
		initialize();
	}
	/**
	 * For Visual Editor ONLY, always pass mstApp
	 */
	@Deprecated
	public MstFrame() {
		super();
		this.mstApp = createDummyMstApp();
		this.isInsideVisualEditor = true;
		initialize(); 
	}

//=============================================================================

	/**MstApp Kontext: nie null*/
	public MstApp 			getMstApp()			{return this.mstApp;}
	/**Rechts Klick support: nie null*/
	public IPopupMenuMgr 	getPopUpMenuMgr()	{return getMstApp().getPopUpMenuMgr();}


//=============================================================================

	/**NUR um Visual Editor den Default Konstruktor anbieten zu koennen - never do it for real*/
	private MstApp createDummyMstApp() {
		final String fcAppName = getClass().getName();	//fully qualified with package name
		final String[] fakedArgs ={fcAppName};
		final ParamDecoder decoder = new ParamDecoder(fakedArgs);
        final Main main =  new Main("Dummy TG");
        final MasterMgrForIniFiles masterMgrForIniFiles = new MasterMgrForIniFiles
        						(decoder.applicationInfo, main.getMessageMgr());
        main.setMasterMgrForIniFiles(masterMgrForIniFiles);
        final MstApp mstApp = new MstApp(main, decoder);
        main.reverseInit(mstApp, decoder.appName);
		return mstApp;
	}

//-----------------------------------------------------------------------------

	private void initialize() {
		this.setSize(683, 57);
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		this.setContentPane(getJContentPane());

		final ParamDecoder startParameter = getMstApp().getStartParameter();
		final String appName = startParameter.appName;
		this.setTitle(appName);

		final IPopupMenuMgr popUpMenuMgr = getMstApp().getPopUpMenuMgr();
		this.addMouseListener(popUpMenuMgr.getStandardPopUpMouseListener());

		//final Image image = getMstApp().getMstImage().createImage();
		//besser folgendes, da getClass im .jar der Applikation ist
		final String alName = startParameter.applicationInfo.getShortFrameName();
		final Image image = ( new MstImage(alName, getClass()) ).createImage();
		if(image != null)
			this.setIconImage(image);

	}
	private JPanel getJContentPane() {
		if (this.jContentPane == null) {
			this.jContentPane = new JPanel();
			this.jContentPane.setLayout(new BorderLayout());
		}
		return this.jContentPane;
	}
}  //  @jve:decl-index=0:visual-constraint="10,10"
