/*
 *   :	Mar 17, 2005 - Mar 21, 2005	creation
 * ws:	May 25, 2007 - May 25, 2007	attack ini problem with values
 * ws:	Apr 18, 2008 - Apr 18, 2008	added center(window)
 */
package de.desy.mst.libs.framework.mstapp;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Window;
import java.util.HashMap;
import java.util.Map;

/**
 * Utility Klasse um Formen auf der Screen zu sizen und anzuordnen.
 * USES {@link ScreenIni} for Base layout.
 * <br> {@link ScreenIni} MUST be called <b>BEFORE</b> using Swing. A good place
 * is the main routine.
 *
 * @author hinsch, schuette
 * @version 1.0.0
 */
public final class Screen {
	private static final Dimension DIMENSION             = ScreenIni.getReferenceScreenSize();
	private static final int       HEADER_HEIGHT         = 41;
	private static final int       TASKBAR_HEIGHT        = 28;
	private static final int       WIDTH  		         = DIMENSION.width;
	private static final int       HEIGHT 		         = DIMENSION.height-HEADER_HEIGHT-TASKBAR_HEIGHT;
	private static final int       HALF_WIDTH            = WIDTH/2;
	private static final int       HALF_HEIGHT           = HEIGHT/2;
	private static final Map<Integer, Screen> values = new HashMap<Integer, Screen>(10);

	//1) FULL_SCREEN
	public static final Screen FULL_SCREEN;

	//2) HALF_SCREEN
	public final static class HALF {
		public static final Screen UPPER;
		public static final Screen LOWER;
		public static final Screen LEFT;
		public static final Screen RIGHT;
		static{
			UPPER	= new Screen("UPPER_HALF", 		1, 0, 			HEADER_HEIGHT,			     WIDTH, HALF_HEIGHT);
			LOWER	= new Screen("LOWER_HALF", 		2, 0, 			HALF_HEIGHT + HEADER_HEIGHT, WIDTH, HALF_HEIGHT);
			LEFT	= new Screen("LEFT_HALF", 		3, 0, 			HEADER_HEIGHT,			     HALF_WIDTH, HEIGHT);
			RIGHT	= new Screen("RIGHT_HALF", 		4, HALF_WIDTH, 	HEADER_HEIGHT,               HALF_WIDTH, HEIGHT);
		}
		private HALF(){/*nicht instanzierbar*/}
	}

	//3) QUARTER_SCREEN
	public final static class QUARTER{
		public static final Screen TOP_LEFT;
		public static final Screen TOP_RIGHT;
		public static final Screen BOTTOM_LEFT;
		public static final Screen BOTTOM_RIGHT;
		static{
			TOP_LEFT 	    = new Screen("TOP_LEFT", 	    5, 0, 			HEADER_HEIGHT,			     HALF_WIDTH, HALF_HEIGHT);
			TOP_RIGHT 	    = new Screen("TOP_RIGHT", 	    6, HALF_WIDTH, 	HEADER_HEIGHT,			     HALF_WIDTH, HALF_HEIGHT);
			BOTTOM_LEFT 	= new Screen("BOTTOM_LEFT", 	7, 0, 			HALF_HEIGHT + HEADER_HEIGHT, HALF_WIDTH, HALF_HEIGHT);
			BOTTOM_RIGHT 	= new Screen("BOTTOM_RIGHT", 	8, HALF_WIDTH, 	HALF_HEIGHT + HEADER_HEIGHT, HALF_WIDTH, HALF_HEIGHT);

		}
		private QUARTER(){/*nicht instanzierbar*/}
	}
	//4) Server
	public final static class SERVER{
		public static final int ICON_SIZE      = WIDTH/7;
		public static final int NORMAL_WIDTH   = 640;
		public static final int NORMAL_HEIGHT  = 530;
		public static final int SERVICE_WIDTH  = 515;//ohne xtras
		public static final int SERVICE_HEIGHT = 770;//ohne xtras

		public final static Rectangle calcIconRectangle(final int serverNo) {
			int x;
			int y;
			int line;
			int positionInLine;
			line = (serverNo - 1)/7;
			positionInLine = (serverNo - 1) - (line * 7);
			x = ICON_SIZE * positionInLine;
			y = ICON_SIZE * line;
			return new Rectangle(x, y, ICON_SIZE, ICON_SIZE);
		}
		public final static Rectangle calcServiceRectangle(final int xtraServiceWidth, final int xtraServiceHeight) {
			return new Rectangle(0, 0
					              , SERVICE_WIDTH  + xtraServiceWidth
					              , SERVICE_HEIGHT + xtraServiceHeight);
		}
		public final static Rectangle calcNormalRectangle() {
			return new Rectangle(0, 0, NORMAL_WIDTH, NORMAL_HEIGHT);
		}
		private SERVER(){/*nicht instanzierbar*/}
	}
	/**Do not use, nicht korrekt initialisiert 2007-05-25*/
	@Deprecated public static final Screen[] VALUES;

//	(int x, int y, int width, int height)
	public final String name;
	public final int index;
	public final int x;
	public final int y;
	public final int width;
	public final int height;

//=============================================================================

	static{
		FULL_SCREEN = new Screen("FULL_SCREEN", 	0, 0, 			HEADER_HEIGHT,			     WIDTH, 	 HEIGHT);
		//Beachte: Index(Array) = index(Objekt)
		VALUES = (new Screen[]{FULL_SCREEN
				             , HALF.UPPER,    HALF.LOWER,     HALF.LEFT,     HALF.RIGHT
				             , QUARTER.TOP_LEFT, QUARTER.TOP_RIGHT, QUARTER.BOTTOM_LEFT, QUARTER.BOTTOM_RIGHT
				             });
	}

//=============================================================================

	private Screen(final String name, final int index,
			final int x, final int y, final int width, final int height) {
		super();
		this.name = name;
		this.index = index;
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		Screen.values.put(Integer.valueOf(index), this);
	}

//	=============================================================================

	/**
	 * @param component	dimensioniert und bewegt die Komponente zu der gewaehlten Screen Postition
	 */
	public void setBoundsOnComponent(final Component component) {
		component.setBounds(this.x, this.y, this.width, this.height);
	}

	/**
	 * @return complete List of predefined Screens (non Server)
	 */
	public static final Screen[] getValues() {
		return values.values().toArray(new Screen[0]);
	}

	/**Zentriert diese Form ohne die Groesse zu aendern*/
	public static void center(final Window window) {
		final int height = window.getHeight();
		final int width  = window.getWidth();
		final int x = HEADER_HEIGHT + (FULL_SCREEN.height - height) / 2;
		final int y = (FULL_SCREEN.width - width) / 2;
		window.setBounds(x, y, width, height);
	}

	/**
	 * @return Returns the hEADER_HEIGHT.
	 * <br>&author K. Hinsch -mcs1-
	 */
	/*
	 * Vers		Date				Description
	 *  01	Oct 29, 2008	Methode erstellt
	 */
	public static int getHEADER_HEIGHT() {
		return HEADER_HEIGHT;
	}
}
