/**
 * ws:	Nov 21, 2007 - Nov 27, 2007 creation from scratch /ein Teil von Main
 * ws:	Jan 29, 2008 - Jan 29, 2008	added: IMstAppKernel
 * wsL	Jan  6, 2009 - Jan  8, 2009	replaced: MsgBox with UncaughtExceptionDialog
 * ws:	Mar  4, 2009 - Mar  4, 2009	replaced "Todesschirm"  mit "Postmortemschirm" (FrWe)
 * 									uncaughtExceptionFormatMsg mit date, time (AnLa)
 * ws:	Jan 12, 2010 - Jan 12, 2010 handleUncaughtException pp wegen Main.exit(int,throwable)
 * ws:	Apr 21, 2010 - Apr 22, 2010 added: Mail Anschluss, Infos zu den jars
 * Nov 25, 2014 - Nov 25, 2014	kh, ws: mehr Output bei fataler final error
 * Nov 25, 2014 - Nov 25, 2014	ws	panic fenced calcJarInfo() due to strange problems with jmsviewer
 */
package de.desy.mst.libs.framework.mstapp;

import java.awt.Component;
import java.util.Collection;
import java.util.Date;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;

import javax.swing.JOptionPane;

import de.desy.mst.libs.framework.mstapp.appinf.ApplikationKind;
import de.desy.mst.libs.framework.mstapp.appinf.IApp;
import de.desy.mst.libs.framework.mstapp.appinf.ILib;
import de.desy.mst.libs.framework.mstapp.io.Mail;
import de.desy.mst.libs.framework.mstapp.logging.EinfacherLogFormatter;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.security.MstSecurity;
import de.desy.mst.libs.framework.mstapp.util.MstDateFormat;

/**
 * Falls in der Applikation ein nichtbehandelter Fehler auftaucht wird hier
 * ein letzter Handler zur Verfuegung gestellt.<br>
 * package private
 * @author ws
 * @version 1.0.0
 */
class UncaughtExceptionHandler {
	private final static AtomicInteger nUncaughtExeptions = new AtomicInteger(0);
	private final static AtomicBoolean showMsgBox = new AtomicBoolean(true);
//=============================================================================
	/**pp, @throws NullPointerException*/
	static void createUncaughtExceptionHandler(final IMstAppKernel mstAppK) {
		if(mstAppK == null)
			throw new NullPointerException("IMstAppKernel may not be null");
		Thread.setDefaultUncaughtExceptionHandler(
				new Thread.UncaughtExceptionHandler(){
					@Override public void uncaughtException(final Thread t, final Throwable e) {
						handleUncaughtException(mstAppK, t, e);
					}
				}
		);
	}
	/**pp*/ static void setShowMsgBox(final boolean doShow){showMsgBox.set(doShow);}
	/**pp*/ static boolean showMsgBox() {return showMsgBox.get();}

	/**pp*/ static void handleUncaughtException(final IMstAppKernel mstAppK
											  , final Thread t
											  , final Throwable e) {
        try {
        	// 0) Zaehlen
        	final int n = nUncaughtExeptions.incrementAndGet();
            // 1) Logging
            final String msg = "Main.uncaughtException (" + n + ")was called from '"
            		   + t.getName() + "' with '" + e.getMessage() + "'";
			Logger.SWFW.fatal(msg, e);
			if(n <= 1)
				giveDetailedFeedback(mstAppK, t, e);
        } catch (final Exception internalE) 	{/*just ignore it*/}
        //Try a decent exit
        try		{Main.exit(1);}
        catch	(final Exception e1)		{/*just ignore it*/}
	}
//=============================================================================
	private static void giveDetailedFeedback(final IMstAppKernel mstAppK, final Thread t, final Throwable e) {
		try {
			final String msg = uncaughtExceptionFormatMsg(t, e) + calcJarInfo();
			final String title = Main.calcAnnotatedTitle("[Postmortem Informationen]");
			if(showMsgBox())
				showUncaughtExceptionDialog(title, msg, mstAppK);
			else /*if(!isInIDE())*/	//isInIDE() Funktion fehlt i.A.
				sendMail(title, msg);
		} catch (final Exception ex) {
			final String msg = "Exception in Exception Handling of Uncaught Exception.";
			final StackTraceElement[] stackTrace = ex.getStackTrace();
			final StringBuilder sb = new StringBuilder(".");
			for (final StackTraceElement el : stackTrace) 
				sb.append(" /n").append(el.toString());
			Logger.SWFW.fatal(msg + sb.toString(), ex);
			if(showMsgBox())
				uncaughtExceptionShowMsgBox(msg);
		}
	}
	private static String calcJarInfo() {
		try{
			final IApp info = getAnLaApplicationInformation();
			if(info == null)
				return "";
	
			final Collection<ILib> jars = info.getJars();
			if(jars == null)
				return "";
	
			final StringBuilder sb = new StringBuilder("\n\nVerwendete Bibliotheken:");
			for (final ILib jar : jars)
				calcJarInfo(sb, jar);
			return sb.toString();
		}
		catch(final Throwable th) {
			//macht im jms viewer unerklaerliche Probleme, dabei nur nice to have
			return "###UncaughtExceptionHandler.calcJarInfo() has no info###";
		}
	}
	private static IApp getAnLaApplicationInformation() {
		try{return ApplikationKind.Automatic.getApplicationInformation();}
		catch(final Throwable th){/*silently ignore*/}
		return null;
	};
	private static void calcJarInfo(final StringBuilder sb, final ILib jar) {
		try{
			sb.append("\n")
			  .append("\n").append(jar.getName())
			  .append("\n").append("Datum: ")	.append(jar.getDate())
			  .append("\n").append("Version: ")	.append(jar.getMajor())
			  						.append(".").append(jar.getMinor())
			  						.append(".").append(jar.getRevision());
		}catch (final Throwable th){/*silently ignor*/}
	}
	private static void sendMail(final String title, final String msg) {
		final String entryAutor = "";
		final String[] to   = null;
		final String   from = null;
		new Mail().sendMail(title, msg, entryAutor, to, from);
	}
	private static void showUncaughtExceptionDialog(final String title, final String msg, final IMstAppKernel mstAppK) {
		final UncaughtExceptionDialog uncaughtExceptionDialog = new UncaughtExceptionDialog(mstAppK, getLastCreatedFrame(), title, msg);
		uncaughtExceptionDialog.setVisible(true);
	}
	@SuppressWarnings("deprecation")
	private static MstFrameMain getLastCreatedFrame() {
		return Main.getLastCreatedFrame();
	}
	/**pp*/ static String uncaughtExceptionFormatMsg(final Thread t, final Throwable e) {
		final String threadName = (t == null) ? "null" : t.getName();
		final String threadGroupName = (t == null || t.getThreadGroup() == null)
									 ? "null"
									 : t.getThreadGroup().getName();
		final String stackTrace = (e == null) ? "null" :
						(new EinfacherLogFormatter()).throwableFormat(e);
		final String appInitMsg =
			(stackTrace.contains(Main.KONSTRUKOR_PROBLEM))
			   ? "\n\nSymptome: ApplikationsFrame Initialisierungs Problem."
				+" Bitte an Applikationsentwickler wenden."
			   : "";
		final String msg =
			  "Ein Fehler ist unerwartet aufgetreten. "
			+ "Damit der Fehler von uns behoben werden kann, eventuell dieses Fenster in das Logbuch drucken."
			+ "\nWann: " + decorate(MstDateFormat.ISO.format(new Date()))
			+ "\nWo:   " + decorate(MstSecurity.get().getHostName())
						 + " fr unsere IP'ler "
						 + decorate(MstSecurity.get().getHostAddress())
			+ "\n\nIn welchem Thread: " + decorate(threadName)
			       + " in der Gruppe: " + decorate(threadGroupName)
			+ "\n\nStacktrace von innen (Ursache) nach Aussen (Folgen): \n"
			+ stackTrace
			+ appInitMsg
		  ;
		return msg;
	}
	private static String decorate(final String decoree) {return "\"" + decoree + "\"";}
	private static void uncaughtExceptionShowMsgBox(final String msg) {
		try {
			final Component parentComp  = null;
			final int       messageType = JOptionPane.ERROR_MESSAGE;
			final String    title       = Main.getAppName();
			JOptionPane.showMessageDialog(parentComp, msg, title, messageType);
		} catch (final Exception ex) {
			Logger.SWFW.fatal("Exception in Exception Handling of Uncaught Exception (2).", ex);
		}
	}
}
