/**
 * 
 * Last changed: $Date$
 * By: $Author$
 * URL: $HeadURL$
 * $Revision$ 
 *
 */
package de.desy.mst.libs.framework.mstapp.appinf;

import java.lang.reflect.Constructor;

import de.desy.mst.libs.framework.mstapp.appinf.client.ClientFactory;
import de.desy.mst.libs.framework.mstapp.appinf.ide.DebugFactory;
import de.desy.mst.libs.framework.mstapp.appinf.server.ServerFactory;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * @author labudda
 *
 */
public enum ApplikationKind {
	Konsole(new ClientFactory()),
	Server(new ServerFactory()),
	Ide(new DebugFactory()),
	Automatic;
	private static class NoFactory extends AApplicationInterfaceFactory {

		/* (non-Javadoc)
		 * @see de.desy.mst.libs.IApplikationInterfaceFactory#getApplicationInformation()
		 */
		@Override public IApp getApplicationInformation() throws IllegalStateException {
			throw new IllegalStateException("There's no factory defined");
		}

		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.appinf.AApplicationInterfaceFactory#areKindSpecificRequirementsFulfilled()
		 */
		@Override
		protected boolean areKindSpecificRequirementsFulfilled() {
			return false;
		}
		
	}
	
	/**
	 * Welche factory erzeugt die Applikationsinformationen 
	 */
	private final IApplikationInterfaceFactory factory;
	/**
	 * Default factory, nur relevant fuer Automatic Kind.
	 * Insbesondere fuer Debug ist es wichtig die automatsche Auswahl zu beeinflussen.
	 * Dafuer kann die Property 'ApplicationKindFactory' definiert werden.
	 * Der Wert der Property entspricht dem Klassennamen der factory
	 */
	private ApplikationKind(){
		if (CustomSystemProperty.ApplicationKindFactory.isAvailable()) {
			IApplikationInterfaceFactory definedApplicationKinfFactory = null;
			try {
				final Class<?> definedClass = Class.forName(CustomSystemProperty.ApplicationKindFactory.getValue());
				final Class<?>[] types = null;
				final Object[] arguments = null;
				final Constructor<?> constructor = definedClass.getConstructor(types);
				definedApplicationKinfFactory = (IApplikationInterfaceFactory) constructor.newInstance(arguments);
			} catch (final Exception e) {
				/* Weil diese Option i.a. nur fuer debug releavant ist,
				 * darf diese Exception keinen negativen Einfluss auf den Kontrollraum haben
				 */ 
				Logger.SWApp.warn("Class defined by 'ApplicationKindFactory' not found :" + CustomSystemProperty.ApplicationKindFactory.getValue());
			}
			this.factory = definedApplicationKinfFactory;
		} else {
			this.factory = null;
		}
	}
	/**
	 * An application kind with information factory
	 * @param factory
	 */
	private ApplikationKind(final IApplikationInterfaceFactory factory) {
		this.factory = factory;
	}
	
	/**
	 * @return factory die "ihre" Umgebung erkennt
	 */
	private IApplikationInterfaceFactory getFactory() {
		Logger.SWFW.debug("Enter: IApplikationInterfaceFactory getFactory()");
		IApplikationInterfaceFactory validFactory = this.factory;	// nur bei Automatic ist die factory nicht definiert
		int checkFactoryIndex = ApplikationKind.values().length - 1;
		while(checkFactoryIndex >= 0 && validFactory == null) {
			final ApplikationKind checkKind = ApplikationKind.values()[checkFactoryIndex]; 
			if (checkKind.factory == null) {
				// ignore
				checkFactoryIndex--;
			} else if (checkKind.factory.isValidKind()) {
				validFactory = checkKind.factory;
				checkFactoryIndex = -1;
			} else {
				checkFactoryIndex--;
			}
		}
		if (validFactory == null) {
			Logger.SWFW.warn("No valid factory found");
			return new NoFactory();
		} else {
			Logger.SWFW.debug(validFactory.getClass().getName() +  " defined as Information factory");
			return validFactory;
		}
	}
	/**
	 * @return the factory
	 * @throws IllegalStateException
	 * @see de.desy.mst.libs.framework.mstapp.appinf.IApplikationInterfaceFactory#getApplicationInformation()
	 */
	public IApp getApplicationInformation() throws IllegalStateException {
		Logger.SWFW.debug("Enter: IApp ApplikationKind:getApplicationInformation()");
		return this.getFactory().getApplicationInformation();
	}
	/**
	 * @return true on valid for this kind of applikation
	 * @see de.desy.mst.libs.framework.mstapp.appinf.IApplikationInterfaceFactory#isValidKind()
	 */
	public boolean isValidKind() {
		return this.getFactory().isValidKind();
	}
	
}
