/**
 *
 *  Feb 25, 2010 - Feb 25, 2010	ws: fixed one warning: Class -> Class<?>
 *
 * Last changed: $Date$
 * By: $Author$
 * URL: $HeadURL$
 * $Revision$
 *
 */
package de.desy.mst.libs.framework.mstapp.appinf.client;

import java.lang.reflect.InvocationTargetException;

import de.desy.mst.libs.framework.mstapp.logging.Logger;


/**
 * @author labudda
 */
class Delegated implements IDelegated {
	/**
	 * Objekt , an das der Aufruf deligiert wird
	 */
	private final Object delegateTo;
	/**
	 * Klasse, in der nach der Methode gesucht wird
	 * Nur wenn nach einer Klassenmethode gesucht wird, ist die Klasse identisch zum Objekt
	 */
	private final Class<?> delegatedClass;
	private final Exception error;
	Delegated(final Object delegateTo) {
		assert delegateTo != null:"Object to delegate to must not be null";
		this.delegateTo = delegateTo;
		this.delegatedClass = delegateTo.getClass();
		this.error = null;
	}

	Delegated(final Class<?> delegatedClass) {
		assert delegatedClass != null:"Object to delegate to must not be null";
		this.delegateTo = delegatedClass;
		this.delegatedClass = delegatedClass;
		this.error = null;
	}

	Delegated(final Exception classNotFound) {
		this.delegateTo = null;
		this.delegatedClass = null;
		this.error = classNotFound;
	}

	/**
	 * Invokes a factory method to create delegated
	 * @param nameOfClass factory class
	 * @param factoryMethod name of factoty method
	 */
	public Delegated(final String nameOfClass, final String factoryMethod) {
		Object theDelegated;
		Class<?> theDelegatedClass;
		Exception exception;
//		this.debug("Look for class " + nameOfClass);
		try {
			final Class<?> theClass;
			theClass = Class.forName(nameOfClass);
//			this.debug("Found " + theClass.getCanonicalName());
/*			for (java.lang.reflect.Method method: theClass.getMethods()) {
				this.debug(method.getName());
			}
*/			exception = null;
			final Class<?>[] parameter = null;
			final java.lang.reflect.Method getInstance = theClass.getMethod("getInstance", parameter);
			theDelegated = getInstance.invoke(theClass, (Object[])null);
			theDelegatedClass = theDelegated.getClass();
		} catch (final Exception e) {
			theDelegated = null;
			theDelegatedClass = null;
			exception = e;
			Logger.SWFW.warn("Catched constructors exception", e);
		}
		this.delegateTo = theDelegated;
		this.delegatedClass = theDelegatedClass;
		this.error = exception;
	}


	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IDelegated#isInformationAvailable()
	 */
	@Override public boolean isInformationAvailable() {
		return this.error == null;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IDelegated#getFailureReason()
	 */
	@Override public Exception getFailureReason(){
		return this.error;
	}


	/**
	 * Executes a method of deletegated object
	 * the mehod hat no parameters
	 * @param nameOfMethod
	 * @return Object of result
	 * @throws InvocationTargetException
	 * @throws IllegalArgumentException
	 * @throws Exception
	 */
	protected Object getDelegatedResult(final String nameOfMethod) throws UnsupportedOperationException, IllegalArgumentException {
		final Object[] parameter = null;
		return this.getDelegatedResult(nameOfMethod, parameter);
	}

	/**
	 * @param nameOfMethod
	 * @param url
	 * @return result of method invocation
	 */
	protected Object getDelegatedResult(final String nameOfMethod, final java.net.URL url) throws UnsupportedOperationException, IllegalArgumentException {
		final Object[] parameter = {url};
		return this.getDelegatedResult(nameOfMethod, parameter);
	}

	/**
	 * Call a method of delegated
	 * @param nameOfMethod
	 * @param parameter
	 * @return result of method invocation
	 * @throws UnsupportedOperationException
	 * @throws IllegalArgumentException
	 */
	protected Object getDelegatedResult(final String nameOfMethod, final Object[] parameter) throws UnsupportedOperationException, IllegalArgumentException {
		final Class<?>[] parameterKind;
		if (parameter == null) {
			parameterKind = null;
		} else {
			parameterKind = new Class[parameter.length];
			for (int parameterIndex = 0; parameterIndex < parameter.length; parameterIndex++) {
				parameterKind[parameterIndex] = parameter[parameterIndex].getClass();
			}
		}
		return this.getDelegatedResult(nameOfMethod, parameterKind, parameter);
	}

	/**
	 * @param nameOfMethod
	 * @param parameterKind
	 * @param parameter
	 * @return  result of method invocation
	 * @throws UnsupportedOperationException
	 * @throws IllegalArgumentException
	 */
	protected Object getDelegatedResult(final String nameOfMethod, final Class<?>[] parameterKind, final Object[] parameter) throws UnsupportedOperationException, IllegalArgumentException {
		if (this.isInformationAvailable()) {
			try {
				final java.lang.reflect.Method method = this.delegatedClass.getMethod(nameOfMethod, parameterKind);
				return method.invoke(this.delegateTo, parameter);
			} catch (final NoSuchMethodException e) {
				this.listMethods();
				e.printStackTrace();
				throw new UnsupportedOperationException(nameOfMethod + " is not supported by " + this.delegateTo.getClass().getCanonicalName(), e);
			} catch (final IllegalAccessException e) {
				this.listMethods();
				throw new UnsupportedOperationException(nameOfMethod + " is not accessible by " + this.delegateTo.getClass().getCanonicalName(), e);
			} catch (final InvocationTargetException e) {
				this.listMethods();
				throw new UnsupportedOperationException(nameOfMethod + " of " + this.delegateTo.getClass().getCanonicalName() + " fails with error", e);
			}
		} else {
			throw new IllegalStateException("There's no valid delegate.", this.getFailureReason());
		}
	}

	/**
	 * Just to turn of/on debug output centraly
	 * @param output
	 */
	protected void debug(final Object output) {
		System.out.println(output.toString());
	}


	private void listMethods(){
		this.debug("+++ Available methods +++");
		for(final java.lang.reflect.Method method: this.delegateTo.getClass().getMethods()) {
			this.debug(method.getName());
			this.debug("--- parameter ---");
			for (final Class<?> type:method.getParameterTypes()){
				this.debug(type.getCanonicalName());
			}
		}

	}
	/**
	 * Just convinience method
	 * @param nameOfMethod
	 * @param value
	 * @return
	 * @throws UnsupportedOperationException
	 * @throws IllegalArgumentException
	 */
	protected Object getDelegatedResult(final String nameOfMethod, final int value) throws UnsupportedOperationException, IllegalArgumentException {
		final Object[] parameter = {value};
		final Class<?>[] parameterKind = {Integer.TYPE};
		return this.getDelegatedResult(nameOfMethod, parameterKind, parameter);
	}

}
