/**
 * ws:	Mar 30, 2007 - Apr 17, 2007 creation from scratch
 * ws:	Apr 11, 2008 - Apr 11, 2008 appName und subSystem -> .toLowerCase()
 * ws:	Apr 18, 2008 - Apr 18, 2008 added: getAboutBoxHTTP(...)
 * ws:	Jun 20, 2008 - Jun 20, 2008	 about jetzt von Raimund in html
 * ws:	Aug  7, 2008 - Aug  7, 2008	changed: getDateiPfad to shortFrameName statt appName
 * Aug 15, 2008 - Aug 15, 2008	ws: getAppOrALName() impl
 */
package de.desy.mst.libs.framework.mstapp.hilfe;

import java.util.HashMap;
import java.util.Map;

import de.desy.mst.libs.framework.mstapp.ApplicationInfo;

/**
 * Pfad zum Hilfesystem. Package privat
 * @author ws
 * version 1.0.0
 */
class PfadeHilfeSystem extends AHilfePfade implements IHilfePfadeEx{
//SUBSYSTEM	"http://doc.desy.de/DOC/show.jsp?dir=/doc/DESY2/applications/elektronenstrahlkontrolle";
//DATEI    	"http://doc.desy.de/DOC/show.jsp?dir=/doc/DESY2/applications/elektronenstrahlkontrolle/d2energien";
//ROOT     	"http://doc.desy.de/DOC/index.jsp";
//getAboutBoxHTTP "http://doc.desy.de/elog/servlet/XMLSingle?file=/DOC/data/doc/common/applications/info/mstlogviewer/about.xml&tag=text"

	private static final String ROOT   = "http://doc.desy.de/DOC/";
	private static final String PREFIX = "http://doc.desy.de/DOC/show.jsp?dir=/doc/";
	private static final String  INFIX = "applications/";

	private static final String ABOUT_PREFIX  = "http://doc.desy.de/elog/servlet/XMLlist?file=/DOC/data/doc/";
	private static final String ABOUT_POSTFIX = "/about.xml&xsl=/elogbook/xsl/elog-single-entry.xsl";

	private static final Map<String, String>  accelaratorsCorrectCase;

	static{
		accelaratorsCorrectCase = new HashMap<String, String>();
		final String [] accArray = new String[] {"XFEL", "PETRA", "LINAC2"
										, "FLASH", "DORIS", "DESY2", "common"};
		for (final String value : accArray) {
			final String key = value.toLowerCase();
			accelaratorsCorrectCase.put(key, value);
		}
	}

//=============================================================================

	public PfadeHilfeSystem(final ApplicationInfo applicationInfo) {
		super(applicationInfo);
	}

//=============================================================================


	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.hilfe.AHilfePfade#getAppOrALName(de.desy.mst.libs.framework.mstapp.ApplicationInfo)
	 */
	@Override public String getAppOrALName(final ApplicationInfo applicationInfo) {
		return applicationInfo.getShortFrameName();
	}


	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.hilfe.IHilfePfade#getDateiPfad(java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override public String getDateiPfad(final String accelerator, final String subSystem, final String appName) {
		return getSubsystemVerzeichnis(accelerator, subSystem)
			 + cleanName(appName).toLowerCase()
			 ;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.hilfe.IHilfePfade#getSubsystemVerzeichnis(java.lang.String, java.lang.String)
	 */
	@Override public String getSubsystemVerzeichnis(final String accelerator, final String subSystem) {
		return PREFIX
			 + correctAcceleratorCase(accelerator) + "/"
			 + INFIX
			 + subSystem.toLowerCase() + "/"
			 ;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.hilfe.IHilfePfade#getSystemRoot()
	 */
	@Override public String getSystemRoot() {return ROOT;}

	@Override public String getAboutBoxHTTP(final String accelerator, final String subSystem, final String appName) {
		return ABOUT_PREFIX
		 	 + correctAcceleratorCase(accelerator) + "/"
		 	 + INFIX
		 	 + subSystem.toLowerCase() + "/"
		 	 + cleanName(appName).toLowerCase()
		 	 + ABOUT_POSTFIX
		 	 ;
	}

//=============================================================================

	private String correctAcceleratorCase(final String accelerator) {
		final String key = accelerator;
		if(accelaratorsCorrectCase.containsKey(key)){
			return accelaratorsCorrectCase.get(key);
		}else{
			return key;
		}
	}
}
