/*
 * Created on 	Apr 26, 2005 - Mai 25, 2005
 *				Aug 16, 2005 - Aug 19, 2005	XTor via ABaseDataOfMstIni und nicht AppName
 *				Dec 16, 2005 - Dec 22, 2005	Support Maschine
 *	Dec 10, 2008 - Dec 10, 2008	ws:	MstSecurityMgr -> MstSecurity
 * 	Jul 14, 2009 - Jul 14, 2009	ws:	fixed some Eclipse 3.5 warnings
 */
package de.desy.mst.libs.framework.mstapp.ini;

import java.text.DateFormat;
import java.util.Date;

import de.desy.mst.libs.framework.mstapp.security.MstSecurity;

/**
 * Ort fr alle Meta Infos zum aktuellen IniFile
 * @author schuette
 * @version 1.0.0
 */
public class FileDataIni implements IIniFileObserver{
    private static final String BAD_APPNAME = "Should never appear, this Constructor is for the XMLDecoder only.";
    private static final int    BAD_MASCHNR = -84322;	//entspricht lediglich nicht initialisiert
	private long   versionUID;
	private String appName;
    private String fileTyp;

	private Date   date;
	private String dateString;
	private String pc;
	private String ipAddress;
	private String user;
    private String localPath;
    private String webAddress;
    private int    maschineNr = BAD_MASCHNR;	//initialised to special, but illegal value! to spot app developer errors

    transient private boolean doInitialization = false;

//=============================================================================

	/**
	 * @param versionUID
	 * @param baseDataOfMstIni
	 * @throws NullPointerException
	 */
	public FileDataIni(final long versionUID, final ABaseDataOfMstIni baseDataOfMstIni) {
		setVersionUID(versionUID);
    	setAppName(baseDataOfMstIni.getAppName());
    	setFileTyp(baseDataOfMstIni.getFileTyp());
    	setLocalPath(baseDataOfMstIni.getFileName());
    	setWebAddress(baseDataOfMstIni.getWebAddress());
        this.doInitialization = true;
        onPrepareForExport(); //calculates good start values
	}


    /**
     * @deprecated DO NOT USE, it is only for the XMLDecoder.
     */
    @Deprecated public FileDataIni() {
        //this(-999, BAD_APPNAME); can not be called due to " doInitialization = true;" statement
        this.setVersionUID(-999);
        this.setAppName(BAD_APPNAME);
        this.doInitialization = false;
    }
//=============================================================================
//    public static FileDataIni createDefaultObject(){
//        FileDataIni retVal = new FileDataIni(-1, BAD_APPNAME);
//            retVal.onPrepareForExport(); //sets the rest
//        return retVal;
//    }


//  =============================================================================

	/* (non-Javadoc)
     * @see de.desy.mst.mstapp.ini.IIniFileObserver#getDefaultIniBaumRoot()
     */
    @Override public Object getDefaultIniBaumRoot() {
        return null;
    }
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.ini.IPrepareForExport#onPrepareForExport()
     * Not for public use, so keep of
	 */
    @Override public Object onPrepareForExport() {
        if(this.doInitialization){
            //assert(getAppName() != BAD_APPNAME)
            setDate(new Date());
            setDateString(DateFormat.getDateTimeInstance().format(this.date));
            final MstSecurity mstSecurity = MstSecurity.get();
			setPc(mstSecurity.getHostName());
            setIpAddress(mstSecurity.getHostAddress());
            setUser(mstSecurity.getUser());
        }
        return this;
    }
    /* (non-Javadoc)
     * @see de.desy.mst.mstapp.ini.IIniFileObserver#onNewIniAvailable(java.lang.Object)
     */
    @Override public void onNewIniAvailable(final Object iniBaumRoot) {}

//	=============================================================================

    public final String getAppName() 					{return this.appName;}
    public final void setAppName(final String appName)		{this.appName = appName;}

    public final Date getDate()							{return this.date;}
    public final void setDate(final Date date)				{this.date = date;}

    public final String getDateString()				   	{return this.dateString;}
    public final void setDateString(final String dateString) 	{this.dateString = dateString;}

    public final String getIpAddress()					{return this.ipAddress;}
    public final void setIpAddress(final String ipAddress)	{this.ipAddress = ipAddress;}

    public final String getLocalPath()					{return this.localPath;}
    public final void setLocalPath(final String localPath)	{this.localPath = localPath;}

    public final String getPc()		   					{return this.pc;}
    public final void setPc(final String pc)					{this.pc = pc;}

    public final String getUser() 						{return this.user;}
    public final void setUser(final String user) 				{this.user = user;}

    public final long getVersionUID() 					{return this.versionUID;}
    public final void setVersionUID(final long versionUID) 	{this.versionUID = versionUID;}

    public final String getWebAddress() 				{return this.webAddress;}
    public final void setWebAddress(final String webAddress) 	{this.webAddress = webAddress;}

    public final String getFileTyp() 					{return this.fileTyp;}
    public final void setFileTyp(final String fileTyp) 		{this.fileTyp = fileTyp;
    }
    /**not checked for validity*/
	public final int getMaschinenNr() 					{return this.maschineNr;}
    /**not checked for validity*/
	public final void setMaschinenNr(final int maschineNr) 	{this.maschineNr = maschineNr;}


	public boolean istInitialisiertMaschinenNr()	{return this.getMaschinenNr() != BAD_MASCHNR;}
    public boolean insSeinenWertenGleich(final FileDataIni rhs) {
    	if(this == rhs){return true;}

    	return	this.appName	.equals(rhs.appName)
		    &&	this.fileTyp	.equals(rhs.fileTyp)
		    &&	this.date		.equals(rhs.date)
		    &&	this.dateString	.equals(rhs.dateString)
		    &&	this.pc			.equals(rhs.pc)
		    &&	this.ipAddress	.equals(rhs.ipAddress)
		    &&	this.user		.equals(rhs.user)
		    &&	this.localPath	.equals(rhs.localPath )
		    &&	this.webAddress	.equals(rhs.webAddress);
    }
}
