/*
 * Created on 	May 26, 2005 - Jun 27, 2005
 * Aug 19, 2005 - Aug 22, 2005 support for general IIniFileRoot not only MstIni, String Backup
 * Oct 25, 2005 - Oct 27, 2005	new logging system
 * Nov 11, 2005 - Nov 18, 2005	Web IO Exception 400 -> no IniFile, extract constants
 * Feb 15, 2006 - Feb 24, 2006	file,web InputStream -> String -> XMLDecoder (against long file Problem)
 * Jun 19, 2007 - ...			ws:	Lokalisierung auf Deutsch etwas nachgezogen
 * Jul 03, 2007 - Jul 04, 2007	ws: added: writeToConsole(...),
 * 									added: getName(), hasWebIni()
 * May  6, 2008 - May 15, 2008	ws: added: statistics on IO, web lesen performa.
 * Aug  6, 2008 - Aug  6, 2008	ws:	timeout 3000ms -> 5000ms (KH Problem)
 * Jan  8, 2010 - Jan  8, 2010	ws: due to strange MaWa once error protect tryBackupWebToFile(...) stronger
 * Jun 10, 2015 - Jun 15, 2015	ws:	track web accesses by name via LinkedMap accessedWebAdresses
 * Jun 24, 2015 - Jun 24, 2015	ws:	extracted write(IIniFileRoot) from write()	
 * Jun 25, 2015 - Jun 25, 2015	ws:	added delete() and added return value to delete(fileName)
 * Aug 14, 2015 - Aug 14, 2015	ws:	changed readWebByName(...) catch potential error on stream closing
 * 	
 */

package de.desy.mst.libs.framework.mstapp.ini;

import static de.desy.mst.libs.framework.mstapp.logging.LogUtil.timedMsg;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import de.desy.mst.libs.framework.mstapp.Main;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.messages.ISetMessage;
import de.desy.mst.libs.framework.mstapp.messages.Message;
import de.desy.mst.libs.framework.mstapp.messages.MessageMgr;
import de.desy.mst.libs.framework.mstapp.util.threads.Sleep;

/**
 * One IniFileIOMgr for each individual file
 * Reads from Web/LocalFile
 * Writes to LocalFile
 * in compliance with PC Settings and the ABaseData
 * Typical uses:
 * App    data
 * User   data
 * System data
 *
 * Reading: tries first web (if address in ABaseData), then file
 * Writing: only to file (if allowed)
 *
 * @author ws
 * @version 1.0.0
 */
class IniFileIOMgr{
	static final int DEFAULT_TIMEOUT_MS = 5000;

	private static final String WRITE = "write";
	private static final String READ = "read";

	private static final String BADINIFILE      = "SchlechteIniDatei";
    private static final String NOINIFILE       = "KeineIniDatei";

    private static final String NOWEBADDRESS    = "KeineWebAddresse";
//    private static final String BADWEBADDRESS   = "SchlechteWebAddresse";
//    private static final String NOWEBSERVER     = "KeinWebServer";

    private static final String CACHEDINIFILE   = "PufferIniDatei";

	private static final String SQLXML_OR_SQL_SERVER_PROBLEM = "SQLXML or SQL Server Problem: '";
	
	private static final List<String> accessedWebAdresses = 
			new AccessedWebAdresses(IniFileIOMgr.class.getSimpleName());
			

//=============================================================================

    public static final List<String> getAccessedWebAdresses() {
    	return Collections.unmodifiableList(accessedWebAdresses) ;
    }
    static final ExecutorService getExecutorService() {
		return Main.getExecutorService();
	}

//=============================================================================

    private final ABaseData<? extends IIniFileRoot> baseData;
    private final IniFileObserverMgr observerMgr;
    private final MessageMgr messageMgr = new MessageMgr();

    private Future<Object> future = null;	//nacktes ungeprueftes IniFileRoot

//=============================================================================

    /**
     * @param fileName
     * @param webAddress
     */
    IniFileIOMgr(final ABaseData<? extends IIniFileRoot> baseData, final ISetMessage messageRcv) {
        //Logger.SWFW.info("Created new IniFileIOMgr " + baseData.getName());
        this.baseData = baseData;
        this.observerMgr = new IniFileObserverMgr(getBaseData());
        getMessageMgr().add(messageRcv);
    	if(Main.mstwebUsageSettings.isUse_mstweb4ini()) 
    		launchWebReaderCallable();
    	else if(getBaseData().hasWebAddress())
    		Logger.SWFW_.info(this.getClass().getSimpleName() 
    				       + ".XTor() launchWebReaderCallable() ignored"
    				       + ", since Main.mstwebUsageSettings.isUse_mstweb4ini() == "
    				       + Main.mstwebUsageSettings.isUse_mstweb4ini()
    				       +". The address would have been: '" + getBaseData().calcWebAddress() 
    				       + "'" );
    }

// =============================================================================

    public final void write() {
    	final IIniFileRoot iniFileRoot = getIniFileRoot();
        write(iniFileRoot);
    }
    public void write(final IIniFileRoot iniFileRoot) {
    	checkUserOkForWriting();//@throws IllegalStateException

    	final String fileName = getBaseData().getFileName();
    	createAllParentDirs(fileName);
    	delete(fileName);
    	writeFile(iniFileRoot, fileName);
    }
    public final boolean delete() {
    	return delete(getBaseData().getFileName());
    }
	public void writeToConsole() {
		final String text = this.getIniBaumRootAsText();
		System.out.println(text);
	}

	public final IIniFileRoot read(){return read(DEFAULT_TIMEOUT_MS);}
	public final IIniFileRoot read(final long timeoutMS) {
		if(getBaseData().hasWebAddress()) {
			if( Main.mstwebUsageSettings.isUse_mstweb4ini()
					|| getBaseData().getClass() == BaseDataForAnApplication.class) {
				// Versuch die Daten vom Webserver zu lesen
				if(!Main.mstwebUsageSettings.isUse_mstweb4ini())
					Logger.SWFW.info(this.getClass().getSimpleName() + ".read(...) try to read web for wst compatibility.");
				final IIniFileRoot ini = readWeb(timeoutMS);
				//good webreads are locally buffered
				if(getMessageMgr().wasASuccess()){
					tryBackupWebToFile();
					return ini;
				}
			} else if(getBaseData().getClass() == BaseDataForTabelle.class) {
				// Versuch Daten von resource zu lesen
				final String tabID = getBaseData().getName();
				Logger.SWFW.info(this.getClass().getSimpleName() + ".read(...) read TableData '" + tabID + "' from Resource.");
				final IIniFileRoot ini = readResource(tabID);
				//good webreads are locally buffered
				if(getMessageMgr().wasASuccess())
					return ini;
				else
					Logger.SWFW.error("Could not read from resource: '" + tabID + "'");
				;
			}
		}
		return readFile();
	}        
	
	private IIniFileRoot readResource(final String tabID) {
		final String PATH = "/resources/tables/" + tabID + ".xml";
	    final InputStream is = getClass().getResourceAsStream(PATH);
	    if (is == null)  
	    	return null;
	    
	    try {
	    	return (IIniFileRoot) readObjektBaumFromXML(new BufferedInputStream(is));
	    } catch (IOException io) {
	    	return null;
	    } finally {
	    	try {
	    		is.close();
	    	} catch (IOException e) {
	    		/* Wenn nur durch close eine Exception generiert wurde, kann sie ignoriert werden
	    		 * (Ist das ueberhaupt moeglich?) sollte jedoch ggf. im Log erscheinen.
	    		 * Ist diese Exception eine folge einer zuvor gefangenen Exception,
	    		 * habe ich keine Idee was damit sinvolles anzufangen ist
	    		 */

	    		e.printStackTrace();
	    	}
	    }
	}
	
	public final String getIniBaumRootAsText() {
        final ByteArrayOutputStream arrayOutputStream = new ByteArrayOutputStream();
        writeOutputStreamInXML(getIniFileRoot(), arrayOutputStream);
        return arrayOutputStream.toString();
    }
	public String getName() 	{return getBaseData().getName();}
	public boolean hasWebIni()	{return getBaseData().hasWebAddress();}

    //for debugging only
    final String getIniBaumRootAsTextWithOutAnyUpdates() {
        final ByteArrayOutputStream arrayOutputStream = new ByteArrayOutputStream();
        final XMLEncoder xmlEncoder = new XMLEncoder(arrayOutputStream);
        xmlEncoder.writeObject(getIniFileRoot());
        xmlEncoder.close();
        return arrayOutputStream.toString();
    }

    final MessageMgr getMessageMgr() {return this.messageMgr;}
    final IniFileObserverMgr getObserverMgr() {return this.observerMgr;}
    final ABaseData<? extends IIniFileRoot> getBaseData() {return this.baseData;}

//=============================================================================
    private final void setFuture(final Future<Object> inFut) 	{this.future = inFut;}
    final Future<Object> getFuture() 					{return this.future;}
	private final Future<Object> getAndClearFuture() {
		final Future<Object> future = getFuture();
		if(future == null)
			return null;
		setFuture(null);
		return future;
	}

	//replaces only non null futures
    private void launchWebReaderCallable() {
    	if(getFuture() != null)
    		return;					//nothing to launch, leave future at original
    	if(!getBaseData().hasWebAddress())
    		return;		 			//nothing to launch, leave future at null
    	if(!getBaseData().hasWebAddress())
    		return;					//nothing to launch, leave future at null

    	setFuture(
			getExecutorService().submit(
				new Callable<Object>(){
					@Override public Object call()  throws IOException {
						return readWebByName(getBaseData().getWebAddress());
					}
				})
    	);
    }

    /** @throws IllegalStateException, if not isOkForWriting */
    private final void checkUserOkForWriting() {
        if(!isUserOkForWriting()){
            throw new IllegalStateException("User does not want data to be stored. "
                    + "You may reconfigure it via the Configurations menu.");
        }
    }

    private final boolean isUserOkForWriting() {
        return getBaseData().isOkForWriting();
    }

    private final void createAllParentDirs(final String fileName) {
        //VORSICHT nicht die local home routine verwenden, da erstmal einen
        //Schritt hochgehen
        final File file = new File(fileName);
        final File parentDir = file.getParentFile();
        if (!parentDir.exists()) {
            parentDir.mkdirs(); // das zugehoerige Verzeichnis koennte auch
            // nicht ex.
        }
    }

    private final boolean delete(final String fileName) {
        //IDEE Maybe better to work with a backup copy mechanism
        final File file = new File(fileName);
        if(file.exists())
        	return file.delete();
        return false;
    }

    private final void writeFile(final Object baumRoot, final String iniFileName){
        OutputStream os;
        try {
            os = new BufferedOutputStream(new FileOutputStream(iniFileName));
            writeOutputStreamInXML(baumRoot, os);
            os.close();
        } catch (final FileNotFoundException ex) {
            handleNoIniFile(iniFileName, WRITE);
        }
        catch (final IOException ex) {
            handleIOException(iniFileName, WRITE, ex);
        }
    }
    private final void writeOutputStreamInXML(final Object baumRoot, final OutputStream os) {
        getObserverMgr().fireOnPrepareForExport();

        final XMLEncoder xmlEncoder = new XMLEncoder(os);
        xmlEncoder.writeObject(baumRoot);
        xmlEncoder.close();
    }

    private final IIniFileRoot getIniFileRoot() {
        return getObserverMgr().getFileRoot();
    }

    /**Package Privat nur zum Testen, ansonsten PRIVATE*/
    final IIniFileRoot readFile() {
        if (!getBaseData().hasFileName())
        	return handleNoFileName();
        final long _start = System.currentTimeMillis();
    		final IIniFileRoot result = readFile(getBaseData().getFileName());
    	logIODauer(getBaseData().getFileName(), _start);
		return result;
    }

	private IIniFileRoot readFile(final String fileName) {
		Object iniCandidate;
        try {iniCandidate = readFileByName(fileName);}
        catch (final FileNotFoundException ex) {return handleNoIniFile(fileName, READ);}
        catch (final IOException ex)           {return handleIOException(fileName, READ, ex);}
        catch (final NullPointerException  ex) {return handleNullPointerException(fileName);}

        return processIniCandidate(iniCandidate, fileName, true);
	}
    private final Object readFileByName(final String fileName) throws FileNotFoundException, IOException {
    	final File file = new File(fileName);
        final InputStream is = new java.io.FileInputStream(file);
        final Object retVal = this.readObjektBaumFromXML(new BufferedInputStream(is));
        is.close();
        return retVal;
    }
    /**Package Privat nur zum Testen, ansonsten PRIVATE*/
    final IIniFileRoot readWeb() {return readWeb(DEFAULT_TIMEOUT_MS);}
    /**Package Privat nur zum Testen, ansonsten PRIVATE*/
    final IIniFileRoot readWeb(final long timeoutMS) {
        if (!getBaseData().hasWebAddress()) {return handleNoWebAddress();}
        final String webAddress = getBaseData().getWebAddress();

        final long _start = System.currentTimeMillis();
        	final IIniFileRoot result = readWeb(webAddress, timeoutMS);
        logIODauer(webAddress, _start);
		return result;
    }
	private IIniFileRoot readWeb(final String webAddress, final long timeoutMS) {
//		boolean isReachable;
//        try {isReachable = getBaseData().isWebSrvReachable();}
//        catch (MalformedURLException ex) {return handleBadWebAddress(webAddress);}
//        if(!isReachable){return handleWebSrvNotReachable(webAddress);}

		launchWebReaderCallable();	//in case future is null
		final Future<Object> future = getAndClearFuture();//spaeter frisches
		if(future == null)
			return handleNoFuture(webAddress);

		Object iniCandidate;
        try {iniCandidate = readWeb(future, timeoutMS);}
        catch (final FileNotFoundException ex) {return handleNoIniFile  (webAddress, READ);}
        catch (final IOException 			 ex) {return handleIOException(webAddress, READ, ex);}
        catch (final NullPointerException  ex) {return handleNullPointerException(webAddress);}
        catch (final Exception  			 ex) {return handleException  (webAddress, READ, ex);}

        return processIniCandidate(iniCandidate, webAddress, false);
	}

	private Object readWeb(final Future<Object> future, final long timeoutMS )
			throws InterruptedException, TimeoutException, ExecutionException
				 , FileNotFoundException, IOException, NullPointerException
			{
			try {
				return future.get(timeoutMS, TimeUnit.MILLISECONDS);
			} catch (final ExecutionException e) {
				final Throwable cause = e.getCause();
				if(cause instanceof FileNotFoundException)	throw (FileNotFoundException)cause;
				if(cause instanceof IOException) 			throw (IOException)cause;
				if(cause instanceof NullPointerException) 	throw (NullPointerException)cause;
				throw e;
			}
	}

	private void logIODauer(final String webAddress, final long _start) {
		final String msg = "Dauer der IO von '" + webAddress + "'.";
        Logger.SWFW_.info(timedMsg(_start, msg));
	}
    private final Object readWebByName(final String webAddress) throws IOException {
        accessedWebAdresses.add(webAddress);
        final URL url = new URL(webAddress);
        final URLConnection con = url.openConnection();
        final InputStream is = con.getInputStream();
		//if(con instanceof HttpURLConnection){
		//	HttpURLConnection httpCon = (HttpURLConnection) con;
		//    Logger.SWFW.debug("WebAddress:      " + webAddress);
		//    Logger.SWFW.debug("ResponseCode:    " + httpCon.getResponseCode());
		//    Logger.SWFW.debug("ResponseMessage: " + httpCon.getResponseMessage());
		//}
        final TextInputStreamUtility streamUtility = new TextInputStreamUtility(is);
        //Logger.SWFW.debug("Contents:      	\n" + streamUtility.getContentsAsString());
        checkXML(streamUtility.getContentsAsString());	//@throws IOException
        final InputStream inMemoryIS = streamUtility.getContentsAsInputStream();
            final Object retVal = this.readObjektBaumFromXML(new BufferedInputStream(inMemoryIS));
        try {is.close();} 
        catch (final Exception e) {
			final String msg = this.getClass().getSimpleName() 
					         + ".readWebByName(" + webAddress + ")"
					         + " stream can not be closed. Will be ignored.";
			Logger.SWFW.warn(msg);
		}
			//Logger.SWFW.debug("=====" + webAddress);
			//Map <String, List<String>> headerFields = con.getHeaderFields();
			//for (String key : headerFields.keySet()) {
			//	Logger.SWFW.debug("(key, field): (" + key + ", " + headerFields.get(key) + ")");
			//}
        return retVal;
    }

	private void checkXML(final String string) throws IOException{
		if(string == null || string.length() == 0) 	{throw new IOException("Empty Result.");}
		if(string.contains("MSSQLError"))			{throw new IOException(SQLXML_OR_SQL_SERVER_PROBLEM + string + "'");}
		//final String trimmedString = string.replace(" ", "").replace("\n", "").replace("\t", "");
		if(!string.contains("</java>"))				{throw new IOException(SQLXML_OR_SQL_SERVER_PROBLEM + string + "'");}
	}

	private final Object readObjektBaumFromXML(final InputStream is) throws IOException {
        final XMLDecoder xmlDecoder = new XMLDecoder(is);
        Object iniCandidate;

        try {iniCandidate = xmlDecoder.readObject();}
        catch (final Exception e) {throw new IOException(e.toString());}
        finally{xmlDecoder.close();}

        if(iniCandidate instanceof IIniFileRoot){
            getObserverMgr().fireOnNewIniAvailable((IIniFileRoot)iniCandidate);
        }

        return iniCandidate;
    }

		private final void tryBackupWebToFile() {
		if(isUserOkForWriting()) {
			new Thread( new Runnable(){
				@Override public void run() {
					Sleep.sleep(5000);
					final long _start = System.currentTimeMillis();
					try {
						final boolean success = getBaseData().backupWebToFile();
						if(!success)
							logWarning(null);
						else
							logSuccess(_start);

					} catch (final Exception ex) {
						logWarning(ex);
					}
				}
				private void logSuccess(final long _start) {
					final String msg = "Geschrieben " + getBaseData().getFileName() +".";
			        Logger.SWFW_.info(timedMsg(_start, msg));
				}
				private void logWarning(final Exception ex) {
					final String ttText = "No local Backup of file '" + getBaseData().getFileName() + "' possible.";
					Logger.SWFW.warn(ttText, ex);
				}}
			).start();
		}
	}

    private final IIniFileRoot processIniCandidate(final Object iniCandidate, final String fileName, final boolean asFile) {
        if(iniCandidate == null)             		{return handleEmptyIniFile(fileName);}
        if(!(iniCandidate instanceof IIniFileRoot))	{return handleBadClassType(fileName, iniCandidate);}
        if(!getBaseData().hasWebAddress())        	{return handlePureLocalIniFile(fileName, iniCandidate);}
        if(asFile)                           		{return handleCachedIniFile(fileName, iniCandidate);}
        return handlePureWebFile(fileName, iniCandidate);
    }

    private final IIniFileRoot handleNoFileName() {
    	if(LocalHome.noLocalHomeWanted())
    		return getIniFileRoot();
        final String toolTipText = "Can not read the ini file, since the filename is not given.";
        Logger.SWFW.warn(toolTipText);
        getMessageMgr().setMessage(new Message(NOINIFILE, toolTipText, Message.ERROR_MESSAGE));
        return getIniFileRoot();
    }
    private final IIniFileRoot handleNoWebAddress() {
        final String toolTipText = "Can not read the ini file, since the webaddress is not given.";
        Logger.SWFW.warn(toolTipText);
        getMessageMgr().setMessage(new Message(NOWEBADDRESS, toolTipText, Message.ERROR_MESSAGE));
        return getIniFileRoot();
    }
    private final IIniFileRoot handleBadClassType(final String fileName, final Object iniCandidate) {
        String toolTipText;
        final String candidateClassName = iniCandidate.getClass().getName();
        //$ANALYSIS-IGNORE
		if(MstIni.class.getName().equals(candidateClassName)){
		            toolTipText = "The ini file '" + fileName + "' can not be cast to Type MstIni."
		                        + "This is strange and an illegal behaviour of the virtual machine. "
		                        + "Sometimes this can be cured by a program restart or a PC boot. "
		                        + "For now I will not use: '" + fileName + "', but stick to what I've got.";
		        }else{
		            toolTipText = "The ini file '" + fileName + "' is not of Type MstIni but: '" + candidateClassName + "'.";
		        }
        Logger.SWFW.error(toolTipText);
        getMessageMgr().setMessage(new Message(BADINIFILE, toolTipText, Message.ERROR_MESSAGE));
        return getIniFileRoot();
    }
    private final IIniFileRoot handleNullPointerException(final String fileName) {
        final String toolTipText = "The ini file '" + fileName + "' is null.";
        getMessageMgr().setMessage(new Message(BADINIFILE, toolTipText, Message.ERROR_MESSAGE));
        Logger.SWFW.error(toolTipText);
        return getIniFileRoot();
    }
    private final IIniFileRoot handleIOException(final String fileName,final String action, final IOException ex) {
    	final String excMsg = ex.getMessage();
		if(        excMsg.startsWith("Server returned HTTP response code: 400 for URL")){
    		return handleNoIniFile(fileName, READ);
    	}else if(  excMsg.startsWith(SQLXML_OR_SQL_SERVER_PROBLEM)
    			|| excMsg.startsWith("java.util.NoSuchElementException")){
    		return handleBadXML(fileName, READ, excMsg);
    	}
        final String toolTipText = "Can not " + action + " the ini file '" + fileName + "', due to general IOException: " + excMsg;
        Logger.SWFW.error(toolTipText, ex);
        getMessageMgr().setMessage(new Message(NOINIFILE, toolTipText, Message.WARNING_MESSAGE));
        return getIniFileRoot();
    }
    private final IIniFileRoot handleException(final String fileName,final String action, final Exception ex) {
        final String toolTipText = "Can not " + action + " the ini file '" + fileName + "', due to general Exception: " + ex.getMessage();
        Logger.SWFW.error(toolTipText, ex);
        getMessageMgr().setMessage(new Message(NOINIFILE, toolTipText, Message.WARNING_MESSAGE));
        return getIniFileRoot();
    }
    private final IIniFileRoot handleNoFuture(final String webAddress) {
        final String toolTipText = "There is no future task created for " + webAddress;
        Logger.SWFW.warn(toolTipText);
        //Warning sufficient, since some apps do not have an ini file.
        getMessageMgr().setMessage(new Message(NOINIFILE, toolTipText, Message.WARNING_MESSAGE));
        return getIniFileRoot();
    }
    private final IIniFileRoot handleNoIniFile(final String fileName, final String action) {
        final String toolTipText = "Can not " + action + " the ini file '" + fileName + "', since it does not exist.";
        Logger.SWFW.warn(toolTipText);
        //Warning sufficient, since some apps do not have an ini file.
        getMessageMgr().setMessage(new Message(NOINIFILE, toolTipText, Message.WARNING_MESSAGE));
        return getIniFileRoot();
    }
    private final IIniFileRoot handleBadXML(final String fileName, final String action, final String someText) {
        final String toolTipText = "Can not " + action + " the ini file '" + fileName
        				   + "', since the XML seems to be corrupted. '" + someText +"'";
        Logger.SWFW.warn(toolTipText);
        //Warning sufficient, since some apps do not have an ini file.
        getMessageMgr().setMessage(new Message(NOINIFILE, toolTipText, Message.WARNING_MESSAGE));
        return getIniFileRoot();
    }
    private final IIniFileRoot handleEmptyIniFile(final String fileName) {
        final String toolTipText = "Local ini file '" + fileName + "' seems to be Empty.";
        Logger.SWFW.error(toolTipText);
        getMessageMgr().setMessage(new Message(BADINIFILE, toolTipText, Message.ERROR_MESSAGE));
        return getIniFileRoot();
    }
//    private final IIniFileRoot handleBadWebAddress(String webAddress) {
//        String toolTipText = "Web Address '" + webAddress + "' is malformed.";
//        Logger.SWFW.error(toolTipText);
//        getMessageMgr().setMessage(new Message(BADWEBADDRESS, toolTipText, Message.ERROR_MESSAGE));
//        return getIniFileRoot();
//    }
//    private final IIniFileRoot handleWebSrvNotReachable(String webAddress) {
//        String toolTipText = "Webserver in '" + webAddress + "' is not reachable.";
//        Logger.SWFW.warn(toolTipText);
//        getMessageMgr().setMessage(new Message(NOWEBSERVER, toolTipText, Message.WARNING_MESSAGE));
//        return getIniFileRoot();
//    }
    private final IIniFileRoot handlePureLocalIniFile(final String fileName, final Object uncastIIniFileRoot) {
        final String toolTipText = "Used local ini file. '" + fileName + "'.";
        Logger.SWFW.info(toolTipText);
        getMessageMgr().setMessage(new Message("", toolTipText, Message.INFORMATION_MESSAGE));
        return (IIniFileRoot)uncastIIniFileRoot;
}
    private final IIniFileRoot handlePureWebFile(final String fileName, final Object uncastIIniFileRoot) {
        final String toolTipText = "Used web hosted ini file. '" + fileName + "'.";
        Logger.SWFW.info(toolTipText);
        getMessageMgr().setMessage(new Message("", toolTipText, Message.INFORMATION_MESSAGE));
        return (IIniFileRoot)uncastIIniFileRoot;
}
    private final IIniFileRoot handleCachedIniFile(final String fileName, final Object uncastIIniFileRoot) {
        final String toolTipText = "Used local ini file and not the standard web one. '" + fileName + "'. It might be out of date.";
        Logger.SWFW.warn(toolTipText);
        getMessageMgr().setMessage(new Message(CACHEDINIFILE, toolTipText, Message.INFORMATION_MESSAGE));//irregular, but still ok
        return (IIniFileRoot)uncastIIniFileRoot;
    }
}