/*
 * Created on 	May 25, 2005 - Jun 27, 2005, Jul 7 2005
 *				Aug 15, 2005 - Aug 16, 2005 move mstIni Info to BaseData
 * 				Oct 25, 2005 - ...			new logging system
 * 				Dec 05, 2005 - ...			bessere Meldungen bei Applikationsprogrammierer Fehler
 * 				Dec 21, 2005 - Dec 21, 2005	Support for IIniFileObserver_FileDataIni
 */
package de.desy.mst.libs.framework.mstapp.ini;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * handles the observer stuff on behave of the IniMgr:<br>
 * <li>(de)registration of observers
 * <li>delegation of IIniFileObserver events: 
 *     onPrepareForExport and onNewIniAvailable
 * by using onPrepareForExport the last non null object succeeds   
 * 
 * @author ws
 * @version 1.0.0
 */
class IniFileObserverMgr{
    private final Set<IIniFileObserver> 			observers 	 = new HashSet<IIniFileObserver>();
    private final Set<IIniFileObserver_FileDataIni> observersFDI = new HashSet<IIniFileObserver_FileDataIni>();
    private final ABaseData<? extends IIniFileRoot> baseData;
  
//=============================================================================
    
    public IniFileObserverMgr(final ABaseData<? extends IIniFileRoot> baseData) {
        super();
        this.baseData = baseData;
        //Logger.SWFW.info("Created new IniFileObserverMgr " + baseData.getName());
    }

//-----------------------------------------------------------------------------

    /**@throws NullPointerException,  falls neuer observer null.*/     
    public final void register(final IIniFileObserver observer){
        if(observer == null){throw new NullPointerException("The new Observer must not be null.");}
        
        if(observer instanceof IIniFileObserver_FileDataIni){
	        final IIniFileObserver_FileDataIni observerFDI = (IIniFileObserver_FileDataIni)observer;
			if(hasFileDataIni()) {fireOnNewIniAvailableProtectedFDI(observerFDI, getFileDataIni());}
	        this.observersFDI.add(observerFDI);    //nach setContentRoot um doppel firing zu vermeiden
        }else{
	        if(getContentRoot() == null) {setContentRoot(fireGetDefaultIniBaumRootProtected(observer));}
	        fireOnNewIniAvailableProtected(observer, getContentRoot());
	        this.observers.add(observer);    //nach setContentRoot um doppel firing zu vermeiden
        }
    }

    public final boolean deregister(final IIniFileObserver observer){
    	if(observer instanceof IIniFileObserver_FileDataIni){return this.observersFDI.remove(observer);}
    	else 												{return this.observers.remove(observer);}
    }

//-----------------------------------------------------------------------------
    /** @return never null*/
    @SuppressWarnings("rawtypes") public final IIniFileRoot fireOnPrepareForExport() {
        //(1)	normal observers
    	for (final Iterator iter = this.observers.iterator(); iter.hasNext();) {
            final IIniFileObserver observer = (IIniFileObserver) iter.next();
            final Object newFile = fireOnPrepareForExportProtected(observer);
            if( newFile != null) {setContentRoot(newFile);}
        }
    	//(2)	special observers
        if(hasFileDataIni()){
        	final FileDataIni fileDataIni = getFileDataIni();
	        for (final IIniFileObserver_FileDataIni observerFDI : this.observersFDI) {
	        	fireOnPrepareForExportProtectedFDI(observerFDI, fileDataIni);
			}
        }
        //(3)	MstIni
        fireOnPrepareForExportProtected(this.baseData);       //file gets notified at end
        
        return this.baseData.getFileRoot();
    }
     
    /** @throws NullPointerException*/
    @SuppressWarnings("rawtypes") public final void fireOnNewIniAvailable(final IIniFileRoot iniFileRoot) {
        if(iniFileRoot == null){throw new NullPointerException("iniFileRoot must not be null");}
        //(1)	normal observers
        for (final Iterator iter = this.observers.iterator(); iter.hasNext();) {
            final IIniFileObserver observer = (IIniFileObserver) iter.next();
            fireOnNewIniAvailableProtected(observer, iniFileRoot.getContentRoot());
        }
    	//(2)	special observers
        if(hasFileDataIni()){
        	final FileDataIni fileDataIni = getFileDataIni();
	        for (final IIniFileObserver_FileDataIni observerFDI : this.observersFDI) {
	        	fireOnNewIniAvailableProtectedFDI(observerFDI, fileDataIni);
			}
        }
        //(3)	MstIni
        fireOnNewIniAvailableProtected(this.baseData, iniFileRoot);
    }
   
//-----------------------------------------------------------------------------
      
      /**
       * package private
       * @return Returns the mstIni never null
       */
      final IIniFileRoot getFileRoot() {
          return this.baseData.getFileRoot();
      }
      /**
       * package private  
       * @return s the application part of the standard ini object tree 
       */
      final Object getContentRoot() {
          return this.getFileRoot().getContentRoot(); 
      }
      
      /**
       * @param contentRoot the user root
       */
      final void setContentRoot(final Object contentRoot) {
          if(getContentRoot() == contentRoot ) {
              return;
          } 
          this.getFileRoot().setContentRoot(contentRoot);
          fireOnNewIniAvailable(getFileRoot());
      }
      
      /**package private zum Test only*/ 
      final boolean hasFileDataIni() {return getFileDataIni() != null;}
      /**package private zum Test only*/ 
      final FileDataIni getFileDataIni() {
		final IIniFileRoot fileRoot = this.baseData.getFileRoot();
		if(fileRoot instanceof MstIni){return ((MstIni)fileRoot).getFileData();}
		return null;
      }
      
//=============================================================================
      
  	private final Object fireOnPrepareForExportProtected(final IIniFileObserver observer) {
		try {
			return observer.onPrepareForExport();			
		} catch (final Exception e) {
			Logger.SWFW.error("Unerwartetes Applikations Problem beim Ereignis OnPrepareForExport.", e);
			return null;
		}
	}
	private final void fireOnPrepareForExportProtectedFDI(final IIniFileObserver_FileDataIni observerFDI, final FileDataIni fileDataIni) {
		try {observerFDI.onPrepareForExport(fileDataIni);} 
		catch (final Exception e) {Logger.SWFW.error("Unerwartetes Applikations Problem beim Ereignis OnPrepareForExport(FileDataIni).", e);}
	}
	
	private final void fireOnNewIniAvailableProtected(final IIniFileObserver observer, final Object contentRoot) {
		try {
			observer.onNewIniAvailable(contentRoot);			
		} catch (final Exception e) {
			Logger.SWFW.error("Unerwartetes Applikations Problem beim Ereignis OnNewIniAvailable.", e);
		}
	}

	private final void fireOnNewIniAvailableProtectedFDI(final IIniFileObserver_FileDataIni observerFDI, final FileDataIni fileDataIni ) {
		try {
			observerFDI.onNewIniAvailable(fileDataIni);			
		} catch (final Exception e) {
			Logger.SWFW.error("Unerwartetes Applikations Problem beim Ereignis OnNewIniAvailable(FileDataIni).", e);
		}
	}

	private final Object fireGetDefaultIniBaumRootProtected(final IIniFileObserver observer) {
		try {
			return observer.getDefaultIniBaumRoot();
		} catch (final Exception e) {
			Logger.SWFW.error("Unerwartetes Applikations Problem beim holen der DefaultIniBaumRoot.", e);
			return null;
		}
	}
}
