/*
 * Created on 	Jun  3, 2005 - Jun 30, 2005
 * 				Aug 16, 2005
 * 				Jul  4, 2007				getIniFileIOMgr private -> pp
 * May 15, 2008 - May 15, 2008	ws: added: timeOut
 * Dec 10, 2008 - Dec 10, 2008	ws:	MstSecurityMgr -> MstSecurity
 */
package de.desy.mst.libs.framework.mstapp.ini;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

import de.desy.mst.libs.framework.mstapp.security.MstSecurity;

/**
 * Bindeglied zwischen MstFrameMain und Initialisierung
 * insbesondere IniSupport, (De)RegisterSupport
 *
 * (only) public class in package
 *
 * @author ws
 * @version 1.0.0
 */
public final class IniFileUIMgr implements IIniFileObserverRegistrar{
    private static final String WRITE_SETTINGS_TO_SYSTEM_DOT_OUT = "Write settings to 'System.out' (Java Konsole).";

    private final IniFileIOMgr iniFileIOMgr;

    private JMenu mainMenu;
        private JMenuItem menuItemRead;
        private JMenuItem menuItemWriteToSystemDotOut;
        private JMenuItem menuItemWrite;

//=============================================================================
    /**@throws NullPointerException*/
    public IniFileUIMgr(final IniFileIOMgr iniFileIOMgr) {
        super();
        if(iniFileIOMgr == null){throw new NullPointerException("iniFileIOMgr may not be null.");}
        this.iniFileIOMgr = iniFileIOMgr;
    }

    public IIniFileRoot read() 				{return getIniFileIOMgr().read();}
	public IIniFileRoot read(final long timeoutMs){return getIniFileIOMgr().read(timeoutMs);}

//=============================================================================
    /**
     * @param menuSettings Hauptmenue fuer die Settings
     * @throws NullPointerException falls menuOptions == null
     */
    public final void initAndRegisterJMenu(final JMenu menuSettings) {
        //IDEE Menu -> intelligente KonfigurationsDialogJFrame mit allen bells & whistles
        menuSettings.add(getMainMenu());
    }


    /**@throws NullPointerException,  falls neuer observer null.*/
    @Override public final void registerIniObserver(final IIniFileObserver observer){
        getIniFileIOMgr().getObserverMgr().register(observer);
    }
    @Override public final boolean deregisterIniObserver(final IIniFileObserver observer){
        return getIniFileIOMgr().getObserverMgr().deregister(observer);
    }

    @Override public final Object getContentRoot(){
        return getIniFileIOMgr().getObserverMgr().getContentRoot();
    }
//=============================================================================

    /**pp*/ final IniFileIOMgr getIniFileIOMgr() {
        return this.iniFileIOMgr;
    }

    private final JMenu getMainMenu() {
        if(this.mainMenu == null){
            this.mainMenu = new JMenu();
            this.mainMenu.setText(getIniFileIOMgr().getBaseData().getShortDescription());

            this.mainMenu.add(getMenuItemRead());
            this.mainMenu.add(getMenuItemWriteToSystemDotOut());
            this.mainMenu.add(getMenuItemWrite());
        }
        return this.mainMenu;
    }
    private final JMenuItem getMenuItemRead() {
        if(this.menuItemRead == null){
            this.menuItemRead = createMenuItemRead();
        }
        return configMenuItemRead(this.menuItemRead);
    }
    private final JMenuItem getMenuItemWrite() {
        if(this.menuItemWrite == null){
            this.menuItemWrite = createMenuItemWrite();
        }
        return configMenuItemWrite(this.menuItemWrite);
    }
    private final JMenuItem getMenuItemWriteToSystemDotOut() {
        if(this.menuItemWriteToSystemDotOut == null){
            this.menuItemWriteToSystemDotOut = createMenuItemWriteToSystemDotOut();
        }
        return configMenuItemWriteToSystemDotOut(this.menuItemWriteToSystemDotOut);
    }
 
    private final JMenuItem createMenuItemRead() {
        final JMenuItem item = new JMenuItem();
        item.addActionListener(new ActionListener() {
            @Override public void actionPerformed(final ActionEvent e) {
                getIniFileIOMgr().read();
            }
        });
        return item;
    }

    private final JMenuItem createMenuItemWrite() {
        final JMenuItem item = new JMenuItem();
        item.addActionListener(new ActionListener() {
            @Override public void actionPerformed(final ActionEvent e) {
                getIniFileIOMgr().write();
            }
        });
        return item;
    }

    private final JMenuItem createMenuItemWriteToSystemDotOut() {
        final JMenuItem item = new JMenuItem();
        item.addActionListener(new ActionListener() {
            @Override public void actionPerformed(final ActionEvent e) {
                final String text = getIniFileIOMgr().getIniBaumRootAsText();
                System.out.println(text);
            }
        });
        return item;
    }

    @SuppressWarnings("rawtypes") private JMenuItem configMenuItemRead(final JMenuItem item) {
        final ABaseData baseData = getIniFileIOMgr().getBaseData();
        item.setText(baseData.getReadComment());
        item.setEnabled( baseData.hasFileName() );
        return item;
    }
    @SuppressWarnings("rawtypes") private JMenuItem configMenuItemWrite(final JMenuItem item) {
        final ABaseData baseData = getIniFileIOMgr().getBaseData();
        item.setText(baseData.getWriteComment());
        item.setEnabled(  MstSecurity.get().isDeveloper()
                     &&   baseData.hasFileName()
                     && !(LocalHome.noLocalHomeWanted())
                       );
        return item;
    }
    private JMenuItem configMenuItemWriteToSystemDotOut(final JMenuItem item) {
        item.setText(WRITE_SETTINGS_TO_SYSTEM_DOT_OUT);
        item.setEnabled(true);
        return item;
    }
 }
