/**
 * ws:	Apr 19, 2007 - May 07, 2007 creation from scratch
 * 												& wst PrefSystemEinstellungen
 * ws:	Jul 03, 2007 - ...			update to writeToConsole in IOMGR
 * ws:	Oct 30, 2008 - Oct 30, 2008	getTitle() private -> public
 */
package de.desy.mst.libs.framework.mstapp.ini;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Arrays;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer;
import de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanelEinfach;
import de.desy.mst.libs.framework.mstapp.util.gui.GUIFactory;

/**
 * Panel fuer die IniDateien im Preferences System (Leaf Level)
 * @author ws
 * version 1.0.0
 */
public class PrefIniDateienPanel extends APrefClientPanelEinfach {
	private static final long serialVersionUID = 1L;
	private final IniFileIOMgr iniFileIOMgr;
	private SELSTATE selState = SELSTATE.EMPTY;

//=============================================================================

	public PrefIniDateienPanel(final IPrefContainer prefContainer
							 , final IniFileIOMgr iniFileIOMgr) {
		super(prefContainer);
		this.iniFileIOMgr = iniFileIOMgr;
		initialize();
	}

//=============================================================================

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanelMitPersistenz#onDispose()
	 */
	@Override public void onDispose() {/*nothing to clean up*/}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#onReset()
	 */
	@Override public void onReset(){initFromSystem();}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#onSet()
	 */
	@Override public void onSet() throws Exception {
		checkForWritePermissions();
		setSystemToValues();
	}

//=============================================================================

	private void initFromSystem() {
		final ABaseData<? extends IIniFileRoot> baseData =
												getIniFileIOMgr().getBaseData();
//		final String textW= "\'http://mstweb.desy.de/IniDaten/_System/_System.xml \'";
		final String textW= baseData.getWebAddress();
		getJCBoxLoadFromWeb().setToolTipText(textW);

//		final String textL= "\'c:\\\\mst_app\\IniDaten\\_System\\_System.xml \'";
		final String textL= baseData.getFileName();
		final JCheckBox boxWriteToLocalDisc = getJCBoxWriteToLocalDisc();
		boxWriteToLocalDisc.setToolTipText(textL);
		boxWriteToLocalDisc.setEnabled(!this.noLocalHomeWanted());

		/**deselect all checkboxes*/
		final JCheckBox[] boxes = getJCBoxes();
		for (final JCheckBox box : boxes) {
			box.getModel().setSelected(false);
		}
	}
	private void setSystemToValues() {
		getSTATE().execute(getIniFileIOMgr());
	}

	private void checkForWritePermissions() throws IllegalAccessException {
		if(noLocalHomeWanted()){
			final String msg = "Benutzer moechte keinen lokale Cache.";
			throw new IllegalAccessException(msg);
		}
	}

	private boolean noLocalHomeWanted() {return LocalHome.noLocalHomeWanted();}


	@Override public final String getTitle()	{
		return 	getIniFileIOMgr().getName() + " Einstellungen (Initialisierungen)";
	}
	/**definiert als eine der Checkboxen ist enabled*/
	private final boolean isChanged(){return this.selState != SELSTATE.EMPTY;}
	private final void updateSTATE(){
		final JCheckBox[] boxes = getJCBoxes();
		final SELSTATE selState =
							SELSTATE.getSTATE(boxes[0].getModel().isSelected()
											, boxes[1].getModel().isSelected()
											, boxes[2].getModel().isSelected()
										 	 );
		if(selState == null){
			Logger.SWFW.warn("Problem Selection State zu setzen ignorier.");
			return;
		}
		this.selState = selState;
		final ENABLESTATE enablestate = selState.getEnablestate();
		enablestate.setEnablement(getJCBoxes());
		getPrefContainer().setChanged(isChanged());
	}
	public final SELSTATE getSTATE  ()			{return this.selState;}
	public final IniFileIOMgr  getIniFileIOMgr(){return this.iniFileIOMgr;}

//=============================================================================
	//HIER BEGINNT DIE GRAFIK:
	private static final String TO_PLATTE_TEXT  =
					"Schreibe Initialisierungen auf die lokale Platte \052)";
	private static final String TO_KONSOLE_TEXT =
					"<html> Schreibe Initialisierungen auf die Java-Konsole";
	private static final String LIES_WEBINI_TEXT =
					"Lies Initialisierungen vom Web-Server \052)";
	private static final String FUSSNOTE =
					"\052  Pfad-Informationen im ToolTipText";

	private JPanel 		jPnlSystemeinstellungen = null;
	private JCheckBox 	jCBoxLoadFromWeb 		= null;
	private JCheckBox 	jCBoxWriteToConsole 	= null;
	private JCheckBox 	jCBoxWriteToLocalDisc 	= null;
	private JLabel 		jLbl 					= null;

	private void initialize() {
		this.setSize(300, 200);
		this.add(getJPnlSystemeinstellungen(), null);
		initFromSystem();
	}
	private JPanel getJPnlSystemeinstellungen() {
		if (this.jPnlSystemeinstellungen == null) {
			this.jPnlSystemeinstellungen = new JPanel();
			this.jPnlSystemeinstellungen.setLayout(null);
			this.jPnlSystemeinstellungen.setLayout(
					new BoxLayout(this.jPnlSystemeinstellungen, BoxLayout.Y_AXIS));
			this.jPnlSystemeinstellungen.setBorder(BorderFactory
					.createTitledBorder(BorderFactory
							.createLineBorder(Color.gray, 1)
								, getTitle()
								, TitledBorder.DEFAULT_JUSTIFICATION
								, TitledBorder.DEFAULT_POSITION, null, null));
			this.jPnlSystemeinstellungen.setPreferredSize(new Dimension(350,200));
			final Component vertGlue=Box.createVerticalGlue();
			final Component vertStrut=Box.createVerticalStrut(40);
			this.jPnlSystemeinstellungen.add(vertStrut, null);
			this.jPnlSystemeinstellungen.add(getJCBoxLoadFromWeb(), null);
			this.jPnlSystemeinstellungen.add(vertGlue, null);
			this.jPnlSystemeinstellungen.add(getJCBoxWriteToConsole(), null);
			this.jPnlSystemeinstellungen.add(vertGlue, null);
			this.jPnlSystemeinstellungen.add(getJCBoxWriteToLocalDisc(), null);
			this.jPnlSystemeinstellungen.add(vertGlue, null);
			this.jPnlSystemeinstellungen.add(getJLbl(), null);
		}
		return this.jPnlSystemeinstellungen;
	}
	private JLabel getJLbl(){
		if (this.jLbl == null){
			this.jLbl = new JLabel(FUSSNOTE);
			this.jLbl.setFont(GUIFactory.createButtonFont());
		}
		return this.jLbl;
	}

	private JCheckBox getJCBoxLoadFromWeb() {
		if (this.jCBoxLoadFromWeb == null) {
			this.jCBoxLoadFromWeb = jcBOXCreationHelper
					(GUIFactory.createCheckBoxWithTooltip(), LIES_WEBINI_TEXT);
			this.jCBoxLoadFromWeb.setEnabled(getIniFileIOMgr().hasWebIni());
		}
		return this.jCBoxLoadFromWeb;
	}
	private JCheckBox getJCBoxWriteToConsole() {
		if (this.jCBoxWriteToConsole == null) {
			this.jCBoxWriteToConsole = jcBOXCreationHelper
											(new JCheckBox(), TO_KONSOLE_TEXT);
		}
		return this.jCBoxWriteToConsole;
	}
	private JCheckBox getJCBoxWriteToLocalDisc() {
		if (this.jCBoxWriteToLocalDisc == null) {
			this.jCBoxWriteToLocalDisc = jcBOXCreationHelper
					(GUIFactory.createCheckBoxWithTooltip(), TO_PLATTE_TEXT);
		}
		return this.jCBoxWriteToLocalDisc;
	}
	private JCheckBox jcBOXCreationHelper(final JCheckBox jCBoxLoadFromWeb, final String text) {
		jCBoxLoadFromWeb.createToolTip();
		jCBoxLoadFromWeb.setFont(GUIFactory.createButtonFont());
		jCBoxLoadFromWeb.setText(text);
		jCBoxLoadFromWeb.addItemListener(new ItemListener() {
			@Override public void itemStateChanged(final ItemEvent e) {updateSTATE();}
		});
		return jCBoxLoadFromWeb;
	}

	/**FromWeb, ToConsole, ToLocalDisc*/
	private JCheckBox[] getJCBoxes(){
		return new JCheckBox[] 	{
									  getJCBoxLoadFromWeb()
									, getJCBoxWriteToConsole()
									, getJCBoxWriteToLocalDisc()
								};
	}
//ENDE GRAPFIK
}


/**State is via isSelected of three checkboxes*/
class SELSTATE {
	public static SELSTATE EMPTY   =new SELSTATE("EMPTY",   ENABLESTATE.ALL, false,false,false);
	public static SELSTATE FROMWEB =new SELSTATE("FROMWEB", ENABLESTATE.FROM,true, false,false){
		@Override public void execute(final IniFileIOMgr iniFileIOMgr) {
			iniFileIOMgr.readWeb();
		}
	};
	public static SELSTATE TOANY   =new SELSTATE("TOANY",   ENABLESTATE.TO,  false,true, true ){
		@Override public void execute(final IniFileIOMgr iniFileIOMgr){
			SELSTATE.TOKON.execute(iniFileIOMgr);
			SELSTATE.TOLOCALD.execute(iniFileIOMgr);
		}
	};
	public static SELSTATE TOKON   =new SELSTATE("TOKON",   ENABLESTATE.TO,  false,true, false){
		@Override public void execute(final IniFileIOMgr iniFileIOMgr){
			iniFileIOMgr.writeToConsole();
		}
	};
	public static SELSTATE TOLOCALD=new SELSTATE("TOLOCALD",ENABLESTATE.TO,  false,false,true ){
		@Override public void execute(final IniFileIOMgr iniFileIOMgr) {
			iniFileIOMgr.write();
		}
	};

	private static SELSTATE[]values= {EMPTY,FROMWEB,TOANY,TOKON,TOLOCALD};
	private final boolean[] state;
	private final String	name;
	private final ENABLESTATE enablestate;
	//---------------------------------------------------------------------
	/**FromWeb, ToConsole, ToLocalDisc*/
	public SELSTATE(final String name, final ENABLESTATE enablestate, final boolean ... state) {
		if(state.length != 3){
			throw new IllegalArgumentException("bad state");
		}
		this.name  = name;
		this.enablestate = enablestate;
		this.state = state;
	}
	//---------------------------------------------------------------------
	/**returns correspondin STATE or if not exists null<br>
	 * FromWeb, ToConsole, ToLocalDisc*/
	public static SELSTATE getSTATE(final boolean ... state){
		for (final SELSTATE s : SELSTATE.values) {
			if(s.isState(state)){
				return s;
			}
		}
		Logger.SWFW.warn("Problem Selection State zu setzen ignorier."
						+ state[0] + ", " + state[1] + ", "  + state[2]);
		return null;
	}
	//---------------------------------------------------------------------
	public boolean isState(final boolean[] b)			{return Arrays.equals(b, this.state);}
	public final ENABLESTATE getEnablestate()	{return this.enablestate;}
	@Override public String toString() 			{return this.name;}
	public void execute(final IniFileIOMgr iniFileIOMgr) {
		final String msg = "Not implemented SELSTATE." + this + ".execute ignore.";
		Logger.SWFW.warn(msg);
	}
}
class ENABLESTATE{
	public static ENABLESTATE ALL =new ENABLESTATE("ALL", true, true,true );
	public static ENABLESTATE TO  =new ENABLESTATE("TO",  false,true,true );
	public static ENABLESTATE FROM=new ENABLESTATE("FROM",true,false,false);
	//---------------------------------------------------------------------
	private final boolean[] state;
	private final String	name;
	//---------------------------------------------------------------------
	/**FromWeb, ToConsole, ToLocalDisc*/
	public ENABLESTATE(final String name, final boolean ... state) {
		if(state.length != 3){
			throw new IllegalArgumentException("bad state");
		}
		this.name  = name;
		this.state = state;
	}
	//---------------------------------------------------------------------
	@Override public String toString() {return this.name;}
	public void setEnablement(final JCheckBox[] boxes){
		for (int i = 0; i < this.state.length; i++) {
			boxes[i].getModel().setEnabled(this.state[i]);
		}
	} 
}
