/**
 *  Created on	Feb 15, 2006 - Feb 16, 2006
 */
package de.desy.mst.libs.framework.mstapp.ini;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PushbackInputStream;
import java.io.UnsupportedEncodingException;

import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Hilft den umgang mit Text Streams<br>
 * Streams muessen ins memory passen<br>
 * input stream wird verbraucht/geschlossen
 * 
 * @author ws
 * @version 1.0.0
 */
public class TextInputStreamUtility {
	private static final String DEFAULT_CHARSETNAME = "UTF-8";

	private byte[] contents = new byte[0]; // never null

	// =============================================================================

	/**
	 * convertiert einen Text InputStream zu einem internen byte array 'contents'<br>
	 * input stream wird geschlossen
	 * 
	 * @throws UnsupportedEncodingException, IOException
	 * @throws NullPointerException, falls stream == null
	 */
	public TextInputStreamUtility(final InputStream is) throws IOException {
		super();
		if (is == null) {throw new NullPointerException("Der stream darf nicht null sein.");}
		final PushbackInputStream stream = new PushbackInputStream(new BufferedInputStream(is));
		int n;
        while(true){
    		if(isAtEndOfFile(stream) ) {//evtl. auf input warten (synchron, ohne Nebenwirkungen)
    			break;
    		}
    		n = stream.available();	
    		if(n > 0){//Zahl, der ohne Warten verfuegbaren Bytes 
	        	byte[] b = new byte[n];
	        	@SuppressWarnings("unused") //result 
	        	int result = stream.read(b);
				this.addContents(b);
        	}
        }
		stream.close();
	}

	// =============================================================================

	/** never null */
	public final byte[] getContentsAsByteArray() {
		return this.contents;
	}

	/**never null*/
	public final String getContentsAsString() {
		try {
			return new String(getContentsAsByteArray(), DEFAULT_CHARSETNAME);
		} catch (UnsupportedEncodingException e) {
			Logger.SWFW.error("Kann keinen String erzeugen im DEFAULT_CHARSETNAME.", e);
			return  new String(getContentsAsByteArray());
		}
	}

	/** never null */
	public final InputStream getContentsAsInputStream() {
		return new ByteArrayInputStream(getContentsAsByteArray());
	}

	/**
	 * protected zum testen only
	 * 
	 * @throws NullPointerException,
	 *             falls contents == null
	 */
	final void setContents(final byte[] contents) {
		if (contents == null) {
			throw new NullPointerException("Der contents darf nicht null sein.");
		}
		this.contents = contents;
	}

	/**
	 * protected zum testen only
	 * packt moreContents hinter contents
	 * @throws NullPointerException, falls contents == null
	 */
	final void addContents(final byte[] moreContents) {
		if(moreContents == null) { throw new NullPointerException("Der moreContents darf nicht null sein.");}
		if(moreContents.length == 0) {return;}	// nichts zu tun
		byte[] oldContents = getContentsAsByteArray();
		if(oldContents.length == 0){
			this.setContents(moreContents);
		}else{
			byte[] newContents = new byte[oldContents.length + moreContents.length];
//			System.arraycopy(Object   src, int srcPos, Object dest,		   int destPos,          int length);
			System.arraycopy( oldContents,			0, newContents,					 0,  oldContents.length);
			System.arraycopy(moreContents, 			0, newContents, oldContents.length, moreContents.length);
			this.setContents(newContents);
		}
	}

	/**Nebenwirkungsfrei, synchron d.h. es kann blockieren*/
	private boolean isAtEndOfFile(PushbackInputStream stream) throws IOException {
		int readResult = stream.read();
		boolean isEOF  = (readResult == -1);
		if(!isEOF) {stream.unread(readResult);}
		return isEOF;
	}
}
