package de.desy.mst.libs.framework.mstapp.io;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.net.UnknownHostException;

import de.desy.mst.libs.framework.mstapp.appinf.ApplikationKind;
import de.desy.mst.libs.framework.mstapp.appinf.IApp;
import de.desy.mst.libs.framework.mstapp.appinf.IContactInformation;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.tine.types.NAME64I;

public class Mail {
	private final String HOST_SMTP = "smtp.desy.de";//Postausgangsserver
	private final int PORT_SMTP = 25;
//	private final String HOST_IMAP = "ximap.desy.de";//Server
//	private final int PORT_IMAP = 993;
	private final String DEFAULT_SENDER = "doocsadm@ttf2svr3.desy.de";
	private NAME64I error = new NAME64I("success", 0);
    private String[] getReceipient(final String[] rec){
    	String[] s = null;
    	if(rec == null){
	    	try{
		    	final IApp app = ApplikationKind.Automatic.getApplicationInformation();
		    	final IContactInformation contactTo = app.getContact();
		    	s = new String[1];
		    	s[0] = contactTo.getEmail();
	    	}catch(final Throwable e){
	    		Logger.SWFW.error("Empfnger, keine gltige Mail Addresse: " + e.getMessage(), e);
	    	}
    	}else{
    		s = rec;
    	}
    	return s;
    }
    private String getSender(final String sender){
    	String s = sender;
    	if(s == null){
    		s = this.DEFAULT_SENDER;
    	}
    	return s;
    }
    private String getAuthor(final String entryAuthor){
    	String s = "";
    	if(entryAuthor != null && entryAuthor.length() >= 1){
    		s = "Autor, " + entryAuthor + "\r\n"+"\r\n";
    	}
    	return s;
    }
    private String getBody(final String entryAuthor, final String body){
    	String s = "Es wurde kein Text als Parameter bergeben" +
    		       "\r\n" +
    	           "No text was submitted"
    	;
    	final String author = getAuthor(entryAuthor);
    	if(body != null && body.length() >= 1){
    		s = body;
    	}
    	s = author +
    	    s +
    	    "\r\n"+"\r\n" +
    	    "."+"\r\n"+
    	    "QUIT\r\n";
    	return s;
    }
    private String getSubject(final String subject){
    	String s = "Mail vom MstApp, kein Betreff; Mail from MstApp, no subject";
    	if(subject != null && subject.length() >= 1){
    		s = subject;
    	}
    	s = "Subject:" +
    	    s +
    	    "\r\n";
    	return s;
    }
    private void sendLine(final BufferedWriter bw, final String line) throws IOException{
      	final boolean DEBUG = false;
		bw.write(line);
		bw.flush();
		if (DEBUG){Logger.SWFW.info(line);}

    }
    public void sendMail(final String subj, final String bd, final String entryAuthor, final String[] to, final String from){
    	final String[] rec  = getReceipient(to);
    	final String sender = getSender(from);
    	final String body = getBody(entryAuthor, bd);
    	final String subject = getSubject(subj);
    	this.error = new NAME64I("Mail erfolgreich versendet", 0);
    	if(rec == null){ 
    		Logger.SWFW.error("Mail kann nicht versendet werden, da der Empfnger null ist");
        	this.error = new NAME64I("Empfnger ist null", 1);
    		return ;
    	}
		try {
			for(int i = 0; i < rec.length; i++){
				final Socket socket = new Socket(this.HOST_SMTP, this.PORT_SMTP);
				final BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream()));
				Logger.SWFW.info("[ANFANG_MAIL] " + rec[i]);
					//Kontakt mit dem Server aufnehmen
					sendLine(bw, "Helo " + this.HOST_SMTP + "\r\n");
					//Absender hinschicken
					sendLine(bw, "MAIL FROM:<" + sender + ">\r\n");
					//Empfnger hinschicken
					sendLine(bw, "RCPT TO:" + rec[i] + "\r\n");
					//dem Server sagen: jetzt kommen die Daten
					sendLine(bw, "DATA\r\n");
					//Betreff und Text senden
					sendLine(bw, subject + body);
					//BufferedWriter schlieen
					bw.close();
					socket.close();
				Logger.SWFW.info("[ENDE_MAIL] " + rec[i]);
			}
		} catch (final UnknownHostException e) {
			Logger.SWFW.error(this.HOST_SMTP + " nicht gefunden", e);
	    	this.error = new NAME64I(this.HOST_SMTP + " nicht gefunden", 2);
		} catch (final IOException e) {
			Logger.SWFW.error("Fehler bei der Verbindung mit " + this.HOST_SMTP, e);
			this.error = new NAME64I("Fehler bei der Verbindung mit " + this.HOST_SMTP, 3);
		}
    }
	public NAME64I getError() {
		return this.error;
	}
}
