/**
 *  Created on 	Oct 21, 2005 - Nov 03, 2005
 *  			Aug 07, 2006 - Aug 07, 2006	ws: added Auftraege
 *  			Mar 13, 2007 - Mar 13, 2007 ws: Any(this.Any) Any() wegen Java6
 *  			Apr 11, 2007 - ...			ws: added XLIB Kategorie fuer JWlg..
 * 	May  6, 2008 - May 19, 2008	ws: added: SWFW_, (NOT HERE!!) always log Start
 * 	Oct  9, 2008 - Oct  9, 2008	ws: added: isTrueChild(...)
 * 	Mar 23, 2009 - Mar 23, 2009	ws:	now Serializable
 *	Jun 03, 2006 - Jun 12, 2009	ws:	support for Tags
 * 	Jun 22, 2008 - Jul  2, 2008	ws:	LogAppenderFile use Factory, temp Files,Tine
 * 	Jul 20, 2008 - Jul 20, 2008	ws:	doLog explizit ohne Tags wegen AnLa, JoWi external Logging Problem
 */

package de.desy.mst.libs.framework.mstapp.logging;

import java.io.Serializable;
import java.util.Date;

/**
 * Haupt Einsprungpunkt in das MST Applikations Logging System.
 * Und aus SyntaxSugar Gruenden auch die Loggingkategorie.
 * So gibt es sehr kompakten Gebrauch: 'Logger.Any.info("Meine Message");'
 * Die Katerorien (Streng hierarchisch mit Wurzel Any) gelten bei
 * Software fuer den Programmteil, der das Problem findet. Also Applikationsfehler,
 * die das Framework findet sind von der Kategorie 'SWFW'.
 * 	  <li>Any		// 0 beliebige Kategorie (parent of all)
 * 	  <li>HW		// 1 HardWare: z.B. Device Fehler
 * 	  <li>Feldbus	// 2 Feldbus wie SEDAC, PROFIBUS etc
 * 	  <li>SEDAC		// 3 SEDAC Protokoll
 * 	  <li>Netzwerk	// 4 Netzwerk insbesondere LAN, WAN, Router, DNS Server...
 * 	  <li>SW		// 5 SoftWare
 * 	  <li>SWApp		// 6 Software bezglich der Applikation
 * 	  <li>SWTine	// 7 Software bezglich von Tine
 * 	  <li>SWFW		// 8 Software bezglich des Frameworks
 * 	  <li>AUFTRAG	// 9 Auftrag an einen Server
 * 	  <li>XLIB		//10 External Library Logging in Java API
 * 	  <li>SWLib		//11 Internal Library Logging (eine Bibliothek, die mstAppLogging benutzt)
 * 	  <li>SWFW_		//12 Software bezglich des Frameworks Core
 * <br>
 * Das Management der Logging Appender ist in LoggingManager zu finden.
 * Die eigentliche Arbeit wird in package privater Klasse
 * LoggingImpl(ementation) gemacht.
 *
 * @author ws
 * @version 1.0.0
 */
public enum Logger implements Serializable{
	  /** 0 beliebige Kategorie (parent of all)*/	Any		()
	, /** 1 HardWare: z.B. Device Fehler*/			HW		(Any)
	, /** 2 Feldbus*/								FeldBus	(Any)
	, /** 3 SEDAC Protokoll*/						SEDAC	(FeldBus)
	, /** 4 Netzwerk*/								Netzwerk(Any)
	, /** 5 SoftWare*/								SW		(Any)
	, /** 6 Software bezglich der Applikation*/	SWApp	(SW)
	, /** 7 Software bezglich von Tine*/			SWTine	(SW)
	, /** 8 Software bezglich des Frameworks*/		SWFW	(SW)
	, /** 9 Auftrag an einen Server*/				AUFTRAG	(Any)
	, /**10 External Library Logging in Java API*/	XLIB	(SW)
	, /**11 Internal Library Logging (Bibliothek)*/	SWLib	(SW)
	, /**12 Software bezglich des Frameworks Core*/SWFW_	(SWFW)
	;
//=============================================================================
	public final Logger parent;
//=============================================================================
	private Logger() {
		this.parent = this;
		if(this.ordinal() != 0 /*Logger.Any*/) throw new RuntimeException("Nur Any erlaubt.");
	}
	private Logger(Logger parentKategorie) {
		this.parent = parentKategorie;
	}
//=============================================================================
	/**Dieses Objekt ist (Unter-)Kategorie von dem Argument*/
	public final boolean isKategorie(final Logger kat){
		return 	 this == kat
			||	(this != Any && this.parent.isKategorie(kat));
	}

	/**excludes Logger.ANY which is its own child*/
	public boolean isTrueDirectChild(final Logger childCandidate) {
		return this.equals(childCandidate.parent) && this != childCandidate;
	}
//=============================================================================
	/**<b>debug(0)</b>, info(1), warn(2), error(3), fatal(4)*/
	public final LogMsg debug(final String    message  							 ){return Logger.doLog(LogLvl.debug, this, message, null     );}
	/**<b>debug(0)</b>, info(1), warn(2), error(3), fatal(4)*/
	public final LogMsg debug(							final Throwable throwable){return Logger.doLog(LogLvl.debug, this, null,    throwable);}
	/**<b>debug(0)</b>, info(1), warn(2), error(3), fatal(4)*/
	public final LogMsg debug(final String    message, 	final Throwable throwable){return Logger.doLog(LogLvl.debug, this, message, throwable);}
	/**debug(0), <b>info(1)</b>, warn(2), error(3), fatal(4)*/
	public final LogMsg info (final String    message  							 ){return Logger.doLog(LogLvl.info,  this, message, null       );}
	/**debug(0), <b>info(1)</b>, warn(2), error(3), fatal(4)*/
	public final LogMsg info (							final Throwable throwable){return Logger.doLog(LogLvl.info,  this, null,    throwable);}
	/**debug(0), <b>info(1)</b>, warn(2), error(3), fatal(4)*/
	public final LogMsg info (final String    message, 	final Throwable throwable){return Logger.doLog(LogLvl.info,  this, message, throwable);}
	/**debug(0), info(1), <b>warn(2)</b>, error(3), fatal(4)*/
	public final LogMsg warn (final String    message  							 ){return Logger.doLog(LogLvl.warn,  this, message, null     );}
	/**debug(0), info(1), <b>warn(2)</b>, error(3), fatal(4)*/
	public final LogMsg warn (							final Throwable throwable){return Logger.doLog(LogLvl.warn,  this, null,    throwable);}
	/**debug(0), info(1), <b>warn(2)</b>, error(3), fatal(4)*/
	public final LogMsg warn (final String    message, 	final Throwable throwable){return Logger.doLog(LogLvl.warn,  this, message, throwable);}
	/**debug(0), info(1), warn(2), <b>error(3)</b>, fatal(4)*/
	public final LogMsg error(final String    message  							 ){return Logger.doLog(LogLvl.error, this, message, null     );}
	/**debug(0), info(1), warn(2), <b>error(3)</b>, fatal(4)*/
	public final LogMsg error(							final Throwable throwable){return Logger.doLog(LogLvl.error, this, null,    throwable);}
	/**debug(0), info(1), warn(2), <b>error(3)</b>, fatal(4)*/
	public final LogMsg error(final String    message, 	final Throwable throwable){return Logger.doLog(LogLvl.error, this, message, throwable);}
	/**debug(0), info(1), warn(2), error(3), <b>fatal(4)</b>*/
	public final LogMsg fatal(final String    message  							 ){return Logger.doLog(LogLvl.fatal, this, message, null     );}
	/**debug(0), info(1), warn(2), error(3), <b>fatal(4)</b>*/
	public final LogMsg fatal(							final Throwable throwable){return Logger.doLog(LogLvl.fatal, this, null,    throwable);}
	/**debug(0), info(1), warn(2), error(3), <b>fatal(4)</b>*/
	public final LogMsg fatal(final String    message, 	final Throwable throwable){return Logger.doLog(LogLvl.fatal, this, message, throwable);}
//=============================================================================
	/**<b>debug(0)</b>, info(1), warn(2), error(3), fatal(4)*/
	public final LogMsg debug(final String    message  							 , final LMTag ...tags){return Logger.doLog(LogLvl.debug, this, message, null     , tags);}
	/**<b>debug(0)</b>, info(1), warn(2), error(3), fatal(4)*/
	public final LogMsg debug(							final Throwable throwable, final LMTag ...tags){return Logger.doLog(LogLvl.debug, this, null,    throwable, tags);}
	/**<b>debug(0)</b>, info(1), warn(2), error(3), fatal(4)*/
	public final LogMsg debug(final String    message, 	final Throwable throwable, final LMTag ...tags){return Logger.doLog(LogLvl.debug, this, message, throwable, tags);}
	/**debug(0), <b>info(1)</b>, warn(2), error(3), fatal(4)*/
	public final LogMsg info (final String    message  							 , final LMTag ...tags){return Logger.doLog(LogLvl.info,  this, message, null     , tags);}
	/**debug(0), <b>info(1)</b>, warn(2), error(3), fatal(4)*/
	public final LogMsg info (							final Throwable throwable, final LMTag ...tags){return Logger.doLog(LogLvl.info,  this, null,    throwable, tags);}
	/**debug(0), <b>info(1)</b>, warn(2), error(3), fatal(4)*/
	public final LogMsg info (final String    message, 	final Throwable throwable, final LMTag ...tags){return Logger.doLog(LogLvl.info,  this, message, throwable, tags);}
	/**debug(0), info(1), <b>warn(2)</b>, error(3), fatal(4)*/
	public final LogMsg warn (final String    message  							 , final LMTag ...tags){return Logger.doLog(LogLvl.warn,  this, message, null     , tags);}
	/**debug(0), info(1), <b>warn(2)</b>, error(3), fatal(4)*/
	public final LogMsg warn (							final Throwable throwable, final LMTag ...tags){return Logger.doLog(LogLvl.warn,  this, null,    throwable, tags);}
	/**debug(0), info(1), <b>warn(2)</b>, error(3), fatal(4)*/
	public final LogMsg warn (final String    message, 	final Throwable throwable, final LMTag ...tags){return Logger.doLog(LogLvl.warn,  this, message, throwable, tags);}
	/**debug(0), info(1), warn(2), <b>error(3)</b>, fatal(4)*/
	public final LogMsg error(final String    message  							 , final LMTag ...tags){return Logger.doLog(LogLvl.error, this, message, null     , tags);}
	/**debug(0), info(1), warn(2), <b>error(3)</b>, fatal(4)*/
	public final LogMsg error(							final Throwable throwable, final LMTag ...tags){return Logger.doLog(LogLvl.error, this, null,    throwable, tags);}
	/**debug(0), info(1), warn(2), <b>error(3)</b>, fatal(4)*/
	public final LogMsg error(final String    message, 	final Throwable throwable, final LMTag ...tags){return Logger.doLog(LogLvl.error, this, message, throwable, tags);}
	/**debug(0), info(1), warn(2), error(3), <b>fatal(4)</b>*/
	public final LogMsg fatal(final String    message  							 , final LMTag ...tags){return Logger.doLog(LogLvl.fatal, this, message, null     , tags);}
	/**debug(0), info(1), warn(2), error(3), <b>fatal(4)</b>*/
	public final LogMsg fatal(							final Throwable throwable, final LMTag ...tags){return Logger.doLog(LogLvl.fatal, this, null,    throwable, tags);}
	/**debug(0), info(1), warn(2), error(3), <b>fatal(4)</b>*/
	public final LogMsg fatal(final String    message, 	final Throwable throwable, final LMTag ...tags){return Logger.doLog(LogLvl.fatal, this, message, throwable, tags);}
//=============================================================================
	/**basic interface im technischen Sinn*/
	public static LogMsg doLog(final LogMsg msg){
		return LoggerImpl.doLog(msg);
	}
	/**komplettes Endbenutzer Interface mit Throwable*/
	public static LogMsg doLog( final LogLvl 	level
							  , final Date 	date
							  , final String 	applikation
							  , final String 	pcName
							  , final Logger 	kategorie
							  , final String 	message
							  , final Throwable throwable
							  , final LMTag ...tags
			     			) {
		LogMsg msg = new LogMsg(level, date, applikation, pcName, kategorie, message, throwable, tags);
		return doLog(msg);
	}
	/**"komplettes" Endbenutzer Interface ohne Throwable*/
	public static LogMsg doLog( final LogLvl 	level
							  , final Date 		date
							  , final String 	applikation
							  , final String 	pcName
							  , final Logger 	kategorie
							  , final String 	message
							  , final LMTag ...tags
			     			) {
		LogMsg msg = new LogMsg(level, date, applikation, pcName, kategorie, message, tags);
		return doLog(msg);
	}
//	/**Basis Interface ohne Throwable*/
//	public static LogMsg doLog( final LogLvl 	level
//						      , final Logger 	kategorie
//						      , final String 	message
//			     			  ) {
//		LogMsg msg = new LogMsg(level, kategorie, message);
//		return doLog(msg);
//	}
//	/**Basis Interface fuer Throwable*/
//	public static LogMsg doLog( final LogLvl 	level
//						      , final Logger 	kategorie
//						      , final Throwable throwable
//			     			  ) {
//		LogMsg msg = new LogMsg(level, kategorie, throwable);
//		return doLog(msg);
//	}
	/**?!?!?STRANGE JOSEF ANDREAS BUG IM JOSEF LOGGING 2009-07-17,20 !?!?!?*/
	public static LogMsg doLog( final LogLvl 	level
						      , final Logger 	kategorie
						      , final String 	message
						      , final Throwable throwable
			     			  ) {
		LogMsg msg = new LogMsg(level, kategorie, message, throwable);
		return doLog(msg);
	}
	/**Basis Interface fuer msg with Throwable*/
	public static LogMsg doLog( final LogLvl 	level
						      , final Logger 	kategorie
						      , final String 	message
						      , final Throwable throwable
							  , final LMTag ... tags
			     			  ) {
		LogMsg msg = new LogMsg(level, kategorie, message, throwable, tags);
		return doLog(msg);
	}
	/**System use only, use MstFrameMain.setToTineLogExportInit_* */
	@Deprecated public static void _initTineLogExport() {
		LoggerImpl.__initTineLogExport();
	}
	public static boolean _hasTineLogAppender() {
		return LoggerImpl._hasTineLogAppender();
	}
}
