/**
 * ws:	Apr 10, 2007 - Apr 12, 2007 creation from scratch
 * ws:	Mai 14, 2008 - Mai 14, 2008	do not log System.out Texte von webstart Umgebung
 */
package de.desy.mst.libs.framework.mstapp.logging;

import java.util.logging.Handler;
import java.util.logging.LogRecord;

import de.desy.mst.libs.framework.mstapp.logging.appender.IFlushable;

/**
 * External (== X) Logging Handler, here only:<br>
 * Java Development Kit Logging Handler <br>
 * Ziel: JDK Logging wird in mstApp Logging gesehen<br>
 * Anwendungen: Tine, Josefs Device Server Wizard
 * Package Privat, da nur intern vom LoggerManager zu benutzen
 * @author ws
 * version 1.0.0
 */
class XLogHandler extends Handler {
	private final IFlushable logAppenderKern;

//=============================================================================

	/**
	 * @param logAppenderKern <> null
	 * @throws NullPointerException, falls logAppenderKern == null
	 */
	public XLogHandler(final IFlushable logAppenderKern) {
		super();
		if(logAppenderKern == null){
			final String msg = "Der logAppenderKern darf nicht null sein.";
			throw new NullPointerException(msg);
		}
		this.logAppenderKern = logAppenderKern;
	}

//=============================================================================

	/* (non-Javadoc)
	 * @see java.util.logging.Handler#close()
	 */
	@Override public void close() {flush();}	//ignoriere das closen selbst
	/* (non-Javadoc)
	 * @see java.util.logging.Handler#flush()
	 */
	@Override public void flush() {this.logAppenderKern.flush();}
	/* (non-Javadoc)
	 * @see java.util.logging.Handler#publish(java.util.logging.LogRecord)
	 */
	@Override public void publish(final LogRecord record) {
		try {
			publishImplementation(record);
		} catch (final Exception e) {
			Logger.SWFW.warn( "Problem mit Logging Message vom Java JDK '"
							+ record
							+ "'. Ignore.", e);
		}
	}
 
//=============================================================================

	/**unsecured*/
	private void publishImplementation(final LogRecord record) {
		final LogLvl	level     = LogLvl.translateFromJDK(record.getLevel());
		final Logger	kategorie = Logger.XLIB;
		final String	message   =   "[" + record.getLoggerName() + "] "
									+ record.getMessage();
		final Throwable	throwable = null;
		if(!isWebStartSystemOut(record, level, record.getMessage()))
			Logger.doLog(level, kategorie, message, throwable);
	}

	private boolean isWebStartSystemOut(final LogRecord record, final LogLvl level, final String message) {
		return level == LogLvl.debug
			&& record.getLoggerName().equals("com.sun.deploy")
			&& message.contains(LogMsg.THIS_RUN_RUNID.toString());
	}
}
