/**
 * Oct 10, 2008 - Oct 14, 2008	ws:	Creation from scratch
 * Jun 24, 2009 - Jun 26, 2009	ws: support Tagstate, blocking
 * 	Aug  6, 2009 - Aug  6, 2009	ws:	Eclipse asked for removing unnessary casts, so I did
 */
package de.desy.mst.libs.framework.mstapp.logging.appender;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;

import de.desy.mst.libs.framework.mstapp.logging.LMTag;
import de.desy.mst.libs.framework.mstapp.logging.LogLvl;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.filter.ALogTaggedFilter.TagState;
import de.desy.mst.libs.framework.mstapp.logging.filter.FilterModus;
import de.desy.mst.libs.framework.mstapp.logging.filter.StdLogFilter;

/**
 * Tabelle die die LogFilter Kofiguration anzeigt und gegebenenfalls editiert
 * @author ws
 * @version 1.0.0
 */
@SuppressWarnings({"rawtypes", "unchecked"})
class JTableForLogFilter extends JTable{
	private static final long serialVersionUID = 1L;
	private static final Object[] COLUMN_NAMES = new Object[]{"Was", "Wert"};
	private static enum NachLvlFiltern{JA, NEIN};
//	@SuppressWarnings("unchecked")
	private static enum RowTyp{
		  Tag(   new Object[] {"TagName"}, "Tagname"){
			  private final String eigentlicherTagName = "";	//hack
			  @Override public Object getWert(final StdLogFilter filter){
				  return getTagName(filter);
			  }
			  @Override public void setWert(final Object wert, final StdLogFilter filter, final ActionListener listener){}
			  @Override public Object[] calcRowData(final StdLogFilter filter){
				  return new Object[]{Tag.was, getTagName(filter)};
			  }
			  private String getTagName(final StdLogFilter filter) {
				  final LMTag tag = filter.getTag(TagState.ON);
				  return tag == null ? "" : tag.getName();
			  }
			  @Override public String toString() {return this.eigentlicherTagName;}
		  }
		, 	  Modus(   FilterModus.values(), "Modus inkl. Subfilter"){
			@Override public Enum getWert(final StdLogFilter filter){return filter.getModus();}
			@Override public void setWert(final Object wert, final StdLogFilter filter, final ActionListener listener){
				if(wert instanceof FilterModus && !filter.getModus().equals(wert)){
					filter.setModus((FilterModus) wert);
					listener.actionPerformed(createActionEvent(filter));
				}
			}
		}
		, Tagstate(   TagState.values(), "Nach Tag filtern"){
			@Override public Enum getWert(final StdLogFilter filter){return filter.getTagState();}
			@Override public void setWert(final Object wert, final StdLogFilter filter, final ActionListener listener){
				if(wert instanceof TagState && !getWert(filter).equals(wert)){
					filter.setTagState((TagState)wert);
					listener.actionPerformed(createActionEvent(filter));
				}
			}
		}
		, Blocking(   NachLvlFiltern.values(), "Nach Level und Kategorie filtern."){
			@Override public Enum getWert(final StdLogFilter filter){return filter.blocksAnyLevel() ? NachLvlFiltern.NEIN : NachLvlFiltern.JA ;}
			@Override public void setWert(final Object wert, final StdLogFilter filter, final ActionListener listener){
				if(wert instanceof NachLvlFiltern && ! getWert(filter).equals(wert)){
					filter.setBlockAnyLevel(wert == NachLvlFiltern.NEIN);
					listener.actionPerformed(createActionEvent(filter));
				}
			}
		}
		, Level(        LogLvl.values(), "Ab Level            "){
			@Override public Enum getWert(final StdLogFilter filter){return filter.getLogLvl();}
			@Override public void setWert(final Object wert, final StdLogFilter filter, final ActionListener listener){
				if(wert instanceof LogLvl && ! filter.getLogLvl().equals(wert)){
					filter.setLogLvl((LogLvl) wert);
					listener.actionPerformed(createActionEvent(filter));
				}
			}
		}
		, Kategorie(    Logger.values(), "Kategorie           "){
			@Override public Enum getWert(final StdLogFilter filter){return filter.getKategorie();}
			@Override public void setWert(final Object wert, final StdLogFilter filter, final ActionListener listener){
				if(wert instanceof Logger && ! filter.getKategorie().equals(wert)){
					filter.setKategorie((Logger) wert);
					listener.actionPerformed(createActionEvent(filter));
				}
			}
		}
		;
		//---------------------------------------------------------------------
		private final Object[] rowValueSet;
		private final String was;
		//---------------------------------------------------------------------
		private RowTyp(final Object[] rowValues, final String was){
			this.rowValueSet = rowValues;
			this.was = was;
		}
		//---------------------------------------------------------------------
		abstract public Object getWert(final StdLogFilter filter);
		abstract public   void setWert(final Object wert, final StdLogFilter filter, final ActionListener listener) ;

		public final TableCellEditor getTableCellEditor(final StdLogFilter filter, final ActionListener listener){
			final JComboBox comboBox = new JComboBox(this.rowValueSet);
			comboBox.addActionListener(new ActionListener(){
				 @Override public void actionPerformed(final ActionEvent evt) {
				        final Object selectedItem = comboBox.getSelectedItem();
				        if(selectedItem instanceof Enum)
				        	if(!getWert(filter).equals(selectedItem))
				        		setWert(selectedItem, filter, listener);
				    }
			});
			return  new DefaultCellEditor(comboBox);
		}
		public Object[] calcRowData(final StdLogFilter filter){
			return new Object[]{this.was, getWert(filter)};
		}
		//---------------------------------------------------------------------
		/**pp*/ boolean isImplemented(final StdLogFilter filter) {return true;}
		private static Object[][] sCalcRowData(final StdLogFilter filter) {
			final int n = RowTyp.values().length;
			final List<Object[]> list = new ArrayList<Object[]>(n);
			for (int i = 0; i < n; i++) {
				final RowTyp rowTyp = RowTyp.values()[i];
				if(rowTyp.isImplemented(filter))
					list.add(rowTyp.calcRowData(filter));
			}

			final Object[][] retval = new Object[n][];
			list.toArray(retval);
			return retval;
		}
		public static TableCellEditor getTableCellEditor(final int ordinal, final StdLogFilter filter, final ActionListener listener){
			return inRange(ordinal) ? RowTyp.values()[ordinal].getTableCellEditor(filter, listener)
									: null;
		}
		//---------------------------------------------------------------------
		private static boolean inRange(final int ordinal) {
			return 0 <= ordinal && ordinal < RowTyp.values().length;
		}
		private static ActionEvent createActionEvent(final StdLogFilter filter){
			return new ActionEvent(filter, ActionEvent.ACTION_PERFORMED, "StdFilter was modified");
		}
	}
//=============================================================================
	private final StdLogFilter filter;
	private final ActionListener filterModifiedListener;
	//=============================================================================
	/**
	 * @param filter intern wird eine Kopie dieses Filters benutzt, filter referenzfreies Objekt
	 * @param filterModifiedListener may be null, ein listener fuer potenzielle Filter Aenderungen
	 */
	public JTableForLogFilter(final StdLogFilter filter, final ActionListener filterModifiedListener) {
		super(RowTyp.sCalcRowData(filter), COLUMN_NAMES);

		this.filter = new StdLogFilter(filter);	//Kopie
		this.filterModifiedListener = filterModifiedListener;

		this.getModel().addTableModelListener(new TableModelListener(){
			@Override public void tableChanged(final TableModelEvent evt) {
//				System.out.println("Table Changed. Type: " + evt.getType() + " col, rowf-rowl:" + evt.getColumn() + ", " + evt.getFirstRow() + " - " + evt.getLastRow());
				if((evt.getLastRow() - evt.getFirstRow()) == 0)
					getModel().fireTableDataChanged();
			}
		});
		this.setDefaultRenderer(Object.class, new DefaultTableCellRenderer(){
			private static final long serialVersionUID = 1L;
			@Override public Component getTableCellRendererComponent(final JTable table, final Object value, final boolean isSelected, final boolean hasFocus, final int row, final int column) {
				final Component cmp = super.getTableCellRendererComponent(table, value, isSelected, hasFocus,row, column);
				final boolean enabled = JTableForLogFilter.this.isEditable(row, column);
//				System.out.println("JTableForLogFilter.getTableCellRendererComponent enabled, column, row " + enabled + ", " + column + ", " + row);
				cmp.setEnabled(enabled);
				return cmp;
			}
		});
	}
//=============================================================================
	/* (non-Javadoc)
	 * @see javax.swing.JTable#getCellEditor(int, int)
	 */
	@Override public TableCellEditor getCellEditor(final int row, final int column) {
		return isEditable(row, column) ?
			RowTyp.getTableCellEditor(convertRowIndexToModel(row), getFilter(), getFilterModifiedListener())
		  : null;
	}
	/* (non-Javadoc)
	 * @see javax.swing.JTable#getModel()
	 */
	@Override public AbstractTableModel getModel() {
		return ((AbstractTableModel)super.getModel());
	}
	private final StdLogFilter getFilter()      {return this.filter;}
	/**pp clone*/ StdLogFilter getFilterClone() {return new StdLogFilter(getFilter());}
	public void setFilter(final StdLogFilter wegwirfFilter) {
		getFilter().set(wegwirfFilter);
	}

//=============================================================================
	private boolean isEditable(final int row, final int col) {
		 if( row < 0 || convertColumnIndexToModel(col) != 1)	//nur editierspalte enabled
			 return false;
		final Object wert = getValueAt(row, col);
		if(wert instanceof FilterModus)
			return true;	//FilterModus immer setzbar
		if(getFilter().getModus() != FilterModus.STANDARD)
			return false;
		if(wert instanceof String)
			return false;
		if(wert instanceof TagState && getFilter().getTag(TagState.ON) == null)
			return false;
		if(getFilter().blocksAnyLevel() && (wert instanceof LogLvl || wert instanceof Logger))
			return false;
		return true;
	}
	private final ActionListener getFilterModifiedListener() {
		return this.filterModifiedListener;
	}
}
