/**
 *  Oct 24, 2005 - Nov 07, 2005	ws: Created on
 *  Feb 01, 2006 - Feb 06, 2006	ws: doLog -> doWrite
 *  Mar 14, 2007 - ...			ws: added:	if(lastWriteToSystemOut)
 *  												{System.out.flush();}
 *  Apr 17, 2007 - ...			ws: changed outString, printout to ln
 *  Jun 10, 2008 - Jun 10, 2008	ws: actually do flush on flush
 * 	Oct 15, 2008 - Oct 15, 2008	ws:	getName()
 * 	Sep  6, 2016 - Sep  6, 2016	ws:	add support for Logger suppression (MaWa 5-9-2016) 
 */
package de.desy.mst.libs.framework.mstapp.logging.appender;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.desy.mst.libs.framework.mstapp.logging.EinfacherLogFormatter;
import de.desy.mst.libs.framework.mstapp.logging.ILogFormatter;
import de.desy.mst.libs.framework.mstapp.logging.LogLvl;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Schreibt nach System.Out oder System.Err je nach Level
 * @author ws
 * @version 1.0.0
 */
public class LogAppenderJavaKonsole extends ALogAppenderFormatted {
	public volatile static Logger suppressedKategorie = null;

//=============================================================================

	public final String NAME = "LogAppenderJavaKonsole";
	private boolean lastWriteToSystemOut = false;	//synchronized on this

//=============================================================================

	public LogAppenderJavaKonsole() {this(new EinfacherLogFormatter());}

	public LogAppenderJavaKonsole(final ILogFormatter logFormatter) {
		super(logFormatter);

		//evtl spaeter in so eine Art Header Support
		System.out.println();
		System.out.println("===============================================================================\n");
		System.out.println(this.getClass().getName() + " startet mit folgenden Einstellungen:");
		final int nTabs = 1;
		System.out.println(getXMLBeschreibung(nTabs));
		System.out.println("===============================================================================");
		System.out.println();

	}

//=============================================================================

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppender#getName()
	 */
	@Override public String getName() {return "Log auf Java Konsole";}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.IFlushable#flush()
	 */
	@Override public void flush() {System.out.flush(); System.err.flush();}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ALogAppender#doWrite(de.desy.mst.libs.framework.mstapp.logging.LogMsg)
	 */
	@Override
	public synchronized void doWrite(final LogMsg msg) {
		if(msg.kategorie == suppressedKategorie)
			return;
		
		final String outString = getLogFormatter().format(msg);
		if(msg.level.ordinal() <= LogLvl.warn.ordinal()){
			System.out.println(outString);
			this.lastWriteToSystemOut = true;
		}else{
			if(this.lastWriteToSystemOut)	{System.out.flush();}
			System.err.println(outString);
			this.lastWriteToSystemOut = false;
		}
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ILogFilter#getBeschreibungAsElement()
	 */
	@Override public Element getBeschreibungAsElement(final Document document) {
		return getBeschreibungAsElement(document, this.NAME);
	} 
}
