/**
 *  Created on	Jan 31, 2006 - Feb 06, 2006
 *	Jul 26, 2006 - Aug 03, 2006	ws: neuer Copy XTor HauptLogFilter(HauptLogFilter filter)
 *	Jun 08, 2006 - Jun 30, 2009	ws:	support for Tags
 */
package de.desy.mst.libs.framework.mstapp.logging.filter;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import de.desy.mst.libs.framework.mstapp.logging.LMTag;
import de.desy.mst.libs.framework.mstapp.logging.LogLvl;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Der HauptLogFilter ist ein Standard Logfilter mit folgender Zusatzeigenschaft:<br>
 * Im HauptLogFilter Standard Mode lt er alles durch, was der BypassFilter durchlt,
 * auch wenn es die Hauptfilereinstellung nicht zulassen wrde.<br>
 * Zweck: Lifecycle Messages der Applikationen koennen unabhaengig von den
 * Applikations Filtereinstellungen protokolliert werden.
 *
 * @author ws
 * @version 1.0.0
 */
public class HauptLogFilter extends StdLogFilter {
	public  static final String NAME	= "HauptLogFilter";

	private static final String BYPASS_FILTER	= "BypassFilter";
	/**was diesen Filter pass'ed, pass'ed den HauptLogfilter (falls HauptLogfilter im Standard Modus)*/
	private final StdLogFilter bypassFilter;

//=============================================================================

	/**initialisiert den BypassFilter, call once routine*/
	@Deprecated protected static final  StdLogFilter createDefaultBypassFilter() {return createDefaultBypassFilter(null);}
	/**pp <br> initialisiert den BypassFilter, call once routine*/
	static final  StdLogFilter createDefaultBypassFilter(final LMTag tag) {
		final StdLogFilter bypassFilter = new StdLogFilter(new LogFilterKategorie	(BYPASS_FILTER_DEFAULT_LOG_KATEGORIE)
				                                   , new LogFilterLogLvl	(BYPASS_FILTER_DEFAULT_LOG_LVL)
												   , tag);
		return bypassFilter;
	}

//=============================================================================

	public HauptLogFilter() {this.bypassFilter	= createDefaultBypassFilter(null);}
	public HauptLogFilter(final LMTag tag) {
		super(tag);
		this.bypassFilter	= createDefaultBypassFilter(tag);
	}
	public HauptLogFilter(final Logger kategorie, final LogLvl lvl, final LMTag tag) {
		super(kategorie, lvl, tag);
		this.bypassFilter	= createDefaultBypassFilter(tag);
	}
	public HauptLogFilter(final LogFilterKategorie kategorie, final LogFilterLogLvl lvl) {
		super(kategorie, lvl);
		this.bypassFilter	= createDefaultBypassFilter(null);
	}
	/**Copy XTor, @throws NullpointerException, falls filter == null*/
	public HauptLogFilter(final HauptLogFilter filter) {
		super(filter);
		this.bypassFilter = new StdLogFilter(filter.getBypassFilter());
	}
	public HauptLogFilter(final FilterModus modus) {
		this();
		this.setModus(modus);
	} 


//=============================================================================

	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ALogFilter#passOnStandardModus(de.desy.mst.mstapp.logging.LogMsg)
	 */
	@Override protected boolean passOnStandardModus(final LogMsg logMsg)
		{return super            .passOnStandardModus(logMsg)	//HauptLogFilter als StdFilter
		     || getBypassFilter().pass               (logMsg);}	//Special Treatment of ByPassFilter

	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ALogFilter#getName()
	 */
	@Override protected String getName() {return NAME;}
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ALogFilter#decorateElement(org.w3c.dom.Document, org.w3c.dom.Element)
	 */
	@Override protected void decorateElement(final Document document, final Element element) {
		super.decorateElement(document, element);
		final Element BypassFilterElement       = getBypassFilter().getBeschreibungAsElement(document);
		final Element BypassFilterElementWrapIn = wrapInNamedElementWrapIn(document, BYPASS_FILTER, BypassFilterElement);
		element.appendChild(BypassFilterElementWrapIn);
	}

//=============================================================================

	/**never null<br>getBypassFilter.pass ist strker als die HauptFilterKriterien selbst,
	 * falls der Hauptfilter im Standard Modus ist).<br>
	 * Erlaubt die Lifecycle Loggings des Frameworks an dem normal Filter vorbeizuschleusen.
	 */
	public final StdLogFilter 	getBypassFilter() 	{return this.bypassFilter;}
	public final boolean  isBypassFilterEnabled()	{return FilterModus.NEVER_PASS != getBypassFilter().getModus();}
	public final void    setBypassFilterEnabled(final boolean enabled){
		if(enabled != isBypassFilterEnabled())	{
			if(enabled)
				{getBypassFilter().setModus(FilterModus.STANDARD);}//FilterModus.ALWAYS_PASS waere auch .enabled!!!
			else
				{getBypassFilter().setModus(FilterModus.NEVER_PASS);}
		}
	}
	/**Clones the Setting of the wegWirfFilter*/
	public void set(final HauptLogFilter wegWirfFilter) {
		super.set(wegWirfFilter);
		getBypassFilter().set(wegWirfFilter.getBypassFilter());
	}

	@Override public LogLvl getDefaultLogLvl   () {return StdLogFilter.DEFAULT_LOG_LVL;}
	@Override public Logger getDefaultKategorie() {return StdLogFilter.DEFAULT_LOG_KATEGORIE;}

	@Override public boolean equals(final Object obj) {
		if(obj instanceof HauptLogFilter){
			final HauptLogFilter filter = (HauptLogFilter) obj;
			return super.equals(filter)
			    && this.getBypassFilter().equals(filter.getBypassFilter())
			    ;
		}
		return false;
	}
	@Override public int hashCode() {
		return super.hashCode()
		     + this.getBypassFilter().hashCode()
		     ;
	}
}
