/**
 *  Created on	Jan 12, 2006 - Feb 02, 2006
 *  			Jul 30, 2006 - Aug 01, 2006, ws: implement: set, equals, hashCode
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 *  Oct 30, 2008 - Nov  4, 2008	ws:	added setFromSaveString(...), getSaveString(...)
 */
package de.desy.mst.libs.framework.mstapp.logging.filter;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.desy.mst.libs.framework.mstapp.ini.PrefStorage;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * <b>Leaffilter fuer Logger Kategorien.</b><br>
 * pass == true, falls die Kategorie der Message die Filter Kategorie ist (oder eine Subkategorie der Filter Kategorie).<br>
 * <br>
 *
 * @author ws
 * @version 1.0.0
 */
public class LogFilterKategorie extends ALogFilter {
	private static final String NAME = "LogKategorienFilter";
	private static final String ATTRIBUTNAME = "Kategorie";

	private static final Logger DEFAULT_KATEGORIE = Logger.Any;
	private Logger kategorie;	//never null

//=============================================================================

	public LogFilterKategorie() {this(DEFAULT_KATEGORIE);}
	public LogFilterKategorie(final Logger kategorie) {
		super();
		this.setKategorie(kategorie);
	}
	public LogFilterKategorie(LogFilterKategorie kategorie) {
		super();
		this.set(kategorie);
	}

//=============================================================================

	/**nie null*/
	public Logger getKategorie() {return this.kategorie;}
	public void setKategorie(final Logger kategorie) {
		this.kategorie = (kategorie == null) ? DEFAULT_KATEGORIE : kategorie;
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.filter.ALogFilter#getSaveString(java.lang.String, java.lang.String)
	 */
	@Override protected String getSaveString(final String appenderGruppe, final String saveString) {
		final String key = makeKey(appenderGruppe);
		final String value = getKategorie().toString();
		return PrefStorage.serialize(saveString, key, value);
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.filter.ALogFilter#setFromSaveString(java.lang.String, java.lang.String)
	 */
	@Override protected void setFromSaveString(final String appenderGruppe, final String saveString, final Object defaultKategorie) {
		final String key = makeKey(appenderGruppe);
		final String value = PrefStorage.extract(saveString, key);
		try {
			final Logger kategorie = (value == null || value.isEmpty()) ?
									 (Logger) defaultKategorie : Logger.valueOf(value);
			this.setKategorie(kategorie);
		} catch (final IllegalArgumentException ex) {
			Logger.SWFW.warn("Kann den Filter " + this.getName()
						   + " nicht zu der Kategorie '" + value
						   + "' setzen ignorier den Wert und belasse ihn auf '"
						   + getKategorie().toString() + "'.");
		} catch (final NullPointerException ex) {
			Logger.SWFW.warn("Kann den Filter " + this.getName()
						   + " nicht zu dem Kategorie 'null' setzen"
						   + ", belasse ihn auf '" + getKategorie().toString()
						   + "'.");
		}
	}
	//-------------------------------------------------------------------------
	private String makeKey(final String appenderGruppe) {
		return makeKey(appenderGruppe, ATTRIBUTNAME);
	}

//=============================================================================

	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ALogFilter#passOnStandardModus(de.desy.mst.mstapp.logging.LogMsg)
	 */
	@Override protected boolean passOnStandardModus(final LogMsg logMsg){
		return (logMsg == null) ? false :logMsg.getKategorie().isKategorie(   getKategorie()   );
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ALogFilter#getName()
	 */
	@Override protected String getName() {return NAME;}
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ALogFilter#decorateElement(org.w3c.dom.Document, org.w3c.dom.Element)
	 */
	@Override protected void decorateElement(Document document, Element element) {
		element.setAttribute(ATTRIBUTNAME, getKategorie().toString());
	}

	/**Copy Semantik*/
	public void set(final LogFilterKategorie logFilterKategorie){
		super.set(logFilterKategorie);
		final Logger kategorie = (logFilterKategorie == null) ? (null)
															  : (logFilterKategorie.getKategorie());
		this.setKategorie(kategorie);
	}

	@Override public boolean equals(Object obj) {
		if(obj instanceof LogFilterKategorie){
			LogFilterKategorie filter = (LogFilterKategorie) obj;
			return super.equals(filter)
			    && this.getKategorie().ordinal() == filter.getKategorie().ordinal()
			    ;
		}
		return false;
	}
	@Override public int hashCode() {
		return this.getKategorie().ordinal()
		     ;
	}
}

