/**
 * 	May 20, 2008 - Jun 12, 2008	ws:	Creation from inner class of AMsgModel
 * 	Jun 17, 2008 - Jun 17, 2008	ws: added: hideColumnByDefault
 * 	Jun 23, 2008 - Jun 23, 2008	ws:	cosmetics
 * 	Jul  2, 2008 - Jul  9, 2008	ws:	added filterNotSelect support
 * 	Aug 12, 2008 - Aug 14, 2008	ws:	added Schreib Unterstuetzung
 *	Jan 15, 2008 - Jan 15, 2008	ws:	serializable
 *	Feb 26, 2009 - Feb 27, 2009	ws:	mightBeMultiline
 * 	Mar 11, 2009 - Mar 11, 2009	ws:	added FFEValueRegEx.createFromSubstring
 * 	Mar 12, 2009 - Mar 23, 2009	ws:	added FFEHasSubstring
 * 	Jul 14, 2009 - Jul 14, 2009	ws:	fixed some Eclipse 3.5, 3.2 warnings
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.io.Serializable;
import java.text.ParseException;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.swing.JTable;
import javax.swing.RowFilter.Entry;
import javax.swing.table.TableModel;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.gui.AFFE.AFFEFactory;
import de.desy.mst.libs.framework.mstapp.logging.gui.AFieldFilterGUIFactory.AsJTextArea;
import de.desy.mst.libs.framework.mstapp.logging.gui.LogMsgAuswertung.FieldInfo;
import de.desy.mst.libs.framework.mstapp.util.MstDateFormat;
import de.desy.mst.libs.framework.mstapp.util.MstDateFormatISOParser;

/** Beschreibt ein Feld vom Typ in LogMsg vom Typ L fuer ein JTable */
@SuppressWarnings({"rawtypes", "unchecked"})
abstract class AField<E extends Serializable, L> implements Serializable, Cloneable{
	private static final long serialVersionUID = 1L;
	/**include == true falls subset of ffes, leerer FieldFilter immer true*/
	class FieldFilter implements IFilter
					  		   , Comparable<FieldFilter>	//pretty print
							   , Cloneable
							   , Serializable{
		private static final long serialVersionUID = 1L;
		private final LinkedList<AFFE<E, L>> ffes;
		private final LoggingTable<L, ?> table;
		//-----------------------------------------------------------------
		FieldFilter(final LoggingTable<L, ?> table) {
			super();
			this.table = table;
			this.ffes  = new LinkedList<AFFE<E, L>>();
		}
		//-----------------------------------------------------------------
		public AField<E, L> getField() {return AField.this;}
		public int size() {return getFFEs().size();}

		@Override public FieldFilter clone() throws CloneNotSupportedException {
			return (FieldFilter)super.clone();
		}
		/**@return false falls schon vorhanden, ffe enthaelt invertierte zuerst*/
		public boolean add(final AFFE<E, L> ffe) {
			if(getFFEs().contains(ffe))
				return false;
			if(ffe.getInverted())
				getFFEs().addFirst(ffe);
			else
				getFFEs().addLast(ffe);
			return true;
		}

		/**or of normal allFieldFilterElements OR and of inverted ones*/
		public boolean passes(final E value){
			final List<AFFE<E, L>> ffes = getFFEs();
			final List<AFFE<E, L>> ffesInverted =
				new LinkedList<AFFE<E,L>>();
			final List<AFFE<E, L>> ffesNotInverted =
				new LinkedList<AFFE<E,L>>();
			for (final AFFE<E, L> fieldFilterElement : ffes) {
				if(fieldFilterElement.getInverted())
					ffesInverted   .add(fieldFilterElement);
				else
					ffesNotInverted.add(fieldFilterElement);
			}
			return passesByOr(value, ffesNotInverted)
				|| (ffesInverted.size() > 0 && passesByAnd(value, ffesInverted));
		}
		private boolean passesByOr(final E value, final List<AFFE<E, L>> ffes) {
			for (final AFFE<E, L> fieldFilterElement : ffes) {
				if(fieldFilterElement.passes(value))
					return true;
			}
			return false;
		}
		private boolean passesByAnd(final E value, final List<AFFE<E, L>> ffes) {
			for (final AFFE<E, L> fieldFilterElement : ffes) {
				if(!fieldFilterElement.passes(value))
					return false;
			}
			return true;
		}
		public boolean remove(final AFFE<E, L> ffe){
			return getFFEs().remove(ffe);
		}
		public void removeAll() {getFFEs().clear();}
		public String getStringEncoded() {
			final List <AFFE<E, L>> ffes = getFFEs();
			if(ffes.size() <= 0)
				return "*";
			final String entrySep = getEntrySeparatorOutput();
			final StringBuilder sb = new StringBuilder();
			sb.append(ffes.get(0).asString());
			for (int i = 1; i < ffes.size(); i++) {
				sb.append(entrySep)
				  .append(ffes.get(i).asString());
			}
			return sb.toString();
		}
		/**
		 * @param text
		 * @throws IllegalArgumentException, falls text nicht geparsed werdend kann
		 */
		public final void setStringEncoded(final String text)
										throws IllegalArgumentException {
			this.removeAll();
			if(		text == null
				||	text.trim().isEmpty()
				||	text.trim().equals("*")
			  )
				return;
			final String[] entries = text.split(getEntrySeparatorRegex());
			for (int i = 0; i < entries.length; i++) {
				final AFFE<E, L> ffe = getField().fromString(entries[i]);
				if(ffe != null)
					this.add(ffe);
			}
		}
		/* (non-Javadoc)
		 * @see java.lang.Comparable#compareTo(java.lang.Object)
		 */
		@Override public int compareTo(final FieldFilter other) {
			final Integer tLength =  this.getBeschreibung().length();
			final Integer oLength = other.getBeschreibung().length();
			return tLength.compareTo(oLength);
		}

		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.logging.gui.LoggingTableDisplay.AFilter#getBeschreibung()
		 */
		@Override public String getBeschreibung() {
			//or of normal allFieldFilterElements OR and of inverted ones
			//inverted ones zuerst durch add
			final int size = filtert() ? getFFEs().size() : 0;
			final StringBuilder sb = new StringBuilder(10 * size);
			sb.append("( ")
			  .append(getField().getName())
			  .append(" gefiltert nach ");
			wrap1stFFE(sb, size);//takes care of no first ffe at all
			for (int i = 1; i < size; i++) {
				sb.append(getFFEs().get(i).getInverted()? " UND " : " ODER ");
				wrapFFE(sb, i);
			}
			sb.append(" )");
			return sb.toString();
		}

		public String getProblems() {
			final StringBuilder sb = new StringBuilder();
			final List<AFFE<E, L>> ffes = getFFEs();
			for (final AFFE<E, L> fieldFilterElement : ffes) {
				final String ttt = fieldFilterElement.getProblems();
				final String prefix = (sb.length() == 0 && ttt.isEmpty())
										? ""
										: "\n\n";
				sb.append(prefix)
				  .append(ttt);
			}
			return sb.toString();
		}

		void wrap1stFFE(final StringBuilder sb, final int size) {
			if(size <= 0)
				sb.append("*");
			else
				wrapFFE(sb, 0);
		}
		void wrapFFE(final StringBuilder sb, final int i) {
			final String asString = getFFEs().get(i).asString();
			sb.append(" '").append(asString).append("'");
		}
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.logging.gui.LoggingTableDisplay.AFilter#include(javax.swing.RowFilter.Entry)
		 */

		@Override public boolean include(final Entry<? extends TableModel, ? extends Integer> entry) {
			return (!filtert()) || passes(getValue(entry));
		}
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.logging.gui.IFilter#include(E)
		 */
		@Override public boolean include(final int row){
			return (!filtert()) || passes(getValue(row));
		}
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.logging.gui.LoggingTableDisplay.AFilter#filtert()
		 */
		@Override public boolean filtert() {
			final int size = getFFEs().size();
			return 0 < size;
		}

		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override public String toString() {return this.getBeschreibung();}
		//-----------------------------------------------------------------
		LinkedList<AFFE<E, L>> getFFEs(){return this.ffes;}
		public E getValue(final int row){
			final JTable table = getTable();
			final int column = table.getColumn(getField())
									.getModelIndex();
			final Object value = table.getModel().getValueAt(row, column);
			return (E) value;
		}
		
		private E getValue(final Entry<? extends TableModel, ? extends Integer> entry) {
			final int column = getTable().getColumn(getField())
										 .getModelIndex();
			final Object value = entry.getValue(column);
			return (E) value;
		}
		final LoggingTable<L, ?> getTable() {return this.table;}
	}
	/**finete elements: filtert nicht wenn alle elemente drin sind
	 * aktzeptiert nur FFEValueEquality Objekte als Elemente*/
	class FieldFilterSized extends FieldFilter{
		private static final long serialVersionUID = 1L;
		final int maxSize;
		FieldFilterSized(final LoggingTable<L, ?> table) {
			this(table, -1);
		}
		FieldFilterSized(final LoggingTable<L, ?> table, final int maxSize) {
			super(table);
			this.maxSize = calcMaxSize(table, maxSize);
		}
		
		private int calcMaxSize(final LoggingTable<?, ?> table, final int maxSize) {
			if(maxSize > 0)
				return maxSize;

			final AField field = getField();
			final ALogSource<?, ?> logSource = table.getModel()
					    			 			    .getLogSource();
			final long anzahl = logSource.hasAuswertung()?	//against prematuer update
												logSource.getAuswertung()
														 .getFieldInfo(field)
														 .getAnzahl()
											  : 1;
			//$ANALYSIS-IGNORE
			return (int) Math.min(anzahl, Integer.MAX_VALUE);
		}
		public void addAll(){
			final AField<E, L> field = getField();
			final FieldInfo<E,L> fieldInfo = getTable().getModel()
										    		   .getLogSource()
										    		   .getAuswertung()
										    		   .<E>getFieldInfo(field);
			final Set<E> values = fieldInfo.getElements();
			for (final E e : values)
				if(e != null)
					this.add(new FFEValueEquality<E, L>(field, e, false));
				else
					Logger.SWFW.error("Feld " + field.getName() + "hat unerwarteten Wert null.", new IllegalArgumentException());
		}
		/**@throws IllegalArgumentException wenn 'ffe.getClass() != FFEValueEquality.class'*/
		@Override public boolean add(final AFFE<E, L> ffe) {
			if(ffe.getClass() == FFEValueEquality.class)
				return super.add(ffe);
			final String msg = "FieldFilterSized nur fuer"
							 + " FFEValueEquality Objekte.";
			throw new IllegalArgumentException(msg);
		}
		@Override public boolean filtert() {
			return  super.filtert()
				 && size() < this.maxSize;
		}
	}

	public static final boolean IS_WIDE_EDITOR 	= true;
	public static final boolean IS_DISCRETE 	= true;

	private final String name;
	private final String description; // z.B. Tooltip
	private final Class<E> clazz;
	private final E valueForPrefSize;
	private final int index;
	private final AFieldFilterGUIFactory<E, L, ?> filterFactory;
//-------------------------------------------------------------------------
	public AField( final int index
				 , final String name
				 , final String description
				 , final Class<E> clazz
				 , final E valueForPrefSize
				 , final AFieldFilterGUIFactory<E, L, ?> filterFactory) {
		super();
		this.index = index;
		this.name = name;
		this.description = description;
		this.clazz = clazz;
		this.valueForPrefSize = valueForPrefSize;
		this.filterFactory = filterFactory;
	}
	//---------------------------------------------------------------------
	/** logMsg !- null */
	abstract E get(final L logMsg);
	/**default ignore*/
	public void set(final L logMsg, final E value) {}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override public int hashCode() {
		final int prime = 31; int result = 1;
		result = prime * result
				+ ((this.clazz == null) ? 0 : this.clazz.hashCode());
		result = prime
				* result
				+ ((this.description == null) ? 0 : this.description.hashCode());
		result = prime * result + this.index;
		result = prime * result
				+ ((this.name == null) ? 0 : this.name.hashCode());
		return result;
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override public String toString()	{return this.getName();}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	
	@Override public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final AField other = (AField) obj;
		if (this.clazz == null) {
			if (other.clazz != null)
				return false;
		} else if (!this.clazz.equals(other.clazz))
			return false;
		if (this.description == null) {
			if (other.description != null)
				return false;
		} else if (!this.description.equals(other.description))
			return false;
		if (this.index != other.index)
			return false;
		if (this.name == null) {
			if (other.name != null)
				return false;
		} else if (!this.name.equals(other.name))
			return false;
		return true;
	}
	//---------------------------------------------------------------------
	public String getName() 		{return this.name;}
	public String getDescription()	{return this.description;}
	public Class<E> getClazz() 		{return this.clazz;}
	/**Beide Objekte vom Typ E eigentlich*/
	public E getValueForPrefSize(final E value) {
		if(			value == null
				||	asString(value).length()
					<
					asString(this.valueForPrefSize).length()
		   ) {
			return this.valueForPrefSize;
		} else {
			return value;
		}
	}
	public int getIndex() {return this.index;}
	public String getRangeToken()			{return "NOT IMPLEMENTED";}
	public String getEntrySeparatorOutput()	{return "\n";}
	public String getEntrySeparatorRegex() {
		return getFieldFilterGUIFactory().isWideEditor() ? "[\n]" : "[,\n]";
	}
	public FieldFilter createFieldFilter(final LoggingTable<?, ?> tableIn){
		final LoggingTable<L, ?> table = (LoggingTable<L, ?>) tableIn;
		return isDiscrete()	? new FieldFilterSized(table)
							: new FieldFilter     (table);
	}

	public String asString(final E e) {return (e==null)?"null":e.toString();}
	/**o typically the RefValue of an FFE, i.e. vom Typ E oder String*/
	public String asStringLong (final Object o)  {return (o==null)?"null":o.toString();}
	public String asStringLongM(final L l)  {return asString(get(l));}
	public String serialize(final L l)	  {return asStringLongM(l);}
	public E      asValue(final String entry){return null;}
	public E getDefaultVon() {return null;}
	public E getDefaultBis() {return null;}

	public AFFE<E, L> fromString (String entry) throws IllegalArgumentException {
		try {
			if(entry.contains(getRangeToken())){
				final int stop1 = entry.indexOf(getRangeToken());
				final String sValV = entry.substring(0, stop1).trim();
				final int start2 = stop1 + getRangeToken().length();
				final String sValB = entry.substring(start2).trim();
				return new FFERange<E, L>(this, asValue(sValV), asValue(sValB));
			}

			final boolean isInverted = entry.startsWith(AFFE.INVERTED_TOKEN);
			if(isInverted)
				entry = entry.substring(AFFE.INVERTED_TOKEN.length());

			final E asValue = asValue(entry);
			if(asValue instanceof String)
				return fromString(entry, (String)asValue, isInverted);
			else {
				final String infix = AFFE.INFIX_TERMINATOR;
				final int infixPos = entry.indexOf(infix);
				if(infixPos < 0)
					return new FFEValueEquality<E, L>(this, asValue, isInverted);
				else
					return new FFEValueEquality<E, L>(this, asValue(entry.substring(infixPos + infix.length())), isInverted);
			}
		} catch (final Exception e) {
			throw new IllegalArgumentException(e.getMessage());
		}
	}
	private AFFE<E, L> fromString(final String entry, final String entryOhneInversion, final boolean isInverted) {
		if(entryOhneInversion.startsWith(FFEValueEquality.INFIX)) {
			final String valueString = entryOhneInversion.substring(FFEValueEquality.INFIX.length());
			return new FFEValueEquality<E, L>(this, asValue(valueString), isInverted);
		}
		else if(entryOhneInversion.startsWith(FFEHasSubstring.INFIX)){
			final String valueString = entryOhneInversion.substring(FFEHasSubstring.INFIX.length());
			return toAFFE(createFFEHasSubstring(isInverted, asValue(valueString)));
		}
		else if(entryOhneInversion.startsWith(FFEValueRegEx.INFIX)){
			final String valueString = entryOhneInversion.substring(FFEValueRegEx.INFIX.length());
			return new FFEValueRegEx<E, L>(this, valueString, isInverted);
		}
		else{
			final String valueString = entryOhneInversion;
			return new FFEValueEquality<E, L>(this, asValue(valueString), isInverted);
		}
	}
	private AFFE<E, L> toAFFE(final AFFE ffe) {return ffe;}
	private FFEHasSubstring createFFEHasSubstring(final boolean isInverted, final E asValue) {
		return new FFEHasSubstring(this, asValue.toString(), isInverted);
	}
	public final E min(final E lhs, final E rhs){
		if(lhs == null)
			return rhs;
		if(rhs == null)
			return lhs;
		return lhsIsSmaller(lhs, rhs) ? lhs : rhs;
		}
	public final E max(final E lhs, final E rhs){
		if(lhs == null)
			return rhs;
		if(rhs == null)
			return lhs;
		return lhsIsSmaller(lhs, rhs) ? rhs : lhs;
	}
	boolean lhsIsSmaller(final E lhs, final E rhs){
		return (lhs.toString().length() < rhs.toString().length());
	}
	public boolean isDiscrete() 		 {return false;}
	public boolean hideColumnByDefault() {return false;}
	public final <M extends AMsgModel<L>> AFieldFilterGUIFactory<E, L, M> getFieldFilterGUIFactory(){
		final AFieldFilterGUIFactory filterFactory = this.filterFactory;
		return filterFactory;
	}

//=============================================================================

	/**nur Date get(L logMsg) muss noch spezifieziert werden*/
	abstract static class ADateField<L, M extends AMsgModel<L>> extends AField<Date, L> {
		private static final long serialVersionUID = 1L;
		private static final String RANGE_TOKEN = " - ";// need blanks!
		private final String offsetPartNow 	= createOffsetPartNow();
		private final String datePartNow 	= createDatePartNow();
//(index, DATE_NAME, "Datum zur Zeit des Eintrags", Date.class, new Date(), fieldFilterFactory)
			//DateFilterFactory Anpassung
		ADateField(final int index, final String name, final String editorText) {
			super(index, name, "Datum/Zeit", Date.class, new Date(), createDateFilterFactory(editorText));
		}

			//damit in super(...) passt
		private static AsJTextArea createDateFilterFactory(final String editorText) {
			return new AFieldFilterGUIFactory.AsJTextAreaDate(editorText, "Z.B.: \"2008-03-03 14:26:52.637, 2008-03-22 14:26:52, 10:18:33 - 10:18:40\"", false);
		}

		/**besseres toString fuer Paste*/
		class ISODate extends Date{
			private static final long serialVersionUID = 1L;
			public ISODate(final long date) {super(date);}
			@Override public String toString() {
				return asStringAbbreviated(this);
			}
		}

		//-----------------------------------------------------------------
//		@Override Date get(final L logMsg) {
//			return new ISODate(0/*logMsg.getDate().getTime()*/);
//		}

		@Override public boolean lhsIsSmaller(final Date lhs, final Date rhs) {
			return lhs.getTime() < rhs.getTime();
		}

		@Override public String asString(final Date date) {
			return (date == null)	? super.asString(date)
									: asStringAbbreviated(date);
		}

		private String asStringAbbreviated(final Date date) {
			return MstDateFormat.ISO
								.format(date)
								.replace(getDatePartNow(), "")
								.replace(getOffsetPartNow(), "");
		}

		@Override public String asStringLong(final Object objectMayBeDate) {
			if(!(objectMayBeDate instanceof Date))
				return super.asStringLong(objectMayBeDate);
			final Date date = (Date) objectMayBeDate;
			return MstDateFormat.ISO.format(date) + " (" + date.getTime() + ")";
		}

		@Override public String getEntrySeparatorRegex() {return "[,\n]";}

		@Override public String getRangeToken() {return RANGE_TOKEN;}

		@Override public Date asValue(final String dateText) {
			if(dateText.isEmpty())
				return null;
			if(!hasDatePart(dateText))
				return asValue(getDatePartNow() + dateText.trim());
			if(!hasOffsetPart(dateText))
				return asValue(dateText.trim() + getOffsetPartNow());
			try {
				return MstDateFormatISOParser.parse(dateText.trim());
			} catch (final ParseException e) {
				throw new IllegalArgumentException(e.getMessage());
			}
		}

		private String getDatePartNow() {return this.datePartNow ;}

		private String createDatePartNow() {
			final String now = MstDateFormat.ISO.format(new Date());
			final String datePart = now.substring(0, now.indexOf(" ") + 1);
			return datePart;
		}

		private boolean hasDatePart(final String dateText) {
			return dateText.contains("-");
		}

		private String getOffsetPartNow() {return this.offsetPartNow ;}

		private String createOffsetPartNow() {
			final String now = MstDateFormat.ISO.format(new Date());
			final String offsetPart = now.substring(now.lastIndexOf(" "));
			return offsetPart;
		}

		private boolean hasOffsetPart(final String dateText) {
			return dateText.contains(" +") || dateText.contains(" -");
		}

		@Override public Date getDefaultVon() {return new Date(0);}

		@Override public Date getDefaultBis() {return new Date(Long.MAX_VALUE);}
	}//END ADateField

	/**nur String get(L logMsg) muss noch spezifieziert werden*/
	abstract static class AStringField<L, M extends AMsgModel<L>> extends AField<String, L> {
		private static final long serialVersionUID = 1L;
		final boolean isDiscrete;
		AStringField(final int index, final String name, final String description, final boolean isDiscrete, final int nCharsLongMessage, final String editorText, final String editorToolTip, final boolean isWideEditor) {
			super(index, name, description, String.class
				, makeString(nCharsLongMessage)
				, isDiscrete ? new AFieldFilterGUIFactory.AsJList    <String, L, AMsgModel<L>> (editorText, editorToolTip)
						     : new AFieldFilterGUIFactory.AsJTextArea<String, L, AMsgModel<L>> (editorText, editorToolTip, isWideEditor)
			);
			this.isDiscrete = isDiscrete;
		}
		private static String makeString(final int nCharsLongMessage) {
			final StringBuilder sb = new StringBuilder(nCharsLongMessage);
			final char averageChar = 'n';
			while(sb.length() < nCharsLongMessage) sb.append(averageChar);
			return sb.toString();
		}
		//---------------------------------------------------------------------
		@Override public String  asValue(final String entry) {return entry;}
		@Override public boolean isDiscrete() {return this.isDiscrete;}
		@Override public boolean mightBeMultiline() {return true;}
		@Override public boolean mightBeMultiline(final String value) {
			return value != null && value.contains("\n");
		}
	}//END AStringField

	public boolean supportsInverted() {return true;}
	public boolean mightBeMultiline() {return false;}
	public boolean mightBeMultiline(final E value) {return mightBeMultiline();}
	public boolean isStringField() {return String.class.equals(getClazz());}
	/**might return null, if not possible to create*/
	public AFFEFactory<E,L> createDefaultFFEFactory() {
		return new AFFEFactory<E,L>(){
			@Override public AFFE<E, L> create() {
				try 								{return fromString("");}
				catch (final IllegalArgumentException ex) {return null;}
			}
		};
	}
	//---------------------------------------------------------------------
}
