/**
 * 	May 28, 2008 - Jun  9, 2008	ws:	extracted from LoggingTableDisplay
 * 	Jun 25, 2008 - Jun 25, 2008	ws:	added getBeschreibung(enforce)
 * 	Mar 20, 2009 - Mar 20, 2009	ws:	cleanup
 * 	Jul 14, 2009 - Jul 14, 2009	ws:	fixed some Eclipse 3.5 warnings
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.swing.RowFilter.Entry;
import javax.swing.table.TableModel;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.gui.AField.FieldFilter;

/**include if all FieldFilter say true*/
@SuppressWarnings({"rawtypes", "unchecked"})
class AndOfFieldsFilter implements IFilter{
	private final Map<AField<?, ?>, AField<?, ?>.FieldFilter> map
						= new HashMap<AField<?, ?>, AField<?, ?>.FieldFilter>(1);
	public AndOfFieldsFilter() {}
	public AndOfFieldsFilter(final AndOfFieldsFilter filter) {
		this();
		this.setFilter(filter);
	}
	//-----------------------------------------------------------------
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.LoggingTableDisplay.FilterHandler.AFilter#getBeschreibung()
	 */
	@Override public String getBeschreibung() {return getBeschreibung(false);}
	/**@param enforce == true: ignore high level filtert()*/
	public String getBeschreibung(final boolean enforce) {
		final AField<?, ?>.FieldFilter[] dieFilter = getMapAsArray();
		final int size = dieFilter.length;
		if(size == 0 || (!enforce && !this.filtert())  )
			return "Alle Zeilen werden akzeptiert.";
		final StringBuffer sb = new StringBuffer(15 * size);
		Arrays.sort(dieFilter);
		extractBeschreibung(dieFilter, 0, sb);
		for (int i = 1; i < dieFilter.length; i++) {
			if(dieFilter[i].filtert()){
				sb.append(" UND ");
				extractBeschreibung(dieFilter, i, sb);
			}
		}
		return sb.toString();
	}
	public String getProblems() {
		final StringBuilder sb = new StringBuilder();
		for (final AField<?, ?>.FieldFilter fieldFilter : getMap().values()) {
			final String problems = fieldFilter.getProblems();
			final String prefix = (	sb.length() == 0
								|| 	problems.isEmpty()) ? ""
														: "\n\n====\n\n";
			sb.append(prefix)
			  .append(problems);
		}
		return sb.toString();
	}
	public String getStringEncoded(final AField<?, ?> field) {
		final AField<?, ?>.FieldFilter fieldFilter = get(field);
		return (fieldFilter == null) ? "*"//nicht filtern gleich wildcard *
									 : fieldFilter.getStringEncoded();
	}
	private StringBuffer extractBeschreibung(
									  final AField<?, ?>.FieldFilter[] dieFilter
									, final int iFilter
									, final StringBuffer sb) {
		return sb.append(dieFilter[iFilter].getBeschreibung());
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.LoggingTableDisplay.FilterHandler.AFilter#include(int)
	 */
	@Override public boolean include(final int row) {
		final Collection<AField<?, ?>.FieldFilter> filterCol = getMap().values();
		for (final AField<?, ?>.FieldFilter filter : filterCol) {
			if(!filter.include(row))
				return false;
		}
		return true;
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.LoggingTableDisplay.FilterHandler.AFilter#include(javax.swing.RowFilter.Entry)
	 */
	@Override public boolean include(final Entry<? extends TableModel, ? extends Integer> entry) {
		final Collection<AField<?, ?>.FieldFilter> filterCol = getMap().values();
		for (final AField<?, ?>.FieldFilter filter : filterCol) {
			if(!filter.include(entry))
				return false;
		}
		return true;
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.LoggingTableDisplay.FilterHandler.AFilter#filtert()
	 */
	@Override public boolean filtert() {
		final Collection<AField<?, ?>.FieldFilter> filterCol = getMap().values();
		for (final AField<?, ?>.FieldFilter filter : filterCol) {
			if(filter.filtert())
				return true;
		}
		return false;
	}
	//-----------------------------------------------------------------
	/**null is ignored*/
	public <E extends Serializable, L, M extends AMsgModel<L>>
	void add(final AField<E, L> field, final AFFE<E, L> ffe, final LoggingTable<L, M> tableIn){
		if(!this.contains(field))
			this.add(field.createFieldFilter(tableIn));
		final AField<E, L>.FieldFilter fieldFilter = this.get(field);
		fieldFilter.add(ffe);
	}
	/**null is ignored*/
	public AField<?, ?>.FieldFilter add(final AField<?, ?>.FieldFilter e){
		if(e == null)
			return null;
		remove(e);
		return getMap().put(e.getField(), e);
	}
	private void remove(final AField<?, ?>.FieldFilter e) {
		if(e == null)
			return;	//nothing to remove
		getMap().remove(e.getField());
	}
	public boolean contains(final AField<?, ?> field){
		return getMap().containsKey(field);
	}
	public <E extends Serializable, L> boolean contains(final AField<E, L> field, final AFFE<E, L> ffe){
		return getMap().containsKey(field) && get(field).getFFEs().contains(ffe);
	}

//	@SuppressWarnings("unchecked")
	public <E extends Serializable, L> AField<E, L>.FieldFilter get(final AField<E, L> field){
		if(contains(field))
			return (FieldFilter) getMap().get(field);
		else
			return null;
	}

	/**<Object> ungenerified version*/
	public AField<Serializable, ?>.FieldFilter get_(final AField<Serializable, ?> field){
		return get(field);
	}
	public void clear()	{getMap().clear();}
	/**Ueberschreibt den Filter*/
	public final void setFilter(final AndOfFieldsFilter filter) {
		clear();
		filter.clean();
		for (final AField<?, ?>.FieldFilter fieldFilter : filter.getMap().values()) {
			try {
				this.add(fieldFilter.clone());
			} catch (final CloneNotSupportedException e) {
				final String msg = "Kann den Feldfilter nicht klonen: "
								 + fieldFilter;
				Logger.SWFW.warn(msg);
				this.add(fieldFilter);
			}
		}
	}
	/**nimmt alle nicht filternden subfilter raus*/
	public void clean() {
		final Collection<AField<?, ?>.FieldFilter> filterCol = getMap().values();
		for (final AField<?, ?>.FieldFilter filter : filterCol) {
			if(!filter.filtert()){
				this.remove(filter);
				clean();	//da die filterCol jetzt invalid redo
				break;
			}
		}
	}
	//-----------------------------------------------------------------
	private Map<AField<?, ?>, AField<?, ?>.FieldFilter> getMap() {
		return this.map;
	}
	private AField<?, ?>.FieldFilter[] getMapAsArray(){
		final Map<AField<?, ?>, AField<?, ?>.FieldFilter> map = getMap();
		final Collection<AField<?, ?>.FieldFilter> list =
			new ArrayList<AField<?, ?>.FieldFilter>(map.size());
		for (final AField<?, ?>.FieldFilter fieldFilter : map.values()) {
			if(fieldFilter.filtert())
				list.add(fieldFilter);
		}
		return list.toArray(new FieldFilter[0]);
	}
}

