/**
 * 	May 23, 2008 - Jun  5, 2008	ws:	Creation from scratch
 * 	Jun 10, 2008 - Jun 16, 2008	ws: support reloadData()
 * 	Jun 23, 2008 - Jun 24, 2008	ws:	nur noch datefilter weiterreichen
 * 	Jul  2, 2008 - Jul 18, 2008	ws:	added filterNotSelect support,
 * 									added chunking
 * 	Aug  1, 2008 - Aug  1, 2008	ws:	added: hasMassiveDecodingProblems()
 * 	Oct  6, 2008 - Oct  6, 2008	ws:	added getBezeichnung
 * 	Jul  2, 2009 - Jul  6, 2009	ws:	added retry on empty data, support filtering
 * 									on parametrized tag, context
 * 	Jul 14, 2009 - Jul 14, 2009	ws:	fixed some Eclipse 3.5 warnings
 *	Jul 28, 2009 - Jul 28, 2009	ws:	added: getAutoAppendServer() etc.
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import de.desy.mst.libs.framework.mstapp.logging.LogUtil;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.gui.AField.FieldFilter;
import de.desy.mst.libs.framework.mstapp.util.Convert;
import de.desy.mst.libs.framework.mstapp.util.MstDate;
import de.desy.mst.libs.framework.mstapp.util.MstDateFormat;
import de.desy.tine.server.logger.Clog;
import de.desy.tine.server.logger.ClogFilter;
import de.desy.tine.server.logger.ClsLog;
//import de.desy.tine.server.logger.ClsLog.ClogPriority;
//import de.desy.tine.server.logger.ClsLog.ClogStatus;

/**
 * Adapter vom Tine Logging Server zum LoggingTableDisplay (pp)
 * @author ws
 * @version 1.0.0
 */
@SuppressWarnings({"rawtypes", "unchecked"})
class LogSource_Clog extends ALogSource<Clog, MsgModel_Clog> {
	private static final int MAXRETRY_ON_TINE_NODATA = 10;
//=============================================================================
	final ClogFilterHandler clogFilterHandler;
	private final String logger;
	private final String tag;
	private final String context;
	private Clog[] entries = new Clog[0];	//never null, but maybe empty
	private final int maxChunkSize = 10000;
//=============================================================================
	/**mit diesen Parametern holt er erstmal Daten
	 * @param context
	 * @param tag */
	LogSource_Clog(final int seitTagen, final String logger, final String tag, final String context) {
		super(MsgModel_Clog.getInstance(), null);
		this.logger  = logger;
		this.tag 	 = tag;
		this.context = context;
		this.clogFilterHandler = new ClogFilterHandler(getMsgModel(), seitTagen, logger, tag, context);
		reloadAllData();
	}
//=============================================================================

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.ILogProvider#reloadData()
	 */
	@Override public void reloadAllData() {
		this.entries = getEntries();
		onNewEntries();
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#onNewEntries()
	 */
	@Override public void onNewEntries() {
		super.onNewEntries();
		final Clog lastClog = (this.getSize() == this.getMaxChunkSize())
							? getMessage(getSize() - 1)	//evtl. noch mehr Daten
							: null;						//habe alles
		getClogFilterHandler().onNewEntries(Convert.toInt(getChunkIndexOfCurrent()), lastClog);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#toString()
	 */
	@Override public String toString() {
		return "Tine Log Source ltc ("
			+ (this.logger == null ? "" : this.logger	) + ", "
			+ (this.tag    == null ? "" : this.tag		) + ", "
			+ (this.context== null ? "" : this.context	)
			+ ") " + System.identityHashCode(this)
			;
	}
	@Override public String getBezeichnung() {
		return getBezeichnung("Tine Logging");
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#getMessages()
	 */
	@Override public __ListAdapt<Clog> getMessages() {
		return new __ListAdapt<Clog>(Collections.unmodifiableList(Arrays.asList(this.entries)));
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#getMessage(int)
	 */
	@Override public Clog getMessage(final int index) {
		return this.entries[index];
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#getSize()
	 */
	@Override public int getSize() {return this.entries.length;}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#initFilter(de.desy.mst.libs.framework.mstapp.logging.gui.FilterHandler.AndOfFieldsFilter, de.desy.mst.libs.framework.mstapp.logging.gui.LoggingTable)
	 */
	@Override
	public void initFilter(final HauptFilter hauptFilter
						 , final LoggingTable<Clog, MsgModel_Clog> table) {
		final MsgModel_Clog msgModel = getMsgModel();
		final AndOfFieldsFilter defaultFilter =
			getClogFilterHandler().createDefaultFilter(msgModel, table);
		hauptFilter.setFilter(defaultFilter);
		onFilterChanged(defaultFilter, table);
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#onFilterChanged()
	 */
	@Override
	public void onFilterChanged(final AndOfFieldsFilter filter
							  , final LoggingTable<Clog, MsgModel_Clog> table) {
		if(getClogFilterHandler().updateClogFilters(filter, getMsgModel(), table))
			reloadAllData();
	}

	@Override public long getChunkAnzahl() {
		return getClogFilterHandler().getChunkAnzahl();
	}
	@Override public long getChunkIndexOfCurrent() 	{
		return getClogFilterHandler().getChunkIndexOfCurrent();
	}
	@Override public void setChunk(final long chunkIndex) throws IOException {
		if(getChunkIndexOfCurrent() == chunkIndex)
			return;
		getClogFilterHandler().setChunkIndexOfCurrent(Convert.toInt(chunkIndex));
		reloadAllData();
	}
	@Override public boolean hasMassiveDecodingProblems() {return false;}

	public int getMaxChunkSize() {return this.maxChunkSize;}

//=============================================================================
	/**never null*/
	private ClogFilterHandler getClogFilterHandler() {
		return this.clogFilterHandler;
	}
	private Clog[] getEntries() {
		final long _start = System.currentTimeMillis();
		final Clog[] entries = getEntries(MAXRETRY_ON_TINE_NODATA);
		doLogGetEntries(_start, entries);
		return entries;
	}
	private Clog[] getEntries(final int maxTry) {
		for (int i = 0; i < maxTry; i++) {
			final Clog[] entries = getEntries_SingleTry();
			if(entries != null)
				return entries;
			final String msg = "LogSource_Clog.getEntries(..) failed on try " 
							 + (i+1) + " of maximum " + maxTry + ".";
			Logger.SWFW.info(msg);
		}
		return new Clog[0];
	}
	private Clog[] getEntries_SingleTry() {
		final ClogFilter[] clogFilters = getClogFilterHandler().createClogFilters();
		Logger.SWFW.info ("TineLogSource der ClogFilter ist: '"	//Zuerst
						+ ClogFilterHandler.asString(clogFilters) + "'.");

		try {
			final Clog[] entries = ClsLog.getEntries(clogFilters, getMaxChunkSize());
			return entries == null ? new Clog[0] : entries;
		} catch (final IOException ex) {
			Logger.SWFW.warn("Kann die Tine Entrge nicht lesen. " + ex.getMessage());
		}
		return null;
	}
	private void doLogGetEntries(final long _start, final Clog[] entries) {
		final String msg = "TineLogSource hat " + entries.length
						 + " entries gefunden."
						 + ((getChunkAnzahl() <= 1) ? "" : " Abschnitt = " + (getChunkIndexOfCurrent() + 1));
		Logger.SWFW.info (LogUtil.timedMsg(_start, msg));
	}

//=============================================================================
	/**koordiniert MstLog Tabellen Filter mit Tine ClogFilter[]*/
	static class ClogFilterHandler{
		static final String DEFAULT_KOMPARATOR = "=";
		private static class Pair{
			final String was; final String komparator; final String wert;
			//-----------------------------------------------------------------
			public Pair(final String was, final String komparator, final String wert) {
		          this.was 			= deNullify(was);
		          this.komparator	= deNullify(komparator);
		          this.wert 		= deNullify(wert);
			}
			//-----------------------------------------------------------------
			@Override public String toString() {
				return (this.wert == null || this.wert.length() == 0) ? "" :
								this.was + " " + this.komparator + " '" + this.wert + "'" ;
			}
			//-----------------------------------------------------------------
			private String deNullify(final String s) {
				return (s == null) ? "" : s;
			}
		}


		//=====================================================================
		/**String Repraesentation eines Tine Filters*/
		public static String asString(final ClogFilter[] filters){
			if(filters == null || filters.length == 0)
				return "";

			if(filters.length == 1)
				return asString(filters[0]);

			final StringBuilder sb = new StringBuilder();
			sb.append(asString(filters[0]));
			for (int i = 1; i < filters.length; i++) {
				sb.append(" UND ").append(asString(filters[i]));
			}
			return sb.toString();
		}

		/**
		 * @param iChunk	... chunkIndex der neuen entries
		 * @param lastClog	... == null keine weiteren Daten, sonst letzter Clog
		 */
		public void onNewEntries(final int iChunk, final Clog lastClog) {
			if(lastClog == null || (iChunk + 1) < getChunkAnzahl())
				return;
			setStart(lastClog.getTimeLogged(), iChunk + 1);
		}


		private static String asString(final ClogFilter filter){
			final List<Pair> list = new ArrayList<Pair>();

			asString_add(list, "text", 		filter.getText()	);
			asString_add(list, "context", 	filter.getContext()	);
			asString_add(list, "tag", 		filter.getTag()		);
			asString_add(list, "logger", 	filter.getLogger()	);
			asString_add(list, "start", 	formatTime(filter.getTimeStart()) );
			asString_add(list, "stop", 		formatTime(filter.getTimeStop ()) );

			final int priority = filter.getPriority();
			final int status = filter.getStatus  ();
			if(priority > 0) asString_add(list, "priority",	"<=" , MsgModel_Clog.asClogPriority(priority).toString());
			if(status   > 0) asString_add(list, "status", 	"<=" , MsgModel_Clog.asClogStatus  (status  ).toString());

			return list.toString();
		}
		private static String formatTime(final long time) {
			return MstDateFormat.ISO.format(new Date( time )) + " = " + time;
		}
		private static void asString_add(final List<Pair> list, final String name, final String komparator, final String inWert) {
			final String wert = MsgModel_Clog.cleanString(inWert);
			if(wert != null && wert.length() > 0)
				list.add(new Pair(name, komparator, wert));
		}
		private static void asString_add(final List<Pair> list, final String name, final String inWert) {
			asString_add(list, name, DEFAULT_KOMPARATOR, inWert);
		}
		//---------------------------------------------------------------------
		private final String text;
		private final String context;
		private final String tag;
		private final String caller;	//== logger
		private final int 	 priority;
		private final int 	 status;

		private final ArrayList<Long> startList = new ArrayList<Long>();
		private long stop;

		private int chunkIndex  = 0;

		//---------------------------------------------------------------------
		public ClogFilterHandler(final MsgModel_Clog msgModel, final int seitTagen, final String logger, final String tag, final String context){
			super();
			this.text		= null;
			this.context	= context;
			this.tag		= tag;
			this.caller		= logger;
			this.priority	= 0;
			this.status		= 0;

			initFilters(MstDate.abrundenAufTage(seitTagen, new Date()).getTime()
					  , Clog_Filter_TimeRange.MAX_STOP);
		}
		//---------------------------------------------------------------------
		public int getChunkIndexOfCurrent() {return this.chunkIndex ;}
		public void setChunkIndexOfCurrent(final int chunkIndex) {
			if(chunkIndex < 0 || chunkIndex >= getChunkAnzahl())
				throw new IllegalArgumentException("ChunkIndex == " + chunkIndex + " muss im Bereich [0, " + getChunkAnzahl() + ") liegen.");
			this.chunkIndex = chunkIndex;
		}

		//---------------------------------------------------------------------
		private AndOfFieldsFilter createDefaultFilter (
							  final MsgModel_Clog msgModel
							, final LoggingTable<Clog, MsgModel_Clog> table) {
			final AndOfFieldsFilter filter = new AndOfFieldsFilter();
			final Clog_Filter_TimeRange clog_Filter_TimeRange = new Clog_Filter_TimeRange(null, msgModel, getStart(0), this.stop);
			filter.add(clog_Filter_TimeRange.createDefaultFilter(table));
			return filter;
		}
		private long getChunkAnzahl() {return this.startList.size();}
		private long getStart(final int iChunk){
			return this.startList.get(iChunk);
		}

		private ClogFilter[] createClogFilters(){
			final Long start = getStart(getChunkIndexOfCurrent());
			return createClogFilters(start, this.stop);
		}
		private ClogFilter[] createClogFilters(final long start, final long stop) {
			return new ClogFilter[] {new ClogFilter(this.text, this.context, this.tag, this.caller, this.priority, this.status, start, stop)};
		}
		private void setStart(final long start, final int iChunk){
			if(this.startList.size() == iChunk)
				this.startList.add(start);
			else
				this.startList.set(iChunk, start);
		}

		/**@return true if the filter changed*/
		private boolean updateClogFilters( final AndOfFieldsFilter  filter
										 , final MsgModel_Clog      msgModel
										 , final LoggingTable<Clog, MsgModel_Clog> table
										 ) {
			final Clog_Filter_TimeRange timeRange = new Clog_Filter_TimeRange(filter, msgModel, getStart(0), this.stop);
			final long startNeu	= timeRange.start;
			final long stopNeu	= timeRange.stop ;
			final boolean needNewClogFilters =  startNeu < getStart(0)
					 						            || this.stop < stopNeu;
			if(needNewClogFilters)
				initFilters(startNeu, stopNeu);
			return needNewClogFilters;
		}

		private void initFilters(final long start, final long stop) {
			this.startList.clear();
			this.stop  = stop;
			setStart(start, 0);
			setChunkIndexOfCurrent(0);
		}
	}

	/**immutable, berechnet start und stop bei creation*/
	static class Clog_Filter_TimeRange{
		public  final static long MAX_STOP = calcMaxStop();
		//---------------------------------------------------------------------
		public final MsgModel_Clog msgModel;
		public final long defaultStart;	//objekt und nicht Klassenvariable!
		public final long defaultStop;	//objekt und nicht Klassenvariable!
		public final long start;
		public final long stop;
		public final String asString;
		Clog_Filter_TimeRange(final AndOfFieldsFilter filter, final MsgModel_Clog msgModel, final long defaultStart, final long defaultStop){
			this.msgModel = msgModel;
			this.defaultStart = defaultStart;
			this.defaultStop  = defaultStop;
			final FFERange<Date, Clog> dateRange = getDateRange(filter, msgModel.getDateField());
			this.start = dateRange.von.getTime();
			this.stop  = dateRange.bis.getTime();
			this.asString = dateRange.asString();
		}
		public FieldFilter createDefaultFilter(final LoggingTable<Clog, MsgModel_Clog> table) {
			final AField<Date, Clog> dateField = this.msgModel.getDateField();
			final FieldFilter fieldFilter = dateField.createFieldFilter(table);
			fieldFilter.add(createDefaultDateRange(dateField));
			return fieldFilter;
		}
		//--------------------------------------------------------------------=
		/**never null*/
		private FFERange<Date, Clog> getDateRange(final AndOfFieldsFilter filter
												, final AField dateField) {
			if(filter == null
			|| filter.get(dateField) == null
			  )
				return createDefaultDateRange(dateField);
			final int size = filter.get(dateField).size();
			for (int i = 0; i < size; i++) {
				final Object anFFE = getFFE(i, filter, dateField);
				if(anFFE instanceof FFERange)
					return (FFERange<Date, Clog> ) anFFE;
			}
			for (int i = 0; i < size; i++) {
				final Object anFFE = getFFE(i, filter, dateField);
				if(anFFE instanceof FFEValueEquality)
					return createDateRange((FFEValueEquality) anFFE, dateField);
			}
			return createDefaultDateRange(dateField);
		}
		private FFERange<Date, Clog> createDefaultDateRange(final AField dateField) {
			return createDateRange(this.defaultStart, this.defaultStop, dateField);
		}
		private static FFERange<Date, Clog> createDateRange(final FFEValueEquality<Date, Clog> firstFFE, final AField dateField) {
			final long start = firstFFE.getRefValue().getTime();
			final long stop = Long.MAX_VALUE;
			return createDateRange(start, stop, dateField);
		}
		private static FFERange<Date, Clog> createDateRange(final long von, final long bis, final AField dateField) {
			return new FFERange(dateField
							  , new Date(fenceStartStop(von))
							  , new Date(fenceStartStop(bis))
			);
		}
		private static  Object getFFE(final int i, final AndOfFieldsFilter filter, final AField dateField) {
			final Object anFFE = filter.get(dateField)
						   			   .getFFEs()
						   			   .get(i);
			return anFFE;
		}

		private static long calcMaxStop() {
			final Calendar cal = Calendar.getInstance();
			cal.clear();
			final int year = 2038;
			final int month = Calendar.JANUARY;
			final int date  = 1;
			cal.set(year, month, date);
			return cal.getTimeInMillis();
		}
		private static  final long fenceStartStop(final long ms){
			if(ms < 0)
				return 0;
			if( ms > MAX_STOP)
				return MAX_STOP;
			return ms;

		}
	}	//END class Clog_Filter_TimeRange
}
