/**
 * 	May 22, 2008 - Jun 11, 2008	ws:	Creation from scratch
 * 	Jun 17, 2008 - Jun 17, 2008	ws: added: hideColumnByDefault
 * 	Jun 20, 2008 - Jun 20, 2008	ws:	removed Phil Clog Problem circumvents
 * 	Aug 28, 2008 - Aug 28, 2008	ws: color coding nicht nur Message sondern auch
 * 							feldspezifisch (Kommentare Anzeige unterdruecken)
 *	Jan 15, 2009 - Jan 15, 2009	ws:	serializable
 *	Jul  3, 2009 - Jul  3, 2009	ws:	class public -> pp
 * 	Jul 14, 2009 - Jul 14, 2009	ws:	fixed some Eclipse 3.5 warnings
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.util.Collections;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Map;

import de.desy.mst.libs.framework.mstapp.util.gui.AppColor;
import de.desy.tine.server.logger.Clog;
import de.desy.tine.server.logger.ClsLog;
import de.desy.tine.server.logger.ClsLog.ClogPriority;
import de.desy.tine.server.logger.ClsLog.ClogStatus;

/**
 * Native Implementation von Tines CLog Message Format
 * @author ws
 * @version 1.0.0
 */
class MsgModel_Clog extends AMsgModel<Clog> {
	private static final String CONTEXT_NAME		= "Context";
	private static final int    CONTEXT_LENGTH		= "DESY2".length();

	private static final String TAG_NAME			= "Tag";
	private static final int    TAG_LENGTH			= "MstDoAlarm".length();

	private static final String LOGGER_NAME			= "Logger";
	private static final int    LOGGER_LENGTH		= "Eventstore".length();

	private static final String TEXT_NAME			= "Text";
	private static final int    TEXT_LENGTH			= 50;

	private static final String PRIORITY_NAME		= "Prioritt";
	private static final String STATUS_NAME			= "Status";

	private static final String TIME_LOGGED_NAME	= "Time Logged";
	private static final String TIME_SENT_NAME 		= "Time Sent";

	private static final int WARN = ClsLog.ClogStatus.CLOG_STATUS_WARN.ordinal();

	private static final MsgModel_Clog instance = new MsgModel_Clog();
	//-------------------------------------------------------------------------
	public static final MsgModel_Clog getInstance() {return instance;}
	//-------------------------------------------------------------------------
	/**Problem: Tine Strings fixed length char = 0 (C Konvention) beendet*/
	public static String cleanString(final String s) {
		char badEntry = 0;
		final int indexOf = s.indexOf(badEntry);
		return (indexOf < 0) ? s.trim() : s.substring(0, indexOf);
	}
	public static final ClogPriority asClogPriority(final int ordinal){
		if(ordinal < 0)
			return asClogPriority(0);
		if(ordinal >= ClogPriority.values().length)
			return asClogPriority(ClogPriority.values().length - 1);
		return ClogPriority.values()[ordinal];

	}
	public static final ClogStatus asClogStatus(final int ordinal){
		if(ordinal < 0)
			return asClogStatus(0);
		if(ordinal >= ClogStatus.values().length)
			return asClogStatus(ClogStatus.values().length - 1);
		return ClogStatus.values()[ordinal];
	}

//=============================================================================
	private MsgModel_Clog() {super(createFieldMap());}
//=============================================================================
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.AMsgModel#getAppColor(java.lang.Object)
	 */
	@Override public AppColor getAppColor(final Clog message, final int spalte) {
		final int status = message.getStatus();
		if(status < WARN)
			return AppColor._Info;
		if(status == WARN)
			return AppColor._WARNUNG;
		return AppColor._FEHLER;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.AMsgModel#getDateField()
	 */
	@Override public AField<Date, Clog> getDateField() {
		return getField_TimeLogged();
	}
//=============================================================================
	/**for XTor only*/
	private static Map<String, AField<?, Clog>> createFieldMap() {
//		  public String getContext()
//		  public String getTag()
//		  public String getLogger()
//		  public String getText()
//		  public int 	getPriority()
//		  public int 	getStatus()
//		  public long 	getTimeSent()
//		  public long 	getTimeLogged()
		Map<String, AField<?, Clog>> map = new LinkedHashMap<String, AField<?, Clog>>(9);
		int ind = 0;
		createFieldMap_AddField(map, createField_Context(ind++)	);
		createFieldMap_AddField(map, createField_Tag	(ind++)	);
		createFieldMap_AddField(map, createField_Logger	(ind++)	);
		createFieldMap_AddField(map, createField_Text	(ind++)	);

		createFieldMap_AddField(map, createField_Prio	(ind++)	);
		createFieldMap_AddField(map, createField_Status	(ind++)	);

		createFieldMap_AddField(map, createField_TimeSent	(ind++)	);
		createFieldMap_AddField(map, createField_TimeLogged	(ind++)	);
		return Collections.unmodifiableMap(map);
	}
	private static AField<String, Clog> createField_Context (final int index) {
		return new AStringFieldCLog(
				  index
				, CONTEXT_NAME									//name
				, "Context."									//description
				, AField.IS_DISCRETE
				, CONTEXT_LENGTH
				, "Erlaubte Kontexte."							//editorText
				, ""											//editorToolTip
				, !AField.IS_WIDE_EDITOR
		){
			private static final long serialVersionUID = 1L;
			@Override String get(final Clog logMsg)
									{return logMsg.getContext();}
		};
	}
	private static AField<String, Clog> createField_Tag     (final int index) {
		return new AStringFieldCLog(
				  index
				, TAG_NAME										//name
				, "Tag."										//description
				, AField.IS_DISCRETE
				, TAG_LENGTH
				, "Erlaubte Tags."								//editorText
				, ""											//editorToolTip
				, !AField.IS_WIDE_EDITOR
		){
			private static final long serialVersionUID = 1L;
			@Override String get(final Clog logMsg)
										{return logMsg.getTag();}
		};
	}
	private static AField<String, Clog> createField_Logger  (final int index) {
		return new AStringFieldCLog(
				  index
				, LOGGER_NAME									//name
				, "Logger."										//description
				, AField.IS_DISCRETE
				, LOGGER_LENGTH
				, "Erlaubte Logger."							//editorText
				, ""											//editorToolTip
				, !AField.IS_WIDE_EDITOR
		){
			private static final long serialVersionUID = 1L;
			@Override String get(final Clog logMsg)
									{return logMsg.getLogger();}
		};
	}
	private static AField<String, Clog> createField_Text    (final int index) {
		return new AStringFieldCLog(
				  index
				, TEXT_NAME										//name
				, "Text."										//description
				, !AField.IS_DISCRETE
				, TEXT_LENGTH
				, "Meldung"										//editorText
				, "Mitteilung im Log/Protokoll"					//editorToolTip
				, AField.IS_WIDE_EDITOR
		){
			private static final long serialVersionUID = 1L;
			@Override String get(final Clog logMsg) {
				return cleanString(logMsg.getText());
//				return AStringFieldCLog.get(logMsg, getName());
			}
		};
	}

	private static AField<ClogPriority, Clog> createField_Prio(final int index) {

		return new AField<ClogPriority, Clog>(
				  index
				, PRIORITY_NAME
				, "Wichtigkeit des Eintrags"
				, ClogPriority.class
				, ClogPriority.CLOG_PRIORITY_NONE
				, new AFieldFilterGUIFactory.ForFixedValues<ClogPriority, Clog, MsgModel_Clog>("Erlaubte Prioritten.", "", ClogPriority.values())
		){
			private static final long serialVersionUID = 1L;
			@Override public boolean isDiscrete() {return true;}
			@Override ClogPriority get(final Clog logMsg) {
				return asClogPriority(logMsg.getPriority());
			}
			@Override public String asString(final ClogPriority e) {
				final String[] split = asStringLong(e).split("_");
				return split[split.length - 1];
			}
			@Override public ClogPriority asValue(final String entry) {
				try {
					return asClogPriority(Integer.parseInt(entry));
				} catch (NumberFormatException e) {
					try {
						return ClogPriority.valueOf(entry.toUpperCase());
					} catch (IllegalArgumentException e2) {
						return ClogPriority.CLOG_PRIORITY_USEFUL;//undekodierbar
					}
				}
			}
		};
	}
	private static AField<ClogStatus, Clog> createField_Status(final int index) {

		return new AField<ClogStatus, Clog>(
				  index
				, STATUS_NAME
				, "Status des Eintrags"
				, ClogStatus.class
				, ClogStatus.CLOG_STATUS_NONE
				, new AFieldFilterGUIFactory.ForFixedValues<ClogStatus, Clog, MsgModel_Clog>("Erlaubte Status-Zustnde.", "", ClogStatus.values())
		){
			private static final long serialVersionUID = 1L;
			@Override public boolean isDiscrete() {return true;}
			@Override ClogStatus get(final Clog logMsg) {
				return asClogStatus(logMsg.getStatus());
			}
			@Override public String asString(final ClogStatus e) {
				final String[] split = asStringLong(e).split("_");
				return split[split.length - 1];
			}
			@Override public ClogStatus asValue(final String entry) {
				try {
					return asClogStatus(Integer.parseInt(entry));
				} catch (NumberFormatException e) {
					try {
						return ClogStatus.valueOf(entry.toUpperCase());
					} catch (IllegalArgumentException e2) {
						return ClogStatus.CLOG_STATUS_WARN;//undekodierbar
					}
				}
			}
		};
	}

	private static AField<Date, Clog> createField_TimeSent  (final int index) {
		return new AField.ADateField<Clog, MsgModel_Clog>(index, TIME_SENT_NAME, "Erlaubte 'Time Sent'"){
			private static final long serialVersionUID = 1L;
			@Override Date get(final Clog logMsg) {
				return new ISODate(logMsg.getTimeSent());
			}
			/**usually exact duplicate to TimeLogged*/
			@Override public boolean hideColumnByDefault() {return true;}
		};
	}
	private static AField<Date, Clog> createField_TimeLogged(final int index) {
		return new AField.ADateField<Clog, MsgModel_Clog>(index, TIME_LOGGED_NAME, "Erlaubte 'Time Logged'*"){
			private static final long serialVersionUID = 1L;
			@Override Date get(final Clog logMsg) {
				return new ISODate(logMsg.getTimeLogged());
			}
		};
	}
//=============================================================================
	@SuppressWarnings("unchecked")
	public AField<String, Clog> getField_Context() {
		return (AField<String, Clog>) getField(CONTEXT_NAME);
	}
	@SuppressWarnings("unchecked")
	public AField<String, Clog> getField_Tag() {
		return (AField<String, Clog>) getField(TAG_NAME);
	}
	@SuppressWarnings("unchecked")
	public AField<String, Clog> getField_Logger() {
		return (AField<String, Clog>) getField(LOGGER_NAME);
	}
	@SuppressWarnings("unchecked")
	public AField<String, Clog> getField_Text() {
		return (AField<String, Clog>) getField(TEXT_NAME);
	}
	@Override public AField<?, Clog> getCoreInfoField(){return getField_Text();}

	@SuppressWarnings("unchecked")
	public AField<ClogPriority, Clog> getField_Priority() {
		return (AField<ClogPriority, Clog>) getField(PRIORITY_NAME);
	}
	@SuppressWarnings("unchecked")
	public AField<ClogStatus, Clog> getField_Status() {
		return (AField<ClogStatus, Clog>) getField(STATUS_NAME);
	}
	@SuppressWarnings("unchecked")
	public AField<Date, Clog> getField_TimeLogged() {
		return (AField<Date, Clog>) getField(TIME_LOGGED_NAME);
	}
	@SuppressWarnings("unchecked")
	public AField<Date, Clog> getField_TimeSent() {
		return (AField<Date, Clog>) getField(TIME_SENT_NAME);
	}
//=============================================================================
//=============================================================================
//=============================================================================
	/**
	 * Problem: Tine Clog Flaw:<br>
	 * erst watchdog anzeigen<br>
	 * dann tine anzeigen<br>
	 * Symptome: in Clogs steht z.B. logger = "watchdog..." aber char korrekt "eventstort"<br>
	 * Also Info via toString, das es direkt aus dem char array holt<br>
	 */
	private abstract static class AStringFieldCLog extends AField.AStringField<Clog, MsgModel_Clog>{
		private static final long serialVersionUID = 1L;
//		private static final String SUFFIX = " ->";
		AStringFieldCLog(final int index, final String name, final String description, final boolean isDiscrete, final int nCharsLongMessage, final String editorText, final String editorToolTip, final boolean isWideEditor) {
			super(index, name, description, isDiscrete, nCharsLongMessage, editorText, editorToolTip, isWideEditor);
		}
//		@Override /*final*/ String get(final Clog logMsg) {
//			final String name = getName();
//			return get(logMsg, name);
//		}
//		public static String get(final Clog logMsg, final String name) {
//			final String sBegin = " " + name.toLowerCase() + SUFFIX;
//			final String msg = logMsg.toString();
//			final int beginPair = msg.indexOf(sBegin);
//			if(beginPair >= 0){
//				final int beginValue = beginPair + sBegin.length();
//				final int end = msg.indexOf("\n", beginValue);
//				if(end >= beginValue){
//					final String value = msg.substring(beginValue, end);
//					return value;
//				}
//			}
//			return "?";
//		}
	}//	END	class	AStringFieldCLog
}
