/**
 * 	Jun  2, 2009 - Jul  6, 2009	ws:	Creation from scratch
 * 	Jun 22, 2009 - Jul 22, 2009	ws:	add createClogMapper and related
 * 	Aug  6, 2009 - Aug  6, 2009	ws:	Eclipse asked for removing unnessary casts, so I did
 * 	Aug 13, 2009 - Aug 13, 2009	ws:	added support for accesories: isSystemInfo(clog)
 * 																, logger2RunId(logger)
 * 																, getSystemInfoSummary_FromSystemClog(systemClog)
 * 	Sep 14, 2009 - Sep 16, 2009	ws:	added SystemInfoSummary is now comparable via Date
 * 									addes getSystemInfoSummaryArray(
 */

package de.desy.mst.libs.framework.mstapp.logging.map_tinelogsrv;

import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;
import java.util.UUID;
import java.util.concurrent.atomic.AtomicReference;

import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppender;
import de.desy.mst.libs.framework.mstapp.logging.appender.IWriter;
import de.desy.mst.libs.framework.mstapp.util.MstDateFormat;
import de.desy.tine.server.logger.Clog;

/**
 * MstLog's can be written to the Tine Log/Protocol Server and read back
 * <br> This is the facade for it
 * <br> only public class in package
 * <br> provides appender, reader for mstLoggings mapped to the Tine Log Server
 * @author ws
 * @version 1.0.0
 */
public class MstLog4TineLog_Facade implements __IDataMapping{
	private static final  MstLog4TineLog_Facade instance = new MstLog4TineLog_Facade();
//-----------------------------------------------------------------------------
	public static final String UNKNOWN_PC_NAME = "?PCName?";
	public static interface ClogMapper{public LogMsg map(final Clog wrappedLogMsg);};
	public static final MstLog4TineLog_Facade getInstance() {return instance;}

//=============================================================================

	private final AtomicReference<ILogAppender> tineLogAppender = new AtomicReference<ILogAppender>();
//-----------------------------------------------------------------------------
	private MstLog4TineLog_Facade() {/**erst mal als singleton*/}
//-----------------------------------------------------------------------------
	/**
	 * @param  	logMsg 				 only used on first call but then must be nonnull
	 * @param 	optionalAppenderSpy	 may be null, for debugging to spy what is written
	 * 								(Clogs -> LogMsg which are written to Tine)
	 * @return 	nonnull 			 lazy created singleton tineLogAppender for the current PC
	 * @throws 	NullPointerException on first call if logMsg is null
	 */
	public final ILogAppender getTineLogAppender(final LogMsg logMsg, final IWriter<ClsLogInfo> optionalAppenderSpy) throws NullPointerException{
		if(this.tineLogAppender.get() == null)
			this.tineLogAppender.set(createTineLogAppender(logMsg, optionalAppenderSpy));
		return this.tineLogAppender.get();
	}
	public final String getSystemContext()  {return ClsLogInfo.getSystemContext();}
	public final boolean isSystemInfo(final Clog clog) {
		return clog != null 
			&& ClsLogInfo.isSystemContext(clog.getContext());
	}
	public static class SystemInfoSummary implements Comparable<SystemInfoSummary>{
		public final static SystemInfoSummary NULL_SIS = new SystemInfoSummary("", new Date(), "###PCNAME###"){
			@Override public String toString() {return "";}
		}; 
		public final String sRunId; 
		public final Date   startDate;
		public final String pcName;
		public SystemInfoSummary(final SystemInfoSummary sis){this(sis.sRunId, sis.startDate, sis.pcName);}
		public SystemInfoSummary(final String sRunId, final Date startDate, final String pcName) {
			this.sRunId = normalize(sRunId);
			this.startDate = startDate;
			this.pcName = pcName;
		}
		@Override public int compareTo(final SystemInfoSummary o) {
			return this.startDate.compareTo(o.startDate);
		}
		@Override public String toString() {
			return  "seit " + MstDateFormat.KURZ.format(this.startDate) 
		     				+ " auf \"" + this.pcName + "\"" 
		     				+ " (" + this.sRunId + ")";
		}
		private String normalize(final String sRunId) {
			if(sRunId != null && sRunId.contains("[-]")) {
				Logger.SWFW.warn("MstLog4TineLog_Facade.SystemInfoSummary.normalize(" + sRunId + ") unnormalized sRunId.", new IllegalArgumentException());
				return __MapRunId.get().mapSRunId(sRunId);
			}
			return sRunId;
		}
	}
	/**Immutable, SystemInfoSummary + AppName*/
	public static class SystemInfoSummaryApp extends SystemInfoSummary{
		public final String appName;
		public SystemInfoSummaryApp(final SystemInfoSummary sis, final String appName) {
			super(sis);
			this.appName = appName;
		}
		public SystemInfoSummaryApp(final String sRunId, final Date startDate, final String pcName, final String appName) {
			super(sRunId, startDate, pcName);
			this.appName	= appName;
		}
		public SystemInfoSummaryApp(final UUID runId, final Date startDate, final String pcName, final String appName) {
			this(__MapRunId.get().map(runId), startDate, pcName, appName);
		}
		@Override public String toString() {return this.appName + " " + super.toString();}
	}
	
	public final SystemInfoSummary getSystemInfoSummary_FromSystemClog(final Clog systemMessage){
		if(systemMessage == null || !isSystemInfo(systemMessage))
			return SystemInfoSummary.NULL_SIS;
		final String text = systemMessage.getText();
		final __MapTextSystem mapSystemText = __MapTextSystem.create(text);
		return new SystemInfoSummary(systemMessage.getLogger()
								   , mapSystemText.start
								   , mapSystemText.pcName);
	}
	public final SystemInfoSummaryApp getSystemInfoSummaryApp_FromSystemClog(final Clog systemMessage){
		final SystemInfoSummary sis = getSystemInfoSummary_FromSystemClog(systemMessage);
		final String        appName = __MapAppName.get().map(systemMessage);
		return new SystemInfoSummaryApp(sis, appName);
	}
	public final UUID logger2RunId(final String logger) {
		return __MapRunId.get().map(logger);
	}
	public String runId2Logger(final UUID runID){return __MapRunId.get().map(runID);}
	/**
	 * @param 	systemMessage, may be null und mstLog -> Tine System Message (z.B. HB) mit gleichem pcName
	 * @return	einen Mapper fuer alle weiteren korrespondierenden ClogMessages
	 * @throws IllegalArgumentException	falls keine System Message
	 */
	public ClogMapper createClogMapper(final Clog systemMessage) throws IllegalArgumentException{
		checkForNullOrSystemMessage(systemMessage);
		final String pcName = extractPCName(systemMessage);
		return new ClogMapper(){
			final ___MapEntireMsg _map = new ___MapEntireMsg(pcName);
			@Override public LogMsg map(final Clog wrappedLogMsg) {
				final ClsLogInfo info = ClsLogInfo.create(wrappedLogMsg);
				final boolean isSystemInfo = info.isSystemInfo();
				return isSystemInfo ? this._map.map((__ClsLogInfoSystem   ) info)
									: this._map.map((__ClsLogInfoNonSystem) info);
			}
		};
	}
	public SystemInfoSummary[] getSystemInfoSummaryArray(final List<Clog> msgs, final String appName) {
		if(appName == null || appName.isEmpty())
			return new SystemInfoSummary[0];
		final Set<SystemInfoSummary> set = new TreeSet<SystemInfoSummary>();
		final __MapAppName mapAppName = __MapAppName.get();
		for (final Clog clog : msgs) 
			if(isSystemInfo(clog) && appName.equals(mapAppName.map(clog)))
				set.add(getSystemInfoSummary_FromSystemClog(clog));
		final SystemInfoSummary[] array = set.toArray(new SystemInfoSummary[0]);
		return array;
	}
	private void checkForNullOrSystemMessage(final Clog systemMessage) throws IllegalArgumentException {
		if(systemMessage == null)
			return;
		final ClsLogInfo logInfo = ClsLogInfo.create(systemMessage);
		if(!logInfo.isSystemInfo())
			throw new IllegalArgumentException("Keine System Message uebergeben: " + systemMessage);
	}
	private String extractPCName(final Clog systemMessage) {
		if(systemMessage == null)
			return MstLog4TineLog_Facade.UNKNOWN_PC_NAME;
		final String text = systemMessage.getText();
		final __MapTextSystem mapSystemText = __MapTextSystem.create(text);
		final String pcName = mapSystemText.pcName;
		return pcName;
	}
//-----------------------------------------------------------------------------
	private final ILogAppender createTineLogAppender(final LogMsg logMsg, final IWriter<ClsLogInfo> optionalWriterSpy) throws NullPointerException{
		final LinkedList<IWriter<ClsLogInfo>> dieWriter = new LinkedList<IWriter<ClsLogInfo>>();
		dieWriter.add(new __ClsLogInfoAppender_Tine());
		if(optionalWriterSpy != null)
			dieWriter.add(optionalWriterSpy);
		return new ___TineLogAppender(logMsg, dieWriter);
	}
}
