/**
 * May 27, 2009 - May 28, 2009	ws:	Creation from scratch
 */
package de.desy.mst.libs.framework.mstapp.logging.map_tinelogsrv;

/**
 * LogMsg.message can have an optional Context Prefix: "{ContextName} "<br>
 * Clog has an eplicit Context field
 * <br><br>
 * Examples:
 * <li> LogMsg.message = "{myContext} myMessage"   maps to   context = "myContext" and rawLogMsg = "myMessage"
 * <li> LogMsg.message = "{} myMessage"            maps to   context = null        and rawLogMsg = "myMessage"
 * <li> LogMsg.message = "myMessage"               maps to   context = null        and rawLogMsg = "myMessage"
 * <br>
 * <li> context = null        and rawLogMsg = "myMessage"   maps to   LogMsg.message = "myMessage"
 * <li> context = ""          and rawLogMsg = "myMessage"   maps to   LogMsg.message = "myMessage"
 * <li> context = "myContext" and rawLogMsg = "myMessage"   maps to   LogMsg.message = "{myContext} myMessage"
 * @author ws
 * @version 1.0.0
 */
class __MapContext {
	private __MapContext(){throw new IllegalStateException("pure static class");}
//=============================================================================
	/**The optional context ist is inside those in the decorated LogMst.message*/
	private static final String CONTEXT_PREFIX = "{";
	private static final String CONTEXT_SUFFIX = "} ";//beachte das blank
//-----------------------------------------------------------------------------
	/**
	 * @param logMsgMessage	... LogMsg.message WITH the optional context prefix
	 * @return				... (Clog.context, LogMsg.message WITHOUT the context prefix)
	 */
	public static Context_RawLogMsgMsg_Pair map(final String logMsgMessage){
		if(!logMsgMessage.startsWith(CONTEXT_PREFIX))
			return createTrivialPair(logMsgMessage);

		final int lPref = CONTEXT_PREFIX.length();
		final int beginSuff = logMsgMessage.indexOf(CONTEXT_SUFFIX, lPref);
		if(beginSuff < 0)
			return createTrivialPair(logMsgMessage);

		final String context = logMsgMessage.substring(lPref, beginSuff);
		final String rawMsg  = logMsgMessage.substring(beginSuff + CONTEXT_SUFFIX.length());
		if(context.isEmpty())
			return createTrivialPair(rawMsg);
		else
			return new Context_RawLogMsgMsg_Pair(context, rawMsg);
	}
	/**
	 * @param cLogContext		... null, SYSTEM_CONTEXT or Tine Context
	 * @param rawLogMsgMessage	... LogMsg.message WITHOUT the optional context prefix
	 * @return					... LogMsg.message WITH the context prefix if context not null
	 */
	public static String map(final String cLogContext, final String rawLogMsgMessage){
		return decorate(cLogContext) + rawLogMsgMessage;
	}
//-----------------------------------------------------------------------------
	private static String decorate(final String cLogContext){
		if(cLogContext == null || cLogContext.isEmpty())
			return "";
		return CONTEXT_PREFIX + cLogContext + CONTEXT_SUFFIX;
	}
	private static Context_RawLogMsgMsg_Pair createTrivialPair(final String logMsgMessage) {
		final String logContext = null;
		return new Context_RawLogMsgMsg_Pair(logContext, logMsgMessage);
	}
//=============================================================================
	static class Context_RawLogMsgMsg_Pair{
		/**TineLog Context*/	final String cLogContext;
		/**LogMsg.message */	final String rawLogMsgMessage;
		Context_RawLogMsgMsg_Pair(final String logContext, final String rawLogMsgMessage) {
			this.cLogContext = logContext;
			this.rawLogMsgMessage = rawLogMsgMessage;
		}
	}
}
