/*
 * Created on Jun 16, 2005 - Jun 16, 2005
 *
 */
package de.desy.mst.libs.framework.mstapp.messages;

import javax.swing.JOptionPane;

/**
 * Reine Datenklasse fuer messages
 * @author ws
 * @version 1.0.0
 */
public final class Message {
    /** Used for error messages. */
    public static final int  ERROR_MESSAGE          = JOptionPane.ERROR_MESSAGE;
    /** Used for information messages. */
    public static final int  INFORMATION_MESSAGE    = JOptionPane.INFORMATION_MESSAGE;
    /** Used for warning messages. */
    public static final int  WARNING_MESSAGE        = JOptionPane.WARNING_MESSAGE;
    /** Used for questions. */
    public static final int  QUESTION_MESSAGE       = JOptionPane.QUESTION_MESSAGE;
    /** No icon is used. */
    public static final int   PLAIN_MESSAGE         = JOptionPane.PLAIN_MESSAGE ;
    
// ===================================================================
      
    public final String text;       //never null
    public final String toolTipText;//never null
    public final int    messageType;
    
//===================================================================
    
    /**
     * @param text
     *            sehr kurzer String zur Anzeige auf dem Statusbar
     * @param toolTipText
     *            mittellanger String zur Anzeige als ToolTipText
     * @param messageType
     *            the type of message to be displayed:
     *            <code>ERROR_MESSAGE</code>,
     *            <code>INFORMATION_MESSAGE</code>,
     *            <code>WARNING_MESSAGE</code>,<code>QUESTION_MESSAGE</code>,
     *            or <code>PLAIN_MESSAGE</code>
     * @throws IllegalArgumentException if bad messageType
     */
    public Message(final String text, 
                   final String toolTipText,
                   final int messageType) {
        
        super();
        if(!isLegalMessageType(messageType)){throw new IllegalArgumentException("Illegal messageType: " + messageType);}
        
        this.text = deNullify(text);
        this.toolTipText = deNullify(toolTipText);
        this.messageType = messageType;
    }
    
    public static final boolean isLegalMessageType(final int messageType){
        if(messageType == ERROR_MESSAGE      )  return true;
        if(messageType == INFORMATION_MESSAGE)  return true;  
        if(messageType == WARNING_MESSAGE    )  return true;  
        if(messageType == QUESTION_MESSAGE   )  return true;  
        if(messageType ==  PLAIN_MESSAGE     )  return true;   
        return false;
    }
    

    /**
     * wird als modaler Dialog angezeigt 
     */
    public void showMessageDialog() {
        JOptionPane.showMessageDialog(null, this.toolTipText, this.text, this.messageType);    
    }
//  ===================================================================
      
    /**make null to nullString*/
    private String deNullify(final String string) { 
        return string == null ? new String() : string;
    }
}
