/**
 *  Created on 	Dec 20, 2005 - Dec 22, 2005
 *  			Feb 10, 2005 - Feb 10, 2005	ws: added getMaschineByString(..)
 *  			May 23, 2005 - May 23, 2005	ws: added observable Interface
 *  			Jun 07, 2006 - ...			ws: added XTor(maschineName)
 *  			Jun 20, 2006 - Jun 22, 2006	ws: support MaschinenKonfigurationen
 *  			Jun 20, 2006 - Jun 22, 2006	ws: setErlaubteMaschinen(Maschine ... erlaubteMaschinen)
 *  											statt nur Array
 * Sep  2, 2009 - Sep  2, 2009	ws:	update maschine in Maschine on setMaschine
 */
package de.desy.mst.libs.framework.mstapp.mode;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import de.desy.mst.libs.framework.mstapp.ini.FileDataIni;
import de.desy.mst.libs.framework.mstapp.ini.IIniFileObserver_FileDataIni;
import de.desy.mst.libs.framework.mstapp.logging.Logger;


/**
 * Maschine Mode Manager<br>
 * Der einzige Platz, wo die Maschine (Pe, Do, ...) der current app (MstApp Object) definiert ist<br>
 * Die Maschine ist nie null.<br>
 * Man kann sich zur Benachrichtigung bei Aenderung der Maschine registrieren lassen.<br>
 *
 * @link 	Maschine enum
 * @author 	ws
 * @version 1.0.0
 */
public final class MaschineMgr  implements IIniFileObserver_FileDataIni{
	private Maschine maschine;	//DIE Maschine
	private final List<IMaschineObserver> 	maschineObservers = new LinkedList<IMaschineObserver>();
	private final MaschinenKonfiguration maschinenKonfiguration;	//privat und intern only, Zugriff via Delegation um Aenderungen mit gesetzter Maschine synchron zu halten
	 		final boolean registerForBetterGuessOfMaschine;			//package for JUnit only

	private final IMaschineGuessBestSettingObserver observer = new IMaschineGuessBestSettingObserver(){
		@Override public void onNewGuess(final Maschine maschine, final List<Maschine> erlaubteMaschinen) {
			MaschineGuessBestSetting.deregisterObserver(MaschineMgr.this.observer);	//nur einmal raten

			final String msgPrefix =  "(" + this.getClass().getName()
									+ ".onNewGuess(" + maschine
									+ ", " + erlaubteMaschinen + ") ";
			Maschine neueMaschine = maschine;
			if(maschine == getMaschine()){
				Logger.SWFW.info(msgPrefix + "Geratenen Maschine ist gleich der gesetzten Maschine, also Kein Aktion.");
			}else if(MaschineMgr.this.registerForBetterGuessOfMaschine){
				Logger.SWFW.info(msgPrefix + " Setze Maschine '" + getMaschine() + "' um.");
				neueMaschine = maschine;
			}else{
				Logger.SWFW.info(msgPrefix + " Behalte alte Maschine '" + getMaschine() + "'.");
				neueMaschine = getMaschine();
			}
			if(!erlaubteMaschinen.contains(neueMaschine)){
				final String msg =  msgPrefix + "erlaubteMaschinen enthaelt die Maschine nicht, fuege sie von Hand zu.";
				Logger.SWFW.warn(msg);
				erlaubteMaschinen.add(neueMaschine);
			}
			try {
				setErlaubteMaschinenUndMaschine(neueMaschine, erlaubteMaschinen);
			} catch (final Exception e) {
				final String msg =  "Problem die Maschine/erlaubte Maschinen zu raten ("
							+ neueMaschine.toString()
							+ ", "
							+ erlaubteMaschinen.toString()
							+ ")";
				Logger.SWFW.warn(msg, e);
			}
		}
	};
//=============================================================================

	public MaschineMgr(final String maschinenName, final boolean registerForBetterGuessOfMaschine) {
		super();
		final Maschine maschine = Maschine.getMaschineByString(maschinenName);
		this.maschinenKonfiguration = new MaschinenKonfiguration(maschine);
		setMaschine(maschine);
		this.registerForBetterGuessOfMaschine = registerForBetterGuessOfMaschine;
		MaschineGuessBestSetting.registerObserver(this.observer);
	}
	/**'maschine' ist die aktuell gltige Maschine der Applikation (MstApp Object) @throws NullPointerException, falls maschine == null*/
	public MaschineMgr(final Maschine maschine) {
		this(maschine.name());
	}
	/**'maschine' ist die aktuell gltige Maschine der Applikation (MstApp Object) @throws NullPointerException, falls maschine == null*/
	public MaschineMgr(final String maschinenName){
		this(maschinenName, false);
	}
//=============================================================================
//
//	/**Die aktuell gltige Maschine der Applikation (MstApp Object)*/
//	public final Maschine getMaschine() {return this.maschine;}
//	/**Die aktuell gltige Maschine der Applikation (MstApp Object) @throws NullPointerException, falls maschine == null*/
//	public final void setMaschine(Maschine maschine) {
//		if(maschine == null){throw new NullPointerException("Der MaschineMgr aktzeptiert nicht null als maschine.");}
//		this.maschine = maschine;
//	}
//
//=============================================================================

	/**@return ein Array aller definierten Maschinen*/
	public final Maschine[] getMaschinenAlle()	{return Maschine.values();}
	/**Maschine dieser Applikation, nie null*/
	public final Maschine getMaschine() 		{return this.maschine;}
	/**Maschine dieser Applikation @throws NullPointerException, falls maschine == null*/
	public final void setMaschine(final Maschine maschine) {
		if(maschine == null)
			throw new NullPointerException("Der MaschineMgr aktzeptiert nicht null als maschine.");
		if(maschine == getMaschine())
			return;	//nichts zu tun, da selbe Maschine
		if(!(getKonfiguration().istErlaubt(maschine))){
			final String msg =   "Der MaschineMgr aktzeptiert die Maschine '"
				         + maschine
				         + "' nicht, da sie nicht in der MaschineKonfiguration '"
				         + getKonfiguration().toString()
				         + "' erlaubt ist.";
			throw new IllegalArgumentException(msg );
		}
		this.maschine = maschine;
		fireNeueMaschine(maschine);
	}

	public final boolean isValidMaschineNr(final int maschineNr){return Maschine.isValidMaschineNr(maschineNr);}
	/**@throws IndexOutOfBoundsException, if not isValidMaschineNr(maschineNr)*/
	public final Maschine getMaschineByNr	 (final int    maschineNr)		{return Maschine.getMaschineByNr(maschineNr);}
	public final Maschine getMaschineByString(final String maschinenString)	{return Maschine.getMaschineByString(maschinenString);}

	private final MaschinenKonfiguration getKonfiguration() 		{return getMaschineKonfiguration();}

	public final void registerIMaschineObserver(final IMaschineObserver observer){
		if(observer != null) {
			getMaschineObservers().add(observer);
			observer.onMaschineChange(this.getMaschine());
		}
	}
	public final boolean deregisterIMaschineObserver(final IMaschineObserver observer){
		return getMaschineObservers().remove(observer);
	}
//-----------------------------------------------------------------------------

	/**maschineMgr wird mit fileDataIni Info synchronisiert (File -> maschineMgr)*/
	@Override public void onNewIniAvailable(final FileDataIni fileDataIni) {
		if(fileDataIni.istInitialisiertMaschinenNr()){
			final int maschinenNr = fileDataIni.getMaschinenNr();
			if(isValidMaschineNr(maschinenNr)){
				final Maschine maschine = Maschine.getMaschineByNr(maschinenNr);
				this.setErlaubteMaschinenUndMaschine(maschine);
//				setMaschine(maschine);
				getKonfiguration().set(maschine);
			}
			else{Logger.SWFW.error("IniFile Problem: Illegale Maschinen Nummer: " + maschinenNr + " . Aktion: Wird ignoriert.");}
		}
	}
	/**fileDataIni wird mit Info maschineMgr synchronisiert (maschineMgr -> File)*/
	@Override public void onPrepareForExport(final FileDataIni fileDataIni) {
            final int maschineNr = getMaschine().getMaschineNr();
			fileDataIni.setMaschinenNr(maschineNr);
	}

	@Override public Object getDefaultIniBaumRoot()				{throw new UnsupportedOperationException();}
	@Override public Object onPrepareForExport()					{throw new UnsupportedOperationException();}
	@Override public void   onNewIniAvailable(final Object iniBaumRoot)	{throw new UnsupportedOperationException();}

//=============================================================================

	private List<IMaschineObserver> getMaschineObservers() {
		return this.maschineObservers;
	}

	private void fireNeueMaschine(final Maschine maschine) {
		Maschine.setMaschine(maschine);	//Vorsicht singleton pro run
		for (final IMaschineObserver maschineObserver : getMaschineObservers()) {
			maschineObserver.onMaschineChange(maschine);
		}
	}

//=============================================================================
	/**
	 * @param anyMaschine
	 * @return true falls diese Maschine erlaubt/enabled ist
	 */
	public boolean istErlaubteMaschine(final Maschine anyMaschine) {
		return this.maschinenKonfiguration.istErlaubt(anyMaschine);
	}

	/**
	 * @return Alle Erlaubten Maschinen als ReadOnlyListe, nie null und .size() > 0
	 */
	public List<Maschine> getErlaubteMaschinenASReadOnlyList() {
		return getMaschineKonfiguration().getASReadOnlyList();
	}
	/**
	 * @param erlaubteMaschinen  maschinen die zusaetzlich erlaubt sind
	 */
	public void addErlaubteMaschinen(final List<Maschine> erlaubteMaschinen) {
		getMaschineKonfiguration().add(erlaubteMaschinen);
	}

	/**
	 * @param maschine  eine Maschine, die zusaetzlich erlaubt ist
	 */
	public void addErlaubteMaschinen(final Maschine maschine) {
		getMaschineKonfiguration().add(maschine);
	}
	/**
	 * @param maschine wird entfernt
	 * @throws NullpointerException, 	falls maschine == null
	 * @throws IllegalArgumentException	falls maschine == getMaschine() ist
	 */
	public void removeErlaubteMaschinen(final Maschine maschine) {
		if(maschine == this.getMaschine()){
			final String msg =  "Kann die Maschine ("
						+ maschine.toString()
						+ ") nicht entfernen, da sie angewaehlt ist.";
			throw new IllegalArgumentException(msg);
		}
		getMaschineKonfiguration().remove(maschine);
	}
	/**
	 * @param erlaubteMaschinen voellig neue Liste erlaubter Maschinen
	 * @throws NullpointerException 	falls erlaubteMaschinen == null
	 * @throws IllegalArgumentException	falls erlaubteMaschinen keine Maschinen enthaelt, oder gesetzte Maschine fehlt
	 */
	public void setErlaubteMaschinen(final List<Maschine> erlaubteMaschinen) {
		if(!erlaubteMaschinen.contains(this.getMaschine())){
			final String msg =  "erlaubteMaschinen ("
						+ erlaubteMaschinen.toString()
						+ ") muessen die gesetzte("
						+ this.getMaschine().toString()
						+ ") enthalten";
			throw new IllegalArgumentException(msg);
		}
		MaschineGuessBestSetting.deregisterObserver(this.observer);
		getMaschineKonfiguration().set(erlaubteMaschinen);
	}
	/**
	 * @param erlaubteMaschinen voellig neue Liste erlaubter Maschinen
	 * @throws NullpointerException, 	falls erlaubteMaschinen == null
	 * @throws IllegalArgumentException	falls erlaubteMaschinen keine Maschinen enthaelt, oder gesetzte Maschine fehlt
	 */
	public void setErlaubteMaschinen(final Maschine ... erlaubteMaschinen) {
		this.setErlaubteMaschinen(Arrays.asList(erlaubteMaschinen));
	} 
	/**
	 * @param maschine
	 * @param erlaubteMaschinen voellig neue Liste erlaubter Maschinen
	 * @throws NullpointerException, 	falls erlaubteMaschinen == null
	 * @throws IllegalArgumentException	falls erlaubteMaschinen nicht maschine
	 */
	public void setErlaubteMaschinenUndMaschine(final Maschine maschine, final List<Maschine> erlaubteMaschinen) {
		if(maschine 			== null) {throw new NullPointerException(         "maschine darf nicht null sein");}
		if(erlaubteMaschinen	== null) {throw new NullPointerException("erlaubteMaschinen darf nicht null sein");}
		if(!erlaubteMaschinen.contains(maschine)){
			final String msg =  "erlaubteMaschinen ("
						+ erlaubteMaschinen.toString()
						+ ") muessen die maschine("
						+ maschine
						+ ") enthalten";
			throw new IllegalArgumentException(msg);
		}
		MaschineGuessBestSetting.deregisterObserver(this.observer);
		getKonfiguration().set(erlaubteMaschinen);
		this.setMaschine(maschine);
	}
	/**setzt Maschine zu maschine und die Liste der erlaubten Maschinen zu lediglich maschine*/
	public void setErlaubteMaschinenUndMaschine(final Maschine maschine) {
		final List<Maschine> list = new ArrayList<Maschine>(1);
		list.add(maschine);
		this.setErlaubteMaschinenUndMaschine(maschine, list);
	}

	/**
	 * @param observer wird bei Konfigurationsaenderungen benachrichtigt
	 */
	public void registerIMaschinenKonfigurationObserver(final IMaschinenKonfigurationObserver observer) {
		getMaschineKonfiguration().registerIMaschinenKonfigurationObserver(observer);
	}
	/**
	 * @param observer wird deregistriert, keine referenzen werden gehalten, keine callbacks mehr
	 */
	public void deregisterIMaschinenKonfigurationObserver(final IMaschinenKonfigurationObserver observer) {
		getMaschineKonfiguration().deregisterIMaschineObserver(observer);
	}
	@Override public void finalize(){
		MaschineGuessBestSetting.deregisterObserver(this.observer);
		try {
			super.finalize();
		} catch (final Throwable e) {

		}
	}

	/**PRIVATE, aussenzugriff nur via Delegation um zu checken ob gesetzte Maschine auch gueltig ist*/
	private MaschinenKonfiguration getMaschineKonfiguration() {
		return this.maschinenKonfiguration;
	}
}
