/*
 * Created on 	Feb 18, 2005 - Apr 7, 2005
 * 				Sep 20, 2006 - Sep 20, 2006	ws: added getModesHigherThan(OpMode)
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 */
package de.desy.mst.libs.framework.mstapp.mode;

import java.util.Arrays;

/**
 * Tells which OpModes are allowed.
 * Value equality == Object equality<br>
 * They can not be changed.<br><br>
 * 
 * Naming convention of constants:<br>
 * 	Betrieb,Betrachten,Expert
 * 	<li>Capital letter: mode is allowed mode	</li>
 *  <li>"_": 			mode is not allowed	</li>
 *  <br><br>
 *  <b>Im Detail:</b>
 *  <li><b>OpModeListe.BBE</b> BetrachtenBetriebExpert	</li>
 *  <li><b>OpModeListe._BE</b> _BetriebExpert			</li>
 *  <li><b>OpModeListe.B_E</b> Betrachten_Expert		</li>
 *  <li><b>OpModeListe.BB_</b> BetrachtenBetrieb_		</li>
 *  <li><b>OpModeListe.__E</b> __Expert					</li>
 *  <li><b>OpModeListe._B_</b> _Betrieb_				</li>
 *  <li><b>OpModeListe.B__</b> Betrachten_				</li>
 *  <li><b>OpModeListe.EMPTY_LISTE</b> ___				</li>
 * <br>
 * @author schuette 
 */
public final class OpModeListe {	//final assumed in usage
//	public static final OpModeListe DEFAULTLIST;
	/**
	 * BetrachtenBetriebExpert:
	 */
	public static final OpModeListe BBE;	
	/**
	 * BetriebExpertListe:
	 */
	public static final OpModeListe _BE;
	/**
	 * BetrachtenExpertListe:
	 */
	public static final OpModeListe B_E;
	/**
	 * BetrachtenBetriebListe:
	 */
	public static final OpModeListe BB_;	
	/**
	 * ExpertListe:
	 */
	public static final OpModeListe __E;	
	/**
	 * BetriebListe:
	 */
	public static final OpModeListe _B_;	
	/**
	 * BetrachtenListe:
	 */
	public static final OpModeListe B__;	
	/**
	 * EmptyListe:
	 */
	public static final OpModeListe EMPTY_LISTE;	
	
	public static final OpModeListe[] VALUES;
	
	static{
		BBE = new OpModeListe("BBE", true,  true,  true );
		_BE = new OpModeListe("_BE", false, true,  true );
		B_E = new OpModeListe("B_E", true,  false, true );
		BB_ = new OpModeListe("BB_", true,  true,  false);
		__E = new OpModeListe("__E", false, false, true );
		B__ = new OpModeListe("B__", true,  false, false);
		_B_ = new OpModeListe("_B_", false, true,  false);
		EMPTY_LISTE = new OpModeListe("EMPTY_LISTE", false, false,  false);
			
//		DEFAULTLIST = BB_;
		
		VALUES = (new OpModeListe[] {BBE, _BE, B_E, BB_, __E, B__, _B_,EMPTY_LISTE});
	}


	public final String name;
	//Index of  == index of OpMode
	private final boolean[] modeArray;	//might later internally be represented by a bitarray
	
//=============================================================================
	
	
	//true if not null and at least one true entry
	public static boolean isGefuellterModeArray(final boolean[] modeArray) {
		if(modeArray == null){return false;}
		for(int index = 0; index < modeArray.length; index++){
			if(modeArray[index]){return true;}
		}
		return false;
	}
	
	
	/**Ergibt eine OpModeListe die nur opModes aus beiden Listen enthaelt*/
	public static OpModeListe and(final OpModeListe list1, final OpModeListe list2){
		final boolean[] a1  = list1.getModeArray(); 
		final boolean[] a2  = list2.getModeArray();
		final boolean[] a12 = new boolean[a1.length];

		for (int i = 0; i < a1.length; i++) {
			a12[i] = a1[i] && a2[i];
		}
		return opModeFromBooleanArray(a12);
	}
	
	/**Ergibt die logisch invertierte OpModeListe*/
	public static OpModeListe not(final OpModeListe list){
		final boolean[] a1  = list.getModeArray(); 
		final boolean[] a2 = new boolean[a1.length];

		for (int i = 0; i < a1.length; i++) {
			a2[i] = ! a1[i];
		}
		return opModeFromBooleanArray(a2);
	}
	
//=============================================================================
	
	private OpModeListe(final String name, final boolean betrachten, final boolean betrieb, final boolean expert) {
		super();
		this.name = name;
		final boolean[] modeArray = new boolean[OpMode.VALUES.length];
		modeArray[OpMode.BETRACHTEN.index] = betrachten;
		modeArray[OpMode.BETRIEB.index] = betrieb;
		modeArray[OpMode.EXPERT.index] = expert;
		this.modeArray = modeArray;
	}
	

	
//=============================================================================
		
	public boolean isInList(final OpMode opMode){
		if(opMode != null){
			return this.modeArray[opMode.index];
		}
		return false;
	}
	/**
	 * @return 	Returns a clone of the modeArray. Always a valid non null array.
	 * 			must be a clone, so user can not change values in an unchecked fashion
	 */
	public boolean[] getModeArray() {
		final boolean[] cloneModeArray = new boolean[this.modeArray.length];
		System.arraycopy(this.modeArray, 0, cloneModeArray, 0, this.modeArray.length);
		return cloneModeArray;
	}

	//OpMode mit kleinstem Index oder null
	public OpMode getDefaultOpMode() {
		for(int index = 0; index < this.modeArray.length; index++){
			if(this.modeArray[index]){return OpMode.valueOf(index);}
		}
		return null;
	}
 
	public static OpModeListe getModesHigherThan(final OpMode opMode) {
		final int minOpModeIndex = (opMode == null) ? -1 : opMode.index;
		for (final OpModeListe liste : VALUES) {
			if(liste.allModesAreHigher(minOpModeIndex)) return liste;
		}
		return null;
	}
	private final boolean allModesAreHigher(final int minOpModeIndex) {
		final boolean[] modeArray = getModeArray();
		for (int i = 0; i < modeArray.length; i++) {
			if( i <= minOpModeIndex && modeArray[i]){
				return false;
			}
			if( i > minOpModeIndex && !modeArray[i]){
				return false;
			}
		}
		return true;
	}


	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override public String toString() {
		return this.name;
	}
	
//=============================================================================	
	
	private static OpModeListe opModeFromBooleanArray(final boolean[] a12) {
		for (int i = 0; i < VALUES.length; i++) {
			if(Arrays.equals(VALUES[i].getModeArray(), a12)){
				return VALUES[i];
			}
		}
		throw new IllegalArgumentException("Bad ModeArray");	//sollte nie passieren 
	}
}
