/**
 * 	May 16, 2007 - May 16, 2007	ws:	creation from scratch
 *	Aug  6, 2009 - Aug  7, 2009	ws:	hunt KiHi Problem 
 */
package de.desy.mst.libs.framework.mstapp.preferences.examples;

import java.awt.Frame;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer;
import de.desy.mst.libs.framework.mstapp.preferences.IPreferences;
import de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanel;
import de.desy.mst.libs.framework.mstapp.preferences.helper.APreferences;

/**
 * @author ws
 * version 1.0.0
 */
/**
 * Managed die Preference AlwaysOnTop<br>
 * Nicht Grafische Preferenz Klasse (IPreferences ohne Speichern)
 * <br> Wird einmal pro Anwendung erzeugt und bleibt im Memory
 * <br> Aufgabe: bei Bedarf frisches Grafisches Teil initialisiert erzeugen
 * <br> Verboten: Halten von Referenzen auf den grafischen Teil ueber das
 * zeigen desselben hinaus.
 */
public class PrefOnTop extends APreferences{
	private final static String TITLE = "Hauptfenster immer vorn";
	private final static String[] KEYWORDS = new String[]{"OnTop", "vorn"};
	
	private final Frame frame;
//=============================================================================	
	/**
	 * @param parent optionaler Hierarchisch uebergeordnete Container 
	 * Preference (kann null sein)
	 * @param frame der Frame der evtl. onTop sein soll (nie null)
	 * @throws NullPointerException, 
	 * falls der frame der onTop sein soll null ist
	 */
	public PrefOnTop(final IPreferences parent, final Frame frame) {
		super(TITLE, parent, KEYWORDS);
		if(frame == null){
			throw new NullPointerException("Der frame darf nicht null sein");
		}
		this.frame = frame;
	}
//=============================================================================	
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APreferences#createPrefClientPanel(de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer)
	 */
	@Override 
	public APrefClientPanel createPrefClientPanel(IPrefContainer prefContainer){
		return new PrefClientPanelOnTop(prefContainer, this.frame);
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APreferences#onDispose()
	 */
	@Override public void onDispose(){super.onDispose();/*no local refs to panel*/}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APreferences#onNewPrefString(java.lang.String)
	 */
	@Override public void onNewSaveString(final String saveString) {
		final boolean _default = this.frame.isAlwaysOnTop();
		final boolean alwaysOnTop = decodeSaveString(saveString, _default);
		this.frame.setAlwaysOnTop(alwaysOnTop);
		Logger.SWFW.info("PrefOnTop.onNewSaveString den frame gesetzt zu "
					   + alwaysOnTop);
	}
//	=============================================================================
	/**Die Preferenzes bestehen nur aus einer boolschen Variablen*/
	public static String encodeSaveString(final boolean alwaysOnTop) {
		return "" + alwaysOnTop;
	}
	/***/
	public static boolean decodeSaveString(final String saveString, final boolean _default) {
		try {
			return Boolean.parseBoolean(saveString);
		} catch (Exception e) {
			final String msg = "Kann den SaveString fr PrefOnTop = '" 
							 + saveString 
							 + "' nicht interpretieren. Assume default: " 
							 + _default;
			Logger.SWFW.warn(msg, e);
			return _default;
		}
	}
}