/**
 *  Created on 	Aug 23, 2005 - Sep 16, 2005
 *	Oct 25, 2005 - ...				new logging system
 *	Apr 10, 2006 - Apr 10, 2006 	impl: getZyklusModeTabelle
 *	Jun 21, 2006 - Jun 21, 2006	ws: guess Maschine und erlaubte Maschinen
 *	Jul 11, 2006 - Jul 12, 2006 ws: getTabelleAsObject wegen Eclipse 3.2 Generics Problem, entferne Zyklus*
 *	Aug 29, 2006 - Aug 29, 2006	ws: added getDeviceDetailTabelle
 *	Oct 05, 2006 - Oct 05, 2006	ws: added getColumnInfoTabelle
 *	Dez 01, 2008 - Dez 01, 2008	ws: improve handleTabIdConsistency(...)
 */
package de.desy.mst.libs.framework.mstapp.tabellendaten;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import de.desy.mst.libs.framework.mstapp.ini.TabDataHome;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.mode.MaschineGuessBestSetting;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.AEntry;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.AEntry2;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.Aufgabe;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.Device;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.DeviceDetail;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.DeviceGruppe;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.Status;
import de.desy.mst.libs.framework.mstapp.tabellendaten.generics.ColumnInfo;

/**
 * Haelt statisch alle geladenen Tabellen.
 * Laedt auf Anfrage weitere.
 * Rein statische Klasse, es existieren keine Objekte.
 * @author ws
 * @version 1.0.0
 */
public class AlleTabellen{
	//Tabellen gelten immer Appweit, sogar PC weit, sogar System Weit
	@SuppressWarnings("rawtypes") private final static Map<String, Tabelle> tabellen = new HashMap<String, Tabelle>();
	private static boolean thisIsTheFirstCallToGetDeviceTabelle = false;

//=============================================================================

	/**Rein Statische Klasse, es existieren keine Objekte dieser Klasse*/
	private AlleTabellen() {super();}

//=============================================================================

	/**Anzahl der geladenen Tabellen*/
	public static final synchronized int size(){
		return tabellen.size();
	}

	public static final boolean istGeladen(final String tabId) {
		return tabellen.containsKey(tabId);
	}

	/**Standard Weg um zu einer bestimmten Status Tabelle zu kommen*/
	public final static Tabelle<Integer, Status> getStatusTabelle(final Aufgabe aufgabe){
		return getStatusTabelleByAufgabenNummer(aufgabe.ordinal());
	}
	/**
	 * falls gewuenschte Aufgabe wirklich noch nicht in der Klasse sein sollte,
	 * kann man sich die Status Tabelle auch hiermit besorgen
	 */
	public final static Tabelle<Integer, Status> getStatusTabelleByAufgabenNummer(final int aufgNr){
		return getTabelle(Aufgabe.createStatusAbfrage(aufgNr));
	}

	/**Standard Weg um zu einer bestimmten Status Tabelle zu kommen*/
	public final static <D extends Device> DeviceTabelle<D> getDeviceTabelle(final DeviceGruppe deviceGruppe){
		return getDeviceTabelle(deviceGruppe.tabId);
	}

	/**Standard Weg um an die Spalteninformation einer Tabelle zu kommen*/
	public final static Tabelle<Integer, ColumnInfo> getColumnInfoTabelle(final String tabellenName){
		return getTabelle("ColumnInfoXML0+@tableName='" + tabellenName + "'");
	}

	/**
	 * LOW Level Retrieval Funktion, man muss hier den String kennen
	 *
	 * Falls tabelle nicht geladen ist,
	 * wird versucht sie zu laden,
	 * falls nicht geht wird null geliefert
	 */
	@SuppressWarnings("rawtypes")
	public static final Tabelle getTabelleAsObject(final String tabId) {
		return istGeladen(tabId) ? tabellen.get(tabId) : load(tabId);
	}
	/**
	 * Retrieval Funktion fuer Tabelle mit einem Schluessel.<br>
	 * Benutzt {@link #getTabelleAsObject(String)}
	 */
	@SuppressWarnings("unchecked")
	public static final <K extends Comparable<K>,  E extends AEntry<K>>
						Tabelle<K,E> getTabelle(final String tabId) {
		return getTabelleAsObject(tabId);
	}
	/**
	 * Retrieval Funktion fuer Tabelle mit zwei Schluesseln.<br>
	 * Benutzt {@link #getTabelleAsObject(String)}
	 */
	@SuppressWarnings("unchecked")
	public final static 	< K1 extends Comparable<K1>
							, K2 extends Comparable<K2>
							, E  extends AEntry2<K1,K2>
							>
							Tabelle2<K1, K2, E> getTabelle2(final String tabId) {
					return (Tabelle2<K1, K2, E>) getTabelleAsObject(tabId);
	}
	/**
	 * LOW Level Retrieval Funktion, man muss hier den String kennen.
	 * Ausserdem muss man wissen, dass auch eine DeviceTabelle geliefert wird.
	 *
	 * Falls tabelle nicht geladen ist,
	 * wird versucht sie zu laden,
	 * falls nicht geht wird null geliefert
	 */
	@SuppressWarnings("unchecked")
	public final static 	<D  extends Device>
							DeviceTabelle<D> getDeviceTabelle(final String tabId) {
					final Object tabelle = getTabelleAsObject(tabId);
					final DeviceTabelle<D> deviceTabelle = (DeviceTabelle<D>)tabelle;
					if(thisIsTheFirstCallToGetDeviceTabelle){
						thisIsTheFirstCallToGetDeviceTabelle = true;
						MaschineGuessBestSetting.guessBestSettings(DeviceGruppe.valueOf(tabId));
					}
					return deviceTabelle;
	}

	/**
	 * LOW Level Retrieval Funktion, man muss hier den String kennen.
	 * Ausserdem muss man wissen, dass auch eine DeviceTabelle geliefert wird.
	 *
	 * Falls tabelle nicht geladen ist,
	 * wird versucht sie zu laden,
	 * falls nicht geht wird null geliefert
	 */
	@SuppressWarnings("unchecked")
	public static <D extends DeviceDetail> DeviceDetailTabelle<D> getDeviceDetailTabelle(final String tabId) {
		final Object tabelle = getTabelleAsObject(tabId);
		final DeviceDetailTabelle<D> deviceDetailTabelle = (DeviceDetailTabelle<D>)tabelle;
		return deviceDetailTabelle;
	}

//	/**Direct in GEntryTabelle machen*/
//	public static GEntryTabelle getGEntryTabelle(final String tabId) {
//		Object tabelle = getTabelleAsObject(tabId);
//		GEntryTabelle gEntryTabelle = (GEntryTabelle)tabelle;
//		return gEntryTabelle;
//	}

	/**read only statische "Version von Iterable" ueber Set<String> getTabellenIdSet()*/
	public static final Iterator<String> iterator() {
		return getTabellenIdSet().iterator();
	}

	/**read only Collection aller geladenen tabIds*/
	public static Set<String> getTabellenIdSet() {
		return Collections.unmodifiableSet(tabellen.keySet());
	}

	/**read only Collection aller geladenen Tabellen*/
	@SuppressWarnings("rawtypes") public static Collection<Tabelle> getTabellenAlsCollection() {
		return Collections.unmodifiableCollection(tabellen.values());
	}

//=============================================================================

	/**ignoriert null Werte*/
	@SuppressWarnings("rawtypes") static final Tabelle add(final Tabelle tabelle) {
		if(tabelle == null){return null;}
		return tabellen.put(tabelle.tabId, tabelle);
	}
	@SuppressWarnings("rawtypes") static final Tabelle remove(final String tabId) {
		return tabellen.remove(tabId);
	}
	static final void clear() {
		tabellen.clear();
	}

//=============================================================================

	@SuppressWarnings("rawtypes") private static final Tabelle load(final String tabId) {
		final Tabelle tabelle = TabDataHome.getTabelleImplementation(tabId);
		handleTabIdConsistency(tabId, tabelle);
		add(tabelle);
		return tabelle;
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private static void handleTabIdConsistency(final String tabId, final Tabelle tabelle) {
		final String bekommeneTabId = tabelle.getTabId();
		if(!tabelle.hasTrivialTabId() 
		&& !tabId.contains(bekommeneTabId)
		&& !tabId.contains(bekommeneTabId.replace("'null'", "null"))) {
			final String msg = "tabId Mismatch bekommen '" + bekommeneTabId
							 + "' setze (eh) zu erwartetem '" + tabId + "'";
			Logger.SWFW.warn(msg, new Exception("tabId Mismatch"));
		}else{
			/*NOP: replacement im Prinzip egal*/;
		}

		tabelle.tabId = tabId;
	}
}
