/**
 *  Created on 	Sep 14, 2005 - Sep 14, 2005
 *	Oct 25, 2005 - ...	new logging system
 * 	Jul 14, 2009 - Jul 14, 2009	ws:	fixed some Eclipse 3.5 warnings
 */
package de.desy.mst.libs.framework.mstapp.tabellendaten;

import java.util.Collections;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.AEntry2;

/**
 * Tabelle fuer AEntry2 entries, 
 * d.h. Entries mit primaeren und sekundaeren 
 * sortierbaren Key (2 == 2 keys)<br>
 * Z.B. Device Verzeichnisse haben 
 * <li> devNo	== primaerer   Key (getKey(); )
 * <li> devName	== sekundrer Key (getKey2();)
 * 
 * @author ws
 * @version 1.0.0
 */
public class Tabelle2 	< K1 extends Comparable<K1>
						, K2 extends Comparable<K2>
						, E  extends AEntry2<K1,K2>
						> 
			 extends Tabelle <K1,E>{
//=============================================================================
	/** 
	 * never null, muss aber im XTor gesetzt werden.
	 * enthaelt gleiche Elemente wie entries in parent, 
	 * (illegale doppelte keys/ints Inkonsistenzen drfen das nicht sprengen)
	 */
	protected Map<K2, E> entries2;	//pseudoFinal
//=============================================================================

	/**@param tabId wie @link de.desy.mst.mstapp.tabellendaten.Tabelle*/
	public Tabelle2(final String tabId) {
		super(tabId);
		this.entries2 = new TreeMap<K2,E>();
	}
	
//=============================================================================
	
	public final boolean containsKey2(final K2 key2) 	{return this.entries2.containsKey(key2);}
	public final E getEntry2(final K2 key2) {return this.entries2.get(key2);}
    /**
     * @return an Iterator over the values as value type sortiert nach dem sekundren Key.
     */
	public final Iterator<E> iterator2() {
		final Map<K2, E> unmodifiableMap = Collections.unmodifiableMap(this.entries2);
		return unmodifiableMap.values().iterator();
	}
	
//=============================================================================
	
	/**
	 * only for XMLDecoder use 
	 * 
	 * @throws NullPointerException, 	 if the specified element is null
	 * @throws IllegalArgumentException, if the specified element already exists
	 */
	@Deprecated
	@Override
	public boolean add(final E zeile) {
		super.add(zeile);	//checks for null
		final K2 key2 = zeile.getKey2();
		if (containsKey2(key2)) {
			final String msg = "Key2 " + key2 + " existiert schon in der Tabelle,"
 				                        + " neuer Eintrag wird ignoriert.";
			final IllegalArgumentException e = new IllegalArgumentException(msg);
			Logger.SWFW.error(msg, e);
			throw e;
		}
		this.entries2.put(key2, zeile);
		if(this.firstEntry == null){this.firstEntry = zeile;}
		return true;
	}
} 
