/**
 * Oct 05, 2006 - Oct 06, 2006	ws:	created from scratch
 * Oct 20, 2006 - Oct 23, 2006	ws:	XMLDecoder kann varArgs nicht also einzelne
 */
package de.desy.mst.libs.framework.mstapp.tabellendaten.generics;

import java.util.Arrays;
import java.util.concurrent.atomic.AtomicLong;

import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.AEntry;

/**
 * Generisches Entry.<br>
 * Der Long Schluessel getKey() ist klassenweit autogeneriert und gehoert nicht
 * zu den Daten der Klasse.<br>
 * Die Sortierfolge ist die Erzeugungsfolge der Objekte.<br>
 * Die Daten sind vollstaendig und koenne mit getParameter(0 basierten Index)
 * abgefragt werden.<br>
 * Der Konstruktor (optionale Parameter) wird nicht direkt vom XMLDecoder 
 * unterstuetzt. Deshalbt bitte die Unterklasse GEntry benutzen.
 *  
 * @author ws
 */
class GEntryImpl extends AEntry<Long> {
	private static AtomicLong	classKey = new AtomicLong(0);
	private final long key;	
	private final Object[] theParameter;//auch die einzelnen Parameter konstant
//=============================================================================
	/**dummy nur um diesen von den expliziten XToren zu unterscheiden*/
	public GEntryImpl(final Object ... theParameter) {
		super();
		this.theParameter = theParameter;
		this.key = createKey();
	}
	
//=============================================================================
	/**
	 * @param index 0 based Index der Parameter (Java Welt)
	 * @return	der entsprechende Index oder null - nie eine Exception
	 */
	public Object getParameter(final int index){
		if(this.theParameter == null)			return null;
		if(index < 0) 						return null;
		if(index >= this.theParameter.length )	return null;
		return this.theParameter[index];
	}
	public int getMaxIndex(){
		return (this.theParameter == null) ? -1 : this.theParameter.length - 1;
	}
//=============================================================================
	@Override public Long 	 getKey() {return Long.valueOf(this.key);}	/**
	 * Der Schluessel getKey() gehoert hierbei NICHT zu den Kriterien sondern
	 * nur die Daten selbst getParameter(allen indexen).
	 */
	@Override public boolean inSeinenWertenGleich(final AEntry<Long> otherEntry) {
		if(otherEntry instanceof GEntryImpl){
			final GEntryImpl other = (GEntryImpl) otherEntry;
			return Arrays.equals( other.theParameter, this .theParameter);
		}
		return false;
	}
//=============================================================================
	/**Threadsafe*/
	private static final long createKey() {
		return classKey.incrementAndGet();
	} 
}
