/**
 * Oct 26, 2006 - Nov 1, 2006	ws:	created from scratch
 * Nov  2, 2006 - Nov 2, 2006 	al: removed @SuppressWarnings("unused")
 * 								ws: put before method 
 * 									fillWithEmptyStringsForEachIteratorEntry()
 * 									damit Modul ohne Warnung
 * Nov  7, 2006 - ...			ws: hasSpalte, getSpaltenNummer ignore case
 */
package de.desy.mst.libs.framework.mstapp.tabellendaten.generics;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.tabellendaten.AlleTabellen;
import de.desy.mst.libs.framework.mstapp.tabellendaten.Tabelle;
import de.desy.mst.libs.framework.mstapp.util.reflection.ISpaltenInfo;

/**
 * SpaltenInfo fuer die Generische Klasse via eine TabName DBAbfrage realisiert.
 * @author ws
 */
public class GEntrySpaltenInfo implements ISpaltenInfo<GEntry> {
	public  final String tabName;
	private final Tabelle<Integer, ColumnInfo> columnInfoTabelle;
	private List<String> spaltenNamen = Collections.emptyList();
	
//=============================================================================
	
	public GEntrySpaltenInfo(final String tabName) {
		super();
		this.tabName = tabName;
		this.columnInfoTabelle = createColumnInfoTabelle(tabName);
	}
	
//=============================================================================	

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.util.reflection.ISpaltenInfo#isTrivial()
	 */
	@Override public boolean isTrivial() {
		return isTrivialString(this.tabName) || this.columnInfoTabelle == null;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.util.reflection.ISpaltenInfo#getSpaltenNamen()
	 */
	@Override public List<String> getSpaltenNamen() {
		if (this.spaltenNamen.size() == 0){
			this.spaltenNamen =  createSpaltenNamen();
		}
		return this.spaltenNamen;
	}
	
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.util.reflection.ISpaltenInfo#getSpalte(java.lang.String, java.lang.String, java.lang.Iterable)
	 */
	@Override
	@SuppressWarnings({ "rawtypes", "unchecked" }) 
	public List<Object> getSpalte(final String	 id
								, final String	 spaltenName
								, final Iterable entryIterator) {
		final List<Object> spalte = new ArrayList<Object>();
		if(!this.hasSpalte(spaltenName)){
			final String msg = this.getClass().getName() + ".getSpalte: SpaltenName '" 
					   + spaltenName  + "' unbekannt in Tabelle '"
					   + this.tabName + "'.";
			Logger.SW.warn(msg);
			fillWithEmptyStringsForEachIteratorEntry(entryIterator, spalte);
		}
		else{
			final int iSpalte = this.getSpaltenNummer(spaltenName);
			for (final Object object : entryIterator) {
				if(object instanceof GEntry){
					spalte.add(((GEntry)object).getParameter(iSpalte));
				}else{
					spalte.add("");
				}
			}
		}
		return spalte;
	}

//=============================================================================
	
	protected int getSpaltenNummer(final String spaltenName) {
		return hasSpalteEnforceCase(spaltenName) ?
				getSpaltenNamen().indexOf(spaltenName) 
				:
				getSpaltenNummerIgnoreCase(spaltenName)	;
	}

	protected boolean hasSpalte(final String spaltenName) {
		return hasSpalteEnforceCase(spaltenName)
			|| hasSpalteIgnoreCase (spaltenName);
	}

//=============================================================================
		
	@SuppressWarnings("unchecked")
	private List<String>  createSpaltenNamen() {
		if(this.isTrivial()){
			return Collections.emptyList();
		}else{
			final List<?> spaltenNamen 
			= this.columnInfoTabelle.getSpalte(ColumnInfo.NAME);
			return (List<String>)spaltenNamen;
		}
	}
	@SuppressWarnings("unused") 
	private void fillWithEmptyStringsForEachIteratorEntry(
											final Iterable<Object> entryIterator, 
											final List<Object> spalte) {
		for (final Object object : entryIterator) {
			spalte.add("");
		}
	}

	private boolean isTrivialString(final String tabName) {
		return (tabName == null) || (tabName.length() == 0);
	} 
	private Tabelle<Integer, ColumnInfo> 
							createColumnInfoTabelle(final String tabName) {
		Tabelle<Integer, ColumnInfo> columnInfoTabelle = null; 
		try {
			columnInfoTabelle = AlleTabellen.getColumnInfoTabelle(tabName);
		} catch (final Exception e) {
			final String msg = "Can not create the ColumnInfoTable for: '" 
							  + tabName + "'.";
			Logger.SWFW.error(msg, e);
		} 
		return columnInfoTabelle;
	}
	
	private boolean hasSpalteEnforceCase(final String spaltenName) {
		return getSpaltenNamen().contains(spaltenName);
	}
	private boolean hasSpalteIgnoreCase(final String spaltenName) {
		final List<String> spaltenNamen = getSpaltenNamen();
		for (final String name : spaltenNamen) {
			if(!(name == null) && name.equalsIgnoreCase(spaltenName)){
				return true;
			}
		}
		return false;
	}

	private int getSpaltenNummerIgnoreCase(final String spaltenName) {
		final List<String> spaltenNamen = getSpaltenNamen();
		for (final String name : spaltenNamen) {
			if(!(name == null) && name.equalsIgnoreCase(spaltenName)){
				return getSpaltenNummer(name);
			}
		}
		return -1;
	}
}
