/**
 * Dec 11, 2007 - Dec 14, 2007	ws:	Creation from scratch/MstDateFormat
 * Apr  4, 2008 - Apr  7, 2008	ws: verstaendlichere Fehlermeldung/date robuster
 * May 30, 2008 - May 30, 2008	ws: in early return of parse wirt this.date
 * 									nicht gefuellt: fix it
 * Jun 10, 2008	- Jun 10, 2008	ws:	parseIsoDate(..) allow for 1970 not to be 3970
 * May 14, 2009	- May 14, 2009	ws:	added _parse(..) for ignorability of parseexception in safe cases
 */
package de.desy.mst.libs.framework.mstapp.util;

import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * Komplementarklasse zu MstDateFormat.ISO (Parsed die serialisierten Dates
 * wieder zu Dates zurueck)
 * @author ws
 * @version 1.0.0
 */
public class MstDateFormatISOParser {
	//-------------------------------------------------------------------------
	/**parsed MstDateFormat.ISO, falls kein Offset nehme UT an*/
	public static Date parse(final String source) throws ParseException {
		checkSourceForNullOrEmpty(source);
		final MstDateFormatISOParser parser = new MstDateFormatISOParser(source);
		return parser.getDate();
	}
	/**like pars but without a checked exception (for test classes only)*/
	public static Date _parse(final String source) throws IllegalArgumentException{
		try {
			return MstDateFormatISOParser.parse(source);
		} catch (final ParseException ex) {
			throw new IllegalArgumentException();
		}
	}
	//-------------------------------------------------------------------------
	private static void checkSourceForNullOrEmpty(final String source)
	throws ParseException {
		if( source == null
 		 || source.isEmpty()){
			throwParseException(source, "Text ist leer oder null.", -1);
		}
	}
	private static void throwParseException(final String source
										  , final String was
										  , final int errorOffset)
														throws ParseException {
		throw new ParseException("MstDateFormatParser: " + was + " in '"
								 + source
								 + "' ok z.B.: '2007-02-28 09:59:23.209'"
								 , errorOffset);
	}
	//-------------------------------------------------------------------------
	/**pp*/ static String[] dateParts(final String source) {
		return source.split("[ T]");
	}

//=============================================================================
	private Date	date;

	private int		year;
	private int 	month;
	private int 	day ;
	private int 	hrs;
	private int 	min;
	private int 	sec;
	private int		millis;

	private int 	offsetMS;

	private final String source;

//=============================================================================
	/**pp, since only for testing
	 * @throws ParseException */
	MstDateFormatISOParser(final String source) throws ParseException {
		this.source    = source;
		parse();
	}
//=============================================================================
	/**pp debug only*/ Date getDate() 			{return this.date;}
	/**pp debug only*/ int  getYear() 			{return this.year;}
	/**pp debug only*/ int  getMonth() 			{return this.month;}
	/**pp debug only*/ int  getDay() 			{return this.day;}
	/**pp debug only*/ int  getHour() 			{return this.hrs;}
	/**pp debug only*/ int  getMinute() 		{return this.min;}
	/**pp debug only*/ int  getSecond() 		{return this.sec;}
	/**pp debug only*/ int  getMillis() 		{return this.millis;}
	/**pp debug only*/ int  getOffsetInMillis()	{return this.offsetMS;}

//=============================================================================
	private String getSource() {return this.source;}
	private void throwParseException(final String was, final int dateIndex)
														throws ParseException {
		throwParseException(getSource(), was, dateIndex);
	}
//ISO: 	"2005-07-24 07:35:56.978 +0200"	"YYYY-MM-DD HH:mm:SS +HH.mm"
	private void parse() throws ParseException {
		parseParts();
		fillDateFieldWithParseResults();
	}
	private void parseParts() throws ParseException {
		final String[] dateParts = this.source.replaceAll("  ", " ")
										 .replaceAll("  ", " ")
										 .split("[ T]");
		int nextPartsIndex = 0;
		if( !(dateParts.length > nextPartsIndex)  ) {
			final String was = "Kein Datums Teil (wie z.B. '2007-04-01')";
			throwParseException( was, 0);
		}
		if(parseIsoDate(dateParts[nextPartsIndex]))
			nextPartsIndex++;
		if( !(dateParts.length > nextPartsIndex) )
			return;	//ignore time and offset part
		if(parseTime(dateParts[nextPartsIndex]))
			nextPartsIndex++;

		if( !(dateParts.length > nextPartsIndex) )
			return;	//ignore offset part
		parseOffset(dateParts[nextPartsIndex]);
	}

	private boolean parseIsoDate(final String datePart) {
		final String[] split = datePart.split("[-/\\.]");
		this.year  = Integer.valueOf(split[0]);
		if(this.year < 1969)	//0 ms entspricht 1970
			this.year += 2000; 
		this.month = Integer.valueOf(split[1]) - 1;
		this.day   = Integer.valueOf(split[2]);

		return true;
	}

	private boolean parseTime(final String timePart) {
		final String[] split = timePart.split("[:.]");
		if(split.length <= 1)
			return false;	//kein timePart maybe offsetpart

		this.hrs  	= Integer.valueOf(split[0]);
		this.min	= Integer.valueOf(split[1]);
		if(split.length < 3){
			return true;	//keine sekunden
		}
		this.sec	= Integer.valueOf(split[2]);
		if(split.length < 4){
			return true;	//keine subSekunden
		}
		this.millis	= Integer.valueOf(parseMillis(split[3].trim()));
		return true;
	}
	private int parseMillis(final String source) {
		if(source.isEmpty()){
			return 0;
		}else if(source.length() < 3){
			return parseMillis(source + "000");
		}else{
			final String substring = source.substring(0, 3);
			final Integer retVal = Integer.valueOf(substring);
			return retVal;
		}
	}
	private void parseOffset(final String offsetPart) throws ParseException {
		//+0200
		if(offsetPart.length() != 5){
			final String was = "Zeitzonen-Offset muss 5 Zeichen lang sein"
							 + " (ist = '" + offsetPart + "')"
							 + " statt z.B. '+0200'";
			throwParseException(was, Integer.MAX_VALUE);
		}
		final int vorzeichen = (offsetPart.charAt(0) == '+') ? (1) : (-1);
		final String hours 	 = offsetPart.substring(1, 3);
		final String minutes = offsetPart.substring(3);
		this.offsetMS = (Integer.valueOf(hours) * 60 + Integer.valueOf(minutes))
						* 60000 * vorzeichen;	//60*1000 ms in der Minute
	}

	private void fillDateFieldWithParseResults() {
		final GregorianCalendar calendar = new GregorianCalendar(
				this.year, this.month, this.day, this.hrs, this.min, this.sec);
		final long timeInMillisOhneOffsetKorrektur = calendar.getTimeInMillis() + this.millis;
		final int zone_offset = calendar.get(Calendar.ZONE_OFFSET);
		final int  dst_offset = calendar.get(Calendar.DST_OFFSET);
		final int offsetKorrektur = (dst_offset + zone_offset) - this.offsetMS;
		calendar.setTimeInMillis(timeInMillisOhneOffsetKorrektur + offsetKorrektur);
		this.date = calendar.getTime();
	}

//=============================================================================
}

