package de.desy.mst.libs.framework.mstapp.util.applicationresizemanager.listeners;

import java.awt.*;

import de.desy.mst.libs.framework.mstapp.util.applicationresizemanager.resizers.*;

/**
 * Listener resizing a JComponent
 * 
 * @author R.Bacher
 * @version 3.0.0
 * 
 */	
public class ComponentResizeListener {
	
	private Component myReferenceComponent = null;
	private ComponentResizer myComponentResizer = null;
	
	/**
	 * 
	 * Constructor
	 * 
	 * @param referenceComponent reference component to be resized
	 * @param componentResizer component resize manager attached
	 */
	public ComponentResizeListener(final Component referenceComponent, final ComponentResizer componentResizer) {
		
		this.myReferenceComponent = referenceComponent;
		this.myComponentResizer = componentResizer;
		initialize();
		
	}
	
	/**
	 * 
	 * Initializes the component resize manager
	 *
	 */
	private void initialize() {
	
		this.myReferenceComponent.addComponentListener(new java.awt.event.ComponentAdapter() {
	    	 
	    	/**
	    	 * Recieves the component listener event "componentResized" and calls the corresponding method to resize a Swing component.
	    	 * 
	    	 * @param listenerEvent Listener event
	    	 */	
	    	 @Override public void componentResized(final java.awt.event.ComponentEvent listenerEvent) {  
	    		 if (ComponentResizeListener.this.myComponentResizer != null) {
	    			 ComponentResizeListener.this.myComponentResizer.resizeComponent();
	    		 }
	    	 }
	     });	 
	}
	
}