/*
 * Created on Feb 15, 2005
 * 	Oct 16, 2006 - 	Oct 16, 2006	ws:	removed Guilleaumes (-> Markus Walla) 
 *
 */
package de.desy.mst.libs.framework.mstapp.util.observables.setvalue;

import javax.swing.AbstractButton;
import javax.swing.JLabel;

/**
 * Typesafe observable for objects that have a 'setText(String)' method
 *  
 * 'setText(String)' is the call back function/event
 * 
 * Support for JButton, JMenuItem, JLabel is already implemented.
 *  
 * If you need more kinds of objects i.e. classes
 * 1) provide a SPECIFIC (not for any Object) method add(NewClass newClass),
 * 2) provide a testcase in SetTextManagerTest
 * AND 
 * 3) run the test successfully
 *
 * Implemented with the non typesafe setvalue class
 * 
 * @author schuette
 */
public class SetTextManager extends SetValueManager{
	/**
	 * @param startString	a default String start with.
	 * 						null will be replaced by empty string "" 
	 */
	public SetTextManager(String startString) {
		//replace null with empty string, if necessary
		super((startString == null) ? "" : startString, "setText"); 
	}
	
	
	/**
	 * @param 	label 	a label for automated setText(...)
	 */
	public void add(JLabel label) {
		this.addObject(label);	//ok, since JLabel is ok
	}
	
	/**
	 * @param 	menuItemOrButtonOrEtc 	any type of AbstractButton 
	 * 									like JMenuItem, JButton, ...
	 * 									for automated setText(...)
	 */
	public void add(AbstractButton menuItemOrButtonOrEtc) {
		this.addObject(menuItemOrButtonOrEtc);	//ok, since AbstractButton is ok
	}

	/**
	 * Added setText(newText) for easy recognition of functionality
	 * Using setValue directly is equivalent.
	 * @param newText	all registered clients will be receive a method call 
	 * 					to setText(newText)
	 */
	public void setText(String newText) {
		this.setValue(newText);
	}
}
