/**
 *  Created on	Jan 27, 2006 - Feb 01, 2006
 * ws:	Jun 5, 2007 - Jun 5, 2007 added: createElementWithTextContent
 */
package de.desy.mst.libs.framework.mstapp.util.xml;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * @author ws
 * @version 1.0.0
 */
public abstract class AXmlBeschreibung implements IXmlBeschreibung {
	public final static Document createDocument() {
		try {
			final DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
			return builder.newDocument();
		} catch (final ParserConfigurationException e) {
			Logger.SWFW.error(e);
			return null;
		}
	}
	public static Element createElementWithTextContent
		(final Document document, final String elemName
								, final String textContent) {
		final Element element = document.createElement(elemName);
		element.setTextContent(textContent);
		return element;
	}

	public final static String getXMLBeschreibung(final Element element, int nTabs){
		if(element == null)			{return "";}
		if(nTabs < 0) 				{nTabs = 0;}
		return Appender.getXMLBeschreibung(new StringBuffer(), element, nTabs).toString();
	}
	
	
//=============================================================================
	
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.util.xml.IXmlBeschreibung#getXMLBeschreibung()
	 */
	@Override public final String getXMLBeschreibung(final int nTabs) {
		return getXMLBeschreibung(getBeschreibungAsElement(createDocument()), nTabs);
	}

//=============================================================================
	
	/**Statische Hilfsklassen fuer die Generierung der Beschreibung, package privat zum testen only*/
	static class Appender{
		/**sb never null, element never null, tabs >= 0*/
		public static StringBuffer getXMLBeschreibung(final StringBuffer sb, final Element element, final int nTabs) {
			appendElement(sb, element, nTabs);
			return sb;
		}

		static void appendElement(final StringBuffer sb, final Element element, final int nTabs) {
			final NodeList children = element.getChildNodes();
			if(children.getLength() > 0){
				Appender.appendTagStart	  	 (sb, element,  nTabs);
					Appender.appendElementList(sb, children, nTabs + 1);
				Appender.appendTagEnd	  	 (sb, element,  nTabs);
			}else{
				Appender.appendTagStartAndEnd(sb, element,  nTabs);
			}
		}
		
		static final void appendElementList(final StringBuffer buf, final NodeList children,final int nTabs) {
			if(children == null){return;}
			for (int i = 0; i < children.getLength(); i++) {
				final Node node = children.item(i);
				if(node == null){continue;}
				if(node instanceof Element){
					appendElement(buf, (Element)node, nTabs);
				}else if(node instanceof Text){
					appendText(buf, (Text)node, nTabs);
				}
			}
			
		}
		
		static void appendText(final StringBuffer buf, final Text text, final int nTabs) {
			doTabs(buf, nTabs);
			buf.append(text.getTextContent()).append('\n');
		}
		
		static void appendTagStartAndEnd(final StringBuffer buf, final Node node, final int nTabs) {
			doTabs(buf, nTabs);
			buf.append('<').append(node.getNodeName());
			appendTagAttributes(buf, node.getAttributes());
			buf.append('/').append('>').append('\n');
		}
		
		static final void doTabs(final StringBuffer buf, final int nTabs) {
			if(nTabs <= 0){return;}
			for (int i = 0; i < nTabs; i++) {
				buf.append('\t');
			}
		}

		static final void appendTagStart(final StringBuffer buf, final Element element, final int nTabs) {
			doTabs(buf, nTabs);
			buf.append('<').append(element.getNodeName());
			appendTagAttributes(buf, element.getAttributes());
			buf.append('>').append('\n');
		}
		static final void appendTagAttributes(final StringBuffer buf, final NamedNodeMap attributes) {
			if(attributes == null){return;}

			final int n = attributes.getLength();
			for (int i = 0; i < n; i++) {
				final Node attribute = attributes.item(i);
				if(attribute == null){continue;}
				buf.append(' ')
				.append(attribute.getNodeName())
				.append(new char[]{' ', '=', ' ' ,'"'})
				.append(attribute.getNodeValue())
				.append('"');
				if(i < n-1){buf.append(',');}
			}
		}

		static final void appendTagEnd(final StringBuffer buf, final Element element, final int nTabs) {
			doTabs(buf, nTabs);
			buf.append('<').append('/').append(element.getNodeName()).append('>').append('\n');
		}
	}

	protected static Element wrapInNamedElementWrapIn(final Document document, final String childName, final Element wrappeeElement) {
		final Element namedElement = document.createElement(childName);
		namedElement.appendChild(wrappeeElement);
		return namedElement;
	}
}
