/**
 *	Feb  2, 2010 - Mar 11, 2010	ws:	creation from scratch 
 *	Apr 12, 2010 - Apr 12, 2010	ws:	createQueue() mit factory statt sender direkt
 *	Jul  7, 2010 - Jul 15, 2010	ws:	added Update functionality
 *	Aug  5, 2010 - Aug  6, 2010	ws:	added AppAdmin.Statistic.getApplicationStatistic().fireStarted(...)
 *									added IExceptionNotification, ExceptionNotification
 */
package de.desy.mst.libs.info.appadmin;

import static de.desy.mst.libs.info.appadmin._LifeCycleStatisticTypeEntry.APPLICATION;
import static de.desy.mst.libs.info.appadmin._LifeCycleStatisticTypeEntry.MAINPANEL;
import static de.desy.mst.libs.info.appadmin._LifeCycleStatisticTypeEntry.PANEL;

/**
 * <b>Applikation administration support.</b>
 * <br> This class is application global and therefore static .
 * <li>statistics
 * <li>update support
 * 
 * <p>see also: ___AppAdmin___package_info.txt in this package
 * 
 * @author ws
 * @version 1.0.0
 */
public final class AppAdmin {
	static final _Queue queue;	//same internal queue for all messages to Topic
	static{
		AppAdminContext.setAppAdminIsInitialized();
		queue = createQueue();
		final _UpdateNotification _updateNotification = new _UpdateNotification(queue);
		_updateNotification.connectToJMSTopic();
		UpdateNotification = _updateNotification;
	}
	//---------------------------------------------------------------------
	private static _Queue createQueue() {
		final AppAdminContext context = AppAdminContext.getInstance();
		return new _Queue(context.getQueueCapacity(), context.getMsgSenderFactory());
	}
	
//=============================================================================
	
	/**API: fire here statistic events via 
	 * 		AppAdmin.Statistic.get*Statistic().fire*(...)
	 * <br> blocking free and threadsafe from the outside	 								
	 */
	public static final class Statistic{
		private static final String APP_PSEUDONAME = "###APP###";
		private static final long   APP_PSEUDOTAG  = -123L;

		private static final ILifeCycleStatistic applicationStatistic;
		private static final ILifeCycleStatistic panelStatistic;
		private static final ILifeCycleStatistic mainPanelStatistic;
		//---------------------------------------------------------------------
		static{
			applicationStatistic = new _LifeCycleStatistic(queue, APPLICATION); 
			panelStatistic       = new _LifeCycleStatistic(queue, PANEL);
			mainPanelStatistic   = new _LifeCycleStatistic(queue, MAINPANEL);
			
			fireApplicationStarted(); 
			//fireApplicationStopped() will be called in _MsgSenderJMS via shutdown hook
		}
		//---------------------------------------------------------------------
		public static final ILifeCycleStatistic getApplicationStatistic()	{
			return AppAdmin.Statistic.applicationStatistic;
		}
		public static final ILifeCycleStatistic getPanelStatistic()	{
			return AppAdmin.Statistic.panelStatistic;
		}
		public static final ILifeCycleStatistic getMainPanelStatistic(){
			return AppAdmin.Statistic.mainPanelStatistic;
		}
		//---------------------------------------------------------------------
		/**pp*/ static void fireApplicationStarted() {
			getApplicationStatistic().fireStarted(APP_PSEUDONAME, APP_PSEUDOTAG);
		}
		/**pp*/ static void fireApplicationStopped() {
			getApplicationStatistic().fireStopped(APP_PSEUDONAME, APP_PSEUDOTAG);
		}
	}
	//-------------------------------------------------------------------------
	public interface ILifeCycleStatistic{
		/**Started object with the name 'name' and the number 'longTag'*/
		public void fireStarted(final String name, final Long longTag);
		/**Stopped object with the name 'name' and the number 'longTag'*/
		public void fireStopped(final String name, final Long longTag);
	}
	
//=============================================================================
	
	public static final IUpdateNotification UpdateNotification;
	//-------------------------------------------------------------------------
	/**
	 * Central Interface (facade) to the client side update notification system
	 * both for updates and receiving infos about updates.
	 * @author ws
	 */
	public interface IUpdateNotification {
		/**A new update is published*/
		public void fireNewUpdate(final String name);
		
		/**register for update notification*/
		public void   register(final IUpdateCallBack cb);
		
		/**deregister for update notification*/
		public void deregister(final IUpdateCallBack cb);

		public boolean isRegistered(final Object cb);
	}	
	//-------------------------------------------------------------------------
	public interface IUpdateCallBack extends _IMsgListener<String>{
		public void onMessage(final String name);
	}
	
	//=============================================================================

	public static final IExceptionNotification ExceptionNotification = 
		new _ExceptionNotification(AppAdminContext.getInstance().getJMSExcQueueName());
	//-------------------------------------------------------------------------
	/**interface to inform Elke via JMS about exceptions in jddd clients*/
	public interface IExceptionNotification {
		/**
		 * @param sException a stringified version of the exception
		 */
		public void fireNewException(final String sException);
		
	}
}
