package de.desy.mst.libs.pintlk.badgerviewer.network;


/** Definiert die Basiskonfiguration fuer die Tine abhaengigen factories
 * Not thread save
 * @author labudda
 *
 */
public class TineParameter 
extends de.desy.mst.libs.ser.tclientlib.TLinkAddress
implements ITineParameter {
	/**
	 * Ist ein Devicename entsprechend  NullDevice
	 * Ist das entsprechende Device nicht vorhanden/nicht definiert.
	 * Es wird nicht auf '' oder null geprueft. Entsprechende Werte sind Fehler
	 *  und muessen entsprechend behandelt werden
	 */
	private static final String NullDevice = "none"; 
	public TineParameter() {
		super("", "", "", "");
	}

	public TineParameter(String context, String server, String device, String property) {
		super(context, server, device, property);
	}
	public TineParameter(String context, String server, String device) {
		this(context, server, device, "");
	}
	public TineParameter(String context, String server) {
		this(context, server, "", "");
	}
	public TineParameter(String fullPath) {
		super("", "", "", "");	// Just to init super with any Data
		if (fullPath == null) {
			// use default of TLinkAddress
		} else {
			final String components[] = fullPath.split("/");
			if(components.length > 4) this.setProperty(components[4]);
			if(components.length > 3) this.setDevice(components[3]);
			if(components.length > 2) this.setServer(components[2]);
			if(components.length > 1) this.setContext(components[1]);
			if(components.length == 1) this.setContext(components[0]);
		}
	}
	/** Copy contructor
	 * @param address
	 */
	public TineParameter(TineParameter address) {
		super(address);
	}
	
	/** Erzeugt einen TineParameter aus der Adresse
	 * @param linkAddress
	 */
	public TineParameter(de.desy.mst.libs.ser.tclientlib.TLinkAddress linkAddress) {
		super(linkAddress);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.tclientlib.TLinkAddress#getFullPath()
	 * Overide for debug purpose only
	 */
	@Override
	public String getFullPath() {
		final String fullPath = super.getFullPath();
		if (fullPath.length() < 10) {
			// seems to be an error
			throw new AssertionError("Path seems to be to short");
		} else {
			if ("".equals(this.getContext()) || "".equals(this.getServer()) || "".equals(this.getDevice()) || "".equals(this.getProperty())) {
				throw new AssertionError("One of the tine path items is empty. " + fullPath); 
			} else {
				return fullPath;
			}
		}
	}

	/**
	 * 
	 */
	private static final long serialVersionUID = 164538052434205766L;

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.pintlk.picc.mgmnt.ITineParameter#getMode()
	 */
	public int getMode() {
		return de.desy.tine.definitions.TMode.CM_POLL;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.pintlk.picc.mgmnt.ITineParameter#getPollrate()
	 */
	public int getPollrate() {
		return 1000;
	}
	
	/** Erzeugt ein neues Objekt.
	 * Das neue Objekt ist eine kopie dieses Objektes 
	 * exklusive des neu definierten Elementes "Property"
	 * @param property
	 * @return new TineParameter based on this with modified property name
	 */
	public TineParameter deriveProperty(final String property) {
		TineParameter derived = new TineParameter(this);
		derived.setProperty(property);
		return derived;
	}
	/** Erzeugt ein neues Objekt.
	 * Das neue Objekt ist eine kopie dieses Objektes 
	 * exklusive des neu definierten Elementes "Device"
	 * @param device
	 * @return new TineParameter based on this with modified device name 
	 */
	public TineParameter deriveDevice(final String device) {
		TineParameter derived = new TineParameter(this);
		derived.setDevice(device);
		return derived;
	}

	/** Erzeugt ein neues Objekt.
	 * Das neue Objekt ist eine kopie dieses Objektes 
	 * exklusive des neu definierten Elementes "Server"
	 * @param serverName
	 * @return new TineParameter based on this with modified serverName
	 */
	public TineParameter deriveServer(String serverName) {
		TineParameter derived = new TineParameter(this);
		derived.setServer(serverName);
		return derived;
	}

	/**
	 * @return - True, wenn der devicename None ist. 
	 * D.h. hier ist bewusst kein Device definiert.
	 * Diese Konvention stammt aus der Vereinbarung / Dokumentation mit MPS 
	 */
	public boolean isDeviceNone() {
		return TineParameter.NullDevice.equals(this.getDevice());
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.tclientlib.TLinkAddress#isProperAddress()
	 * Im Gegensatz zu supper muss in dieser Applikation auch das device definiert sein
	 */
	@Override
	public boolean isProperAddress() {
		final String device  = this.getDevice(); 
		return super.isProperAddress() && !(device == null || device.isEmpty());
	}
	
}
