package de.desy.mst.libs.ser.tclientlib.writers;
import de.desy.tine.dataUtils.TDataType;
import de.desy.mst.libs.ser.tclientlib.ATineWriter;
import de.desy.mst.libs.ser.tclientlib.TineLinkException;
import de.desy.mst.libs.ser.tclientlib.TLinkAddress;
import de.desy.tine.definitions.TFormat;

/**
 * A boolean writer.
 *
 * The BooleanWriter writes a scalar value of type boolean
 * to a TINE server.
 *
 */
public class BooleanWriter extends ATineWriter {
    
    private static final int ARRAY_SIZE = 1;
    private int[] outputBuffer = new int[ARRAY_SIZE]; // Data buffer for TINE
    
    /**
     * Constructor.
     * @param linkAddress The TINE address
     */
    public BooleanWriter(TLinkAddress linkAddress) {
        super(linkAddress);
    }

    
    /**
     * Constructor.
     * @param fullDevicePath The full TINE path of
     * the property. It must be in the form 
     * <code>
     * /context/server/device/property
     * </code>.
     */
    public BooleanWriter(String fullDevicePath) {
        super(fullDevicePath);
    }
    
    /**
     * Writes a value to the server.
     * If the write operation was not successful,
     * a TineLinkException is thrown.
     * @throws TineLinkException
     */
    public synchronized void setValue(boolean newValue) throws TineLinkException {
        outputBuffer[0] = newValue ? 1 : 0;
        if (writeData(createTDataType()) != 0) {
            throw new TineLinkException(getLinkAddress(), getLinkStatus(), getErrorMessage());
        }
    }
    
    private TDataType createTDataType() {
        TDataType result = null;
        result = new TDataType(outputBuffer);
        result.dFormat = TFormat.CF_BOOLEAN;
        return result;
    }
    
    
}
