/*
 * (c) by DESY 2016 - Group MCS 1, Hamburg, Germany
 * 
 * For more information about the license you should have a look at the
 * following web page:
 * 
 *         http://www.desy.de/legal/license/index_eng.html
 * 
 * If no concrete license can be found at this web page or the connection is
 * broken, then the following license is relevant:
 * 
 * Copyright (C) 2016 DESY.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */
package de.desy.mst.libs.util.helper.ui.version;

import de.desy.mst.libs.util.helper.LogProvider;
import de.desy.mst.libs.util.helper.Version;
import java.awt.Color;
import java.awt.Font;
import java.io.IOException;
import java.io.InputStream;
import java.lang.management.ManagementFactory;
import java.net.URL;
import java.util.Enumeration;
import java.util.jar.Attributes;
import java.util.jar.JarFile;
import java.util.jar.Manifest;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.BorderFactory;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 * Version panel.
 * 
 * Also adds as a first entry the JVM version.
 * 
 * @author <a href="marcus.walla(at)desy.de">Marcus Walla</a>
 * @version $Id: VersionPanel.java 73822 2016-11-15 20:05:55Z walla $
 */
public class VersionPanel extends javax.swing.JPanel
{
    // --- CLASS FIELDS --------------------------------------------------------
    private final static Logger log = LogProvider.getLogger(VersionPanel.class, Level.INFO, true, LogProvider.ELoggingProperty.MCS_LOGGING);
    private final static boolean IS_DEBUG = false;
    private final static int FONT_SIZE = 12;
    private final static int GAP_SPACE = 20;
    private final static int GAP_BETWEEN_ENTRIES = 4;
    private final static int GAP_BETWEEN_ELEMENTS = 2;

    // --- INSTANCE FIELDS -----------------------------------------------------
    private final JDialog parent;
    
    // --- CONSTRUCTOR ---------------------------------------------------------
    /**
     * Creates a new instance of the version panel.
     * 
     * @param parent parent component.
     * @param icon application icon. Can also be {@code null}.
     * @param applicationVersion application version.
     * @param libraryVersions library versions.
     */
    public VersionPanel(final JDialog parent, final Icon icon, final Version applicationVersion, final Version... libraryVersions)
    {
        verify(parent, applicationVersion);
        initComponents();
        this.parent = parent;
        initVersions(icon, null, applicationVersion, libraryVersions);
    }
    
    /**
     * Creates a new instance of the version panel.
     * 
     * @param parent Parent component.
     * @param icon Application icon. Can also be {@code null}.
     * @param tineVersion TINE version string
     * @param applicationVersion Application version.
     * @param libraryVersions Library versions.
     */
    public VersionPanel(final JDialog parent, final Icon icon, final String tineVersion, final Version applicationVersion, final Version... libraryVersions)
    {
        verify(parent, applicationVersion);
        initComponents();
        this.parent = parent;
        initVersions(icon, tineVersion, applicationVersion, libraryVersions);
    }

    // --- INSTANCE METHODS ----------------------------------------------------
 
    private void verify(final JDialog parent, final Version applicationVersion)
    {
        if (parent == null)
            throw new NullPointerException("parent instance must be set");
        if (applicationVersion == null)
            throw new NullPointerException("application version instance must be set");
    }
    
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents()
    {

        lblHeader = new javax.swing.JLabel();
        btnOk = new javax.swing.JButton();
        descissionTab = new javax.swing.JTabbedPane();

        lblHeader.setFont(new java.awt.Font("Dialog", 1, 18)); // NOI18N
        lblHeader.setForeground(new java.awt.Color(110, 170, 240));
        lblHeader.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        lblHeader.setText("Information about the application");

        btnOk.setText("OK");
        btnOk.setToolTipText("");
        btnOk.setActionCommand("CMD_OK");
        btnOk.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                btnOkActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(descissionTab)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(0, 0, Short.MAX_VALUE)
                        .addComponent(btnOk, javax.swing.GroupLayout.PREFERRED_SIZE, 72, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(lblHeader, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 391, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(lblHeader)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(descissionTab, javax.swing.GroupLayout.DEFAULT_SIZE, 234, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(btnOk)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    private void btnOkActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_btnOkActionPerformed
    {//GEN-HEADEREND:event_btnOkActionPerformed
        this.parent.setVisible(false);
    }//GEN-LAST:event_btnOkActionPerformed

    private void initVersions(final Icon icon, final String tineVersion, final Version applicationVersion, final Version... libraryVersions)
    {
        this.descissionTab.addTab("System", null, new VersionSystemPanel(applicationVersion), "Select for system information.");
        this.descissionTab.addTab("Developers", null, createDevelopers(icon, applicationVersion), "Select to see the responsible developers.");
        this.descissionTab.addTab("Versions", null, createVersions(applicationVersion, tineVersion, libraryVersions), "Select to see the used versions.");
    }

    private JComponent createDevelopers(final Icon icon, final Version applicationVersion)
    {
        final JPanel pnlDevelopers = new JPanel();
        final GroupLayout layoutDevelopers = new GroupLayout(pnlDevelopers);
        pnlDevelopers.setLayout(layoutDevelopers);
        layoutDevelopers.setAutoCreateContainerGaps(true);
        layoutDevelopers.setAutoCreateGaps(true);
        GroupLayout.Group horizontalDevelopers = layoutDevelopers.createParallelGroup();
        GroupLayout.Group verticalDevelopers = layoutDevelopers.createSequentialGroup();
        layoutDevelopers.setHorizontalGroup(horizontalDevelopers);
        layoutDevelopers.setVerticalGroup(verticalDevelopers);
        
        if (icon != null)
            this.lblHeader.setIcon(icon);
        
        applicationVersion.developers().stream().forEach((developer) -> {
            addDeveloper(horizontalDevelopers, verticalDevelopers, developer);
        });
        
        return new JScrollPane(pnlDevelopers);
   }

    private JComponent createVersions(final Version applicationVersion, final String tineVersion, final Version... libraryVersions)
    {
        final JPanel pnlVersions = new JPanel();
        final GroupLayout layoutVersions = new GroupLayout(pnlVersions);
        pnlVersions.setLayout(layoutVersions);
        layoutVersions.setAutoCreateContainerGaps(true);
        layoutVersions.setAutoCreateGaps(true);
        GroupLayout.Group horizontalVersions = layoutVersions.createParallelGroup();
        GroupLayout.Group verticalVersions = layoutVersions.createSequentialGroup();
        layoutVersions.setHorizontalGroup(horizontalVersions);
        layoutVersions.setVerticalGroup(verticalVersions);
        
        addJavaVersion(horizontalVersions, verticalVersions);
        verticalVersions.addGap(5);
        
        addVersion(horizontalVersions, verticalVersions, applicationVersion);
        verticalVersions.addGap(5);
        
        for(final Version libraryVersion: libraryVersions)
            addVersion(horizontalVersions, verticalVersions, libraryVersion);
        addAcopBeansVersion(horizontalVersions, verticalVersions);
        addTineVersion(horizontalVersions, verticalVersions, tineVersion);
        
        return new JScrollPane(pnlVersions);
    }
    
    void addDeveloper(final GroupLayout.Group horizontal, final GroupLayout.Group vertical, final Version.Developer developer)
    {
        final JPanel pnlDeveloper = new JPanel();
        if (IS_DEBUG)
            pnlDeveloper.setBorder(BorderFactory.createLineBorder(Color.RED));
        final GroupLayout tmpLayout = new GroupLayout(pnlDeveloper);
        pnlDeveloper.setLayout(tmpLayout);

        final JLabel lblTmpHeader = new JLabel(developer.fullName());
        lblTmpHeader.setFont(new Font(null, Font.BOLD, FONT_SIZE));

        final JLabel lblEMail = new JLabel("(" + developer.email() +")");
        lblEMail.setFont(new Font(null, Font.PLAIN, FONT_SIZE));

        final JLabel lblLocation = new JLabel("phone: " + developer.phone() + ", group: " + developer.group() + ", building: " + developer.room());
        lblLocation.setFont(new Font(null, Font.PLAIN, FONT_SIZE));

        tmpLayout.setHorizontalGroup(tmpLayout.createParallelGroup(Alignment.LEADING)
                .addGroup(tmpLayout.createSequentialGroup()
                        .addComponent(lblTmpHeader)
                        .addGap(GAP_BETWEEN_ENTRIES)
                        .addComponent(lblEMail)
                )
                .addGroup(tmpLayout.createSequentialGroup()
                        .addGap(GAP_SPACE)
                        .addComponent(lblLocation)
                )
        );
        tmpLayout.setVerticalGroup(tmpLayout.createSequentialGroup()
                .addGroup(tmpLayout.createParallelGroup(Alignment.LEADING)
                        .addComponent(lblTmpHeader)
                        .addComponent(lblEMail)
                )
                .addGap(GAP_BETWEEN_ENTRIES)
                .addGroup(tmpLayout.createParallelGroup()
                        .addComponent(lblLocation)
                )
        );
        
        horizontal.addComponent(pnlDeveloper);
        vertical.addComponent(pnlDeveloper);
        
        vertical.addGap(GAP_BETWEEN_ELEMENTS);
    }
    
    void addVersion(final GroupLayout.Group horizontal, final GroupLayout.Group vertical, final Version version)
    {
        final JPanel pnlVersion = new JPanel();
        if (IS_DEBUG)
            pnlVersion.setBorder(BorderFactory.createLineBorder(Color.RED));
        final GroupLayout tmpLayout = new GroupLayout(pnlVersion);
        pnlVersion.setLayout(tmpLayout);

        final JLabel lblTmpHeader = new JLabel(version.application());
        lblTmpHeader.setFont(new Font(null, Font.BOLD, FONT_SIZE));

        final JLabel lblVersion = new JLabel("version " + version.toString());
        lblVersion.setFont(new Font(null, Font.PLAIN, FONT_SIZE));

        tmpLayout.setHorizontalGroup(tmpLayout.createParallelGroup(Alignment.LEADING)
                .addGroup(tmpLayout.createSequentialGroup()
                        .addComponent(lblTmpHeader)
                )
                .addGroup(tmpLayout.createSequentialGroup()
                        .addGap(GAP_SPACE)
                        .addComponent(lblVersion)
                )
        );
        tmpLayout.setVerticalGroup(tmpLayout.createSequentialGroup()
                .addGroup(tmpLayout.createParallelGroup(Alignment.LEADING)
                        .addComponent(lblTmpHeader)
                )
                .addGap(GAP_BETWEEN_ENTRIES)
                .addGroup(tmpLayout.createParallelGroup()
                        .addComponent(lblVersion)
                )
        );

        horizontal.addComponent(pnlVersion);
        vertical.addComponent(pnlVersion);
        
        vertical.addGap(GAP_BETWEEN_ELEMENTS);
    }

    void addTineVersion(final GroupLayout.Group horizontal, final GroupLayout.Group vertical, final String tineVersion)
    {
        if (tineVersion == null || tineVersion.isEmpty())
            return;

        final JPanel pnlVersion = new JPanel();
        if (IS_DEBUG)
            pnlVersion.setBorder(BorderFactory.createLineBorder(Color.RED));
        final GroupLayout layout = new GroupLayout(pnlVersion);
        pnlVersion.setLayout(layout);

        final JLabel lblTmpHeader = new JLabel("TINE Communication Protocol");
        lblTmpHeader.setFont(new Font(null, Font.BOLD, FONT_SIZE));
        
        final JLabel lblVersion = new JLabel(tineVersion);
        lblVersion.setFont(new Font(null, Font.PLAIN, FONT_SIZE));

        layout.setHorizontalGroup(layout.createParallelGroup(Alignment.LEADING)
                .addGroup(layout.createSequentialGroup()
                        .addComponent(lblTmpHeader)
                )
                .addGroup(layout.createSequentialGroup()
                        .addGap(GAP_SPACE)
                        .addComponent(lblVersion)
                )
        );
        layout.setVerticalGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(Alignment.LEADING)
                        .addComponent(lblTmpHeader)
                )
                .addGap(GAP_BETWEEN_ENTRIES)
                .addGroup(layout.createParallelGroup()
                        .addComponent(lblVersion)
                )
        );

        horizontal.addComponent(pnlVersion);
        vertical.addComponent(pnlVersion);
        
        vertical.addGap(GAP_BETWEEN_ELEMENTS);
    }
        
    void addJavaVersion(final GroupLayout.Group horizontal, final GroupLayout.Group vertical)
    {
        final JPanel pnlVersion = new JPanel();
        if (IS_DEBUG)
            pnlVersion.setBorder(BorderFactory.createLineBorder(Color.RED));
        final GroupLayout layout = new GroupLayout(pnlVersion);
        pnlVersion.setLayout(layout);

        final JLabel lblTmpHeader = new JLabel("Java VM " + ManagementFactory.getRuntimeMXBean().getVmName());
        lblTmpHeader.setFont(new Font(null, Font.BOLD, FONT_SIZE));

        // Note: fetching the implementation version from the Java Runtime package
        // which can be different from the used package of this library (for example
        // it can be a different vendor and therefore a version specification,
        // depending on the used manifest file in the build Java archive).
        final JLabel lblVersion = new JLabel("version " + Runtime.class.getPackage().getImplementationVersion() + " (VM: " + ManagementFactory.getRuntimeMXBean().getVmVersion() + "), vendor " + ManagementFactory.getRuntimeMXBean().getSpecVendor());
        lblVersion.setFont(new Font(null, Font.PLAIN, FONT_SIZE));

        layout.setHorizontalGroup(layout.createParallelGroup(Alignment.LEADING)
                .addGroup(layout.createSequentialGroup()
                        .addComponent(lblTmpHeader)
                )
                .addGroup(layout.createSequentialGroup()
                        .addGap(GAP_SPACE)
                        .addComponent(lblVersion)
                )
        );
        layout.setVerticalGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(Alignment.LEADING)
                        .addComponent(lblTmpHeader)
                )
                .addGap(GAP_BETWEEN_ENTRIES)
                .addGroup(layout.createParallelGroup()
                        .addComponent(lblVersion)
                )
        );

        horizontal.addComponent(pnlVersion);
        vertical.addComponent(pnlVersion);
        
        vertical.addGap(GAP_BETWEEN_ELEMENTS);
    }
    
    void addAcopBeansVersion(final GroupLayout.Group horizontal, final GroupLayout.Group vertical)
    {
        try
        {
            final Enumeration<URL> resEnum = Thread.currentThread().getContextClassLoader().getResources(JarFile.MANIFEST_NAME);

            while (resEnum.hasMoreElements()) 
            {
                URL url = resEnum.nextElement();
                if (url.toString().contains("acopbeans.jar")) 
                {
                    try (final InputStream is = url.openStream()) 
                    {
                        Manifest manifest = new Manifest(is);
                        Attributes mainAttribs = manifest.getMainAttributes();
                        String version = mainAttribs.getValue("Build-Version");
                        String time = mainAttribs.getValue("Build-Time");
                        
                        final JPanel pnlVersion = new JPanel();
                        if (IS_DEBUG)
                            pnlVersion.setBorder(BorderFactory.createLineBorder(Color.RED));
                        final GroupLayout layout = new GroupLayout(pnlVersion);
                        pnlVersion.setLayout(layout);

                        final JLabel lblTmpHeader = new JLabel("AcopBeans (CosyLab)");
                        lblTmpHeader.setFont(new Font(null, Font.BOLD, FONT_SIZE));

                        // Note: fetching the implementation version from the Java Runtime package
                        // which can be different from the used package of this library (for example
                        // it can be a different vendor and therefore a version specification,
                        // depending on the used manifest file in the build Java archive).
                        final JLabel lblVersion = new JLabel("version " + version + ", date: " + time);
                        lblVersion.setFont(new Font(null, Font.PLAIN, FONT_SIZE));

                        layout.setHorizontalGroup(layout.createParallelGroup(Alignment.LEADING)
                                .addGroup(layout.createSequentialGroup()
                                        .addComponent(lblTmpHeader)
                                )
                                .addGroup(layout.createSequentialGroup()
                                        .addGap(GAP_SPACE)
                                        .addComponent(lblVersion)
                                )
                        );
                        layout.setVerticalGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(Alignment.LEADING)
                                        .addComponent(lblTmpHeader)
                                )
                                .addGap(GAP_BETWEEN_ENTRIES)
                                .addGroup(layout.createParallelGroup()
                                        .addComponent(lblVersion)
                                )
                        );

                        horizontal.addComponent(pnlVersion);
                        vertical.addComponent(pnlVersion);

                        return;
                    }
                }
            }
        }
        catch(final IOException ex)
        {
            log.log(Level.SEVERE, "unable to load manifest file", ex);
        }
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnOk;
    private javax.swing.JTabbedPane descissionTab;
    private javax.swing.JLabel lblHeader;
    // End of variables declaration//GEN-END:variables
}
